@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -38,42 +38,44 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 static void updateHostNameInfo(HostAddr addr, char* symbolic, int type);
 
 /* #define DNS_DEBUG */
+/* #define MDNS_DEBUG  */
 
 /* **************************************** */
 
 static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDeviceId, int type) {
   HostTraffic *el;
 
-  if(myGlobals.capturePackets != FLAG_NTOPSTATE_RUN) return;
-
+  if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) return;
 
   /* Search the instance and update its name */
-    
+
   if(myGlobals.device[actualDeviceId].virtualDevice) return;
 
-  el = findHostByNumIP(addr, actualDeviceId);
+  for(el=getFirstHost(actualDeviceId); el != NULL; el = getNextHost(actualDeviceId, el)) {
+    if((el->hostNumIpAddress != NULL) && (addrcmp(&el->hostIpAddress, &addr) == 0)) {
+      accessAddrResMutex("updateHostNameInfo");
 
-  accessAddrResMutex("updateHostNameInfo");
+      if(el != NULL) {
+	unsigned short i;
 
-  if(el != NULL) {    
-    int i;
-    
-    if(strlen(symbolic) >= (MAX_LEN_SYM_HOST_NAME-1)) 
-      symbolic[MAX_LEN_SYM_HOST_NAME-2] = '\0';
-    
-    for(i=0; i<strlen(symbolic); i++)
-      if(isupper(symbolic[i])) tolower(symbolic[i]);
+	if(strlen(symbolic) >= (MAX_LEN_SYM_HOST_NAME-1))
+	  symbolic[MAX_LEN_SYM_HOST_NAME-2] = '\0';
 
-    setResolvedName(el, symbolic, type);
+	for(i=0; i<strlen(symbolic); i++)
+	  if(isupper(symbolic[i])) tolower(symbolic[i]);
+
+	setResolvedName(el, symbolic, type);
+      }
+
+      releaseAddrResMutex();
+    }
   }
-
-  releaseAddrResMutex();
 }
 
 /* **************************************** */
 
-static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
-  int i; 
+  static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
+  int i;
 
   for(i=0; i<myGlobals.numDevices; i++)
     updateDeviceHostNameInfo(addr, symbolic, i, type);
@@ -81,8 +83,35 @@ static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
 
 /* ************************************ */
 
-static void resolveAddress(HostAddr *hostAddr,
-			   short keepAddressNumeric) {
+static int validDNSChar(char c) {
+  if((c == '-') || (c == '_') || (c == '.')) return(1);
+  if((c >= '0') && (c <= '9')) return(1);
+  if((c >= 'a') && (c <= 'z')) return(1);
+  if((c >= 'A') && (c <= 'Z')) return(1);
+
+  return(0);
+}
+
+/* ************************************ */
+
+static int validDNSName(char *name) {
+  int i, len;
+
+  if(name == NULL)
+    return(0);
+  else
+    len = strlen(name);
+
+  for(i=0; i<len; i++)
+    if(!validDNSChar(name[i]))
+      return(0);
+
+  return(1);
+}
+
+/* ************************************ */
+
+static void resolveAddress(HostAddr *hostAddr, short keepAddressNumeric) {
   char symAddr[MAX_LEN_SYM_HOST_NAME];
   short symAddrType=FLAG_HOST_SYM_ADDR_TYPE_NONE;
   StoredAddress storedAddress;
@@ -95,7 +124,7 @@ static void resolveAddress(HostAddr *hostAddr,
   datum data_data;
   static int reportedFreaky = FALSE;
 
-  if(myGlobals.capturePackets != FLAG_NTOPSTATE_RUN) return;
+  if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) return;
 
   myGlobals.numResolveAddressCalls++;
 
@@ -105,10 +134,11 @@ static void resolveAddress(HostAddr *hostAddr,
 
   memset(&keyBuf, 0, sizeof(keyBuf));
   memset(&dataBuf, 0, sizeof(dataBuf));
+  memset(&storedAddress, 0, sizeof(storedAddress));
 
   key_data.dptr = _addrtonum(hostAddr, keyBuf, sizeof(keyBuf));
   key_data.dsize = strlen(keyBuf)+1;
-  
+
   if(myGlobals.dnsCacheFile == NULL) {
 #ifdef DNS_DEBUG
     traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Leaving resolveAddress(), dnsCacheFile NULL");
@@ -122,7 +152,7 @@ static void resolveAddress(HostAddr *hostAddr,
   myGlobals.numResolveCacheDBLookups++;
   /* First check whether the address we search for is cached... */
   if((data_data.dptr != NULL) &&
-     (data_data.dsize == (sizeof(StoredAddress)+1)) &&
+     (data_data.dsize == (sizeof(StoredAddress))) &&
      (myGlobals.actTime - ((StoredAddress*)data_data.dptr)->recordCreationTime < CONST_DNSCACHE_LIFETIME) ) {
     StoredAddress *retrievedAddress;
 
@@ -155,7 +185,7 @@ static void resolveAddress(HostAddr *hostAddr,
   } else {
     if(data_data.dptr != NULL) {
 #ifdef GDBM_DEBUG
-      if (data_data.dsize == (sizeof(StoredAddress)+1))
+      if (data_data.dsize == (sizeof(StoredAddress)))
         traceEvent(CONST_TRACE_INFO, "GDBM_DEBUG: Dropped data for %s [wrong data size]", keyBuf);
       else
         traceEvent(CONST_TRACE_INFO, "GDBM_DEBUG: Ignored old record for %s", keyBuf);
@@ -169,10 +199,10 @@ static void resolveAddress(HostAddr *hostAddr,
 
   }
 
-  if((!keepAddressNumeric) && (myGlobals.capturePackets == FLAG_NTOPSTATE_RUN)) {
+  if((!keepAddressNumeric) && (myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN)) {
     char theAddr[17];
     int family, size;
-    
+
 #ifdef HAVE_GETIPNODEBYADDR
     int error_num;
 #endif
@@ -190,16 +220,15 @@ static void resolveAddress(HostAddr *hostAddr,
       FILE* fd;
       char buffer[64];
       int i, len;
-      
+
       if (hostAddr->hostFamily == AF_INET)
 	cwd = "/usr/bin/host";
       else (hostAddr->hostFamily == AF_INET6)
 	cwd = "/usr/bin/host -t aaaa";
-      if(snprintf(buffer, sizeof(buffer),
+      safe_snprintf(__FILE__, __LINE__, buffer, sizeof(buffer),
 		  "%s %s",
-		  cwd,theAddr) < 0)
-	BufferTooShort();
-      
+		  cwd,theAddr);
+
       fd = popen(buffer, "r");
 
       if(fd == NULL) {
@@ -256,7 +285,7 @@ static void resolveAddress(HostAddr *hostAddr,
     traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called getipnodebyaddr(): RC=%d 0x%X [%s]",      error_num,
 	       hp, hp != NULL ? (char*)hp->h_name : "");
 #endif
-    
+
 #else /* default */
 #if defined(HAVE_GETHOSTBYADDR_R) && !defined(LINUX)
     /* Linux seems to ha some problems with gethostbyaddr_r */
@@ -264,14 +293,14 @@ static void resolveAddress(HostAddr *hostAddr,
     {
    struct hostent _hp, *__hp;
    char buffer[512];
-  
-  hp = gethostbyaddr_r((const char*)theAddr, size, 
+
+  hp = gethostbyaddr_r((const char*)theAddr, size,
 			 family,
                          &_hp,
                          buffer, sizeof(buffer),
                          &h_errnop);
 #ifdef DNS_DEBUG
-    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called gethostbyaddr_r(): RC=%d 0x%X [%s]", 
+    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called gethostbyaddr_r(): RC=%d 0x%X [%s]",
                h_errnop,
 	       hp, hp != NULL ? (char*)hp->h_name : "");
 #endif
@@ -284,7 +313,7 @@ static void resolveAddress(HostAddr *hostAddr,
                          &__hp,
                          &h_errnop);
 #ifdef DNS_DEBUG
-    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called gethostbyaddr_r(): RC=%d 0x%X [%s]", 
+    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called gethostbyaddr_r(): RC=%d 0x%X [%s]",
                h_errnop,
 	       hp, hp != NULL ? (char*)hp->h_name : "");
 #endif /* DNS_DEBUG */
@@ -293,20 +322,24 @@ static void resolveAddress(HostAddr *hostAddr,
 #else
     hp = (struct hostent*)gethostbyaddr((char*)theAddr,size,family);
 #ifdef DNS_DEBUG
-    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called gethostbyaddr(): RC=%d 0x%X [%s]", 
+    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Called gethostbyaddr(): RC=%d 0x%X [%s]",
                h_errno,
 	       hp, hp != NULL ? (char*)hp->h_name : "not meaningful, hp is null");
 #endif
 //
 #endif
 #endif
-    
+
     if (
 #ifdef HAVE_NETDB_H
 	(h_errno == NETDB_SUCCESS) &&
 #endif
-	(hp != NULL) && 
+#ifdef WIN32
+	(WSAGetLastError() == 0 /* OK */) &&
+#endif
+	(hp != NULL) &&
         (hp->h_name != NULL) &&
+	validDNSName(hp->h_name) &&
         (strcmp(hp->h_name, addrtostr(hostAddr)) != 0)) {
       char *dotp = (char*)hp->h_name;
 
@@ -317,10 +350,10 @@ static void resolveAddress(HostAddr *hostAddr,
 #endif
       strncpy(dataBuf, dotp, sizeof(dataBuf));
 
-      if(myGlobals.domainName[0] != '\0') {
-	int dataLen = strlen(dataBuf)-strlen(myGlobals.domainName);
+      if(myGlobals.runningPref.domainName[0] != '\0') {
+	int dataLen = strlen(dataBuf)-strlen(myGlobals.runningPref.domainName);
 
-	if((dataLen > 0) && (!strcmp(&dataBuf[dataLen], myGlobals.domainName))) {
+	if((dataLen > 0) && (!strcmp(&dataBuf[dataLen], myGlobals.runningPref.domainName))) {
 	  int foundDot=0;
 
 	  for(i=0; i<dataLen-1; i++)
@@ -402,13 +435,15 @@ static void resolveAddress(HostAddr *hostAddr,
 #endif
     resolvedAddress = _addrtostr(hostAddr, dataBuf, sizeof(dataBuf));
   }
-  
+
 #ifdef HAVE_GETIPNODEBYADDR
   if(hp != NULL)
     freehostent(hp);
 #endif
 
   if (addToCacheFlag == 0) {
+    int len;
+
       if(strlen(resolvedAddress) > MAX_LEN_SYM_HOST_NAME) {
         strncpy(symAddr, resolvedAddress, MAX_LEN_SYM_HOST_NAME-4);
         symAddr[MAX_LEN_SYM_HOST_NAME-1] = '\0';
@@ -422,13 +457,15 @@ static void resolveAddress(HostAddr *hostAddr,
         symAddr[i] = (char)tolower(symAddr[i]);
 
       memset(storedAddress.symAddress, 0, sizeof(storedAddress.symAddress));
-      strcpy(storedAddress.symAddress, symAddr);
+      len = min(sizeof(storedAddress.symAddress)-1, strlen(symAddr));
+      memcpy(storedAddress.symAddress, symAddr, len);
+      storedAddress.symAddress[len] = '\0';
       storedAddress.recordCreationTime = myGlobals.actTime;
       storedAddress.symAddressType = symAddrType;
 
       /* key_data has been set already */
       data_data.dptr = (void*)&storedAddress;
-      data_data.dsize = sizeof(storedAddress)+1;
+      data_data.dsize = sizeof(storedAddress) /* Remember, StoredAddress has 1 byte padding so don't add more here */;
 
       if(updateRecord) {
 	updateHostNameInfo(*hostAddr, symAddr, symAddrType);
@@ -475,15 +512,13 @@ static void resolveAddress(HostAddr *hostAddr,
 
 /* *************************** */
 
-#if defined(CFG_MULTITHREADED) && defined(MAKE_ASYNC_ADDRESS_RESOLUTION)
-
 static void queueAddress(HostAddr elem, int forceResolution) {
   datum key_data, data_data;
   char dataBuf[sizeof(StoredAddress)+4];
   int rc;
 
   if((!forceResolution)
-     && myGlobals.trackOnlyLocalHosts 
+     && myGlobals.runningPref.trackOnlyLocalHosts
      && (!_pseudoLocalAddress(&elem)))
     return;
 
@@ -496,7 +531,7 @@ static void queueAddress(HostAddr elem, int forceResolution) {
 
     if(!shownMsg) {
       shownMsg = 1;
-      traceEvent(CONST_TRACE_WARNING, "Address resolution queue is full [%u slots]", 
+      traceEvent(CONST_TRACE_WARNING, "Address resolution queue is full [%u slots]",
 		 MAX_NUM_QUEUED_ADDRESSES);
       traceEvent(CONST_TRACE_INFO, "Addresses in excess won't be resolved - ntop continues");
     }
@@ -520,8 +555,7 @@ static void queueAddress(HostAddr elem, int forceResolution) {
   }
 #endif
 
-  if(snprintf(dataBuf, sizeof(dataBuf), "%s", addrtostr(&elem)) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, dataBuf, sizeof(dataBuf), "%s", addrtostr(&elem));
   data_data.dptr = dataBuf;
   data_data.dsize = strlen(dataBuf)+1;
 
@@ -531,7 +565,7 @@ static void queueAddress(HostAddr elem, int forceResolution) {
     myGlobals.addressQueuedCurrent++, myGlobals.addressQueuedCount++;
     if (myGlobals.addressQueuedCurrent > myGlobals.addressQueuedMax)
       myGlobals.addressQueuedMax = myGlobals.addressQueuedCurrent;
-    
+
 #ifdef DNS_DEBUG
     traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Queued address '%s' [addr queue=%d/max=%d]",
 	       dataBuf, myGlobals.addressQueuedCurrent, myGlobals.addressQueuedMax);
@@ -551,11 +585,7 @@ static void queueAddress(HostAddr elem, int forceResolution) {
     }
   }
 
-#ifdef MAKE_WITH_SEMAPHORES
-  incrementSem(&myGlobals.queueAddressSem);
-#else
   signalCondvar(&myGlobals.queueAddressCondvar);
-#endif
 }
 
 /* ************************************ */
@@ -566,24 +596,23 @@ void cleanupAddressQueue(void) {
 
 /* ************************************ */
 
-#ifdef CFG_MULTITHREADED
+void* dequeueAddress(void *_i) {
+  int dqaIndex = (int)_i;
 
-void* dequeueAddress(void* notUsed _UNUSED_) {
   HostAddr addr;
   datum key_data, data_data;
 
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT: Address resolution thread running...");
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread running [p%d]",
+             pthread_self(), dqaIndex+1, getpid());
 
-  while(myGlobals.capturePackets == FLAG_NTOPSTATE_RUN) {
+  while(myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN) {
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "DEBUG: Waiting for address to resolve...");
 #endif
 
-#ifdef MAKE_WITH_SEMAPHORES
-    waitSem(&myGlobals.queueAddressSem);
-#else
     waitCondvar(&myGlobals.queueAddressCondvar);
-#endif
+
+    if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
 
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "DEBUG: Address resolution started...");
@@ -593,7 +622,10 @@ void* dequeueAddress(void* notUsed _UNUSED_) {
 
     while(data_data.dptr != NULL) {
       int size = data_data.dsize;
-      if(myGlobals.capturePackets != FLAG_NTOPSTATE_RUN) return(NULL);
+
+      if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN)
+        break;
+
       if (size == sizeof(struct in_addr)) {
 	/*addrput(AF_INET, &addr, data_data.dptr);*/
 	addr.hostFamily = AF_INET;
@@ -606,8 +638,6 @@ void* dequeueAddress(void* notUsed _UNUSED_) {
       }
 #endif
 
-      HEARTBEAT(2, "dequeueAddress()", NULL);
-
 #ifdef DNS_DEBUG
       traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Dequeued address... [%s][key=%s] (#addr=%d)",
 		 addrtostr(&addr), key_data.dptr == NULL ? "<>" : key_data.dptr,
@@ -628,28 +658,29 @@ void* dequeueAddress(void* notUsed _UNUSED_) {
     }
   } /* endless loop */
 
-  traceEvent(CONST_TRACE_WARNING, "THREADMGMT: Address resolution thread terminated...");
-  return(NULL); /* NOTREACHED */
-}
+  myGlobals.dequeueAddressThreadId[dqaIndex] = 0;
 
-#endif /* defined(CFG_MULTITHREADED) && defined(MAKE_ASYNC_ADDRESS_RESOLUTION) */
-#endif
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread terminated [p%d]",
+             pthread_self(), dqaIndex+1, getpid());
+
+  return(NULL);
+}
 
 #ifdef INET6
 
 /* ************************************ */
 
-char* _intop(struct in6_addr *addr, char *buf, u_short buflen) {  
-  return (char *)inet_ntop(AF_INET6, addr, buf, buflen); 
+char* _intop(struct in6_addr *addr, char *buf, u_short buflen) {
+  return (char *)inet_ntop(AF_INET6, addr, buf, buflen);
 }
 
 /* ************************************ */
 
 char* intop(struct in6_addr *addr) {
   static char  ntop_buf[INET6_ADDRSTRLEN+1];
-  
+
   memset(ntop_buf, 0, INET6_ADDRSTRLEN);
-  return (char *)_intop(addr, ntop_buf,sizeof(ntop_buf)); 
+  return (char *)_intop(addr, ntop_buf,sizeof(ntop_buf));
 }
 
 #endif /*INET6 */
@@ -737,13 +768,12 @@ char * _addrtonum(HostAddr *addr, char* buf, u_short bufLen) {
 
   switch(addr->hostFamily) {
   case AF_INET:
-    if(snprintf(buf, bufLen, "%u", addr->Ip4Address.s_addr) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, bufLen, "%u", addr->Ip4Address.s_addr);
     break;
 #ifdef INET6
   case AF_INET6:
     if(_intop(&addr->Ip6Address, buf, bufLen) == NULL)
-      BufferTooShort();
+      BufferTooSmall(buf, bufLen);
     break;
 #endif
   default:
@@ -771,32 +801,30 @@ int fetchAddressFromCache(HostAddr hostIpAddress, char *buffer, int *type) {
     *type = FLAG_HOST_SYM_ADDR_TYPE_IP;
     return(0);
   }
-     
+
   key_data.dptr = _addrtonum(&hostIpAddress,keyBuf, sizeof(keyBuf));
   key_data.dsize = strlen(key_data.dptr)+1;
-  
+
   if(myGlobals.dnsCacheFile == NULL) return(0); /* ntop is quitting... */
-  
+
   data_data = gdbm_fetch(myGlobals.dnsCacheFile, key_data);
-  
-  if((data_data.dptr != NULL) &&
-     (data_data.dsize == (sizeof(StoredAddress)+1)) ) {
+
+  if((data_data.dptr != NULL) && (data_data.dsize == (sizeof(StoredAddress))) ) {
     StoredAddress *retrievedAddress;
-    
+
     retrievedAddress = (StoredAddress*)data_data.dptr;
     *type = retrievedAddress->symAddressType;
 
 #ifdef GDBM_DEBUG
     traceEvent(CONST_TRACE_INFO, "GDBM_DEBUG: gdbm_fetch(..., {%s, %d}) = %s, age %d",
-               key_data.dptr, key_data.dsize, 
+               key_data.dptr, key_data.dsize,
                retrievedAddress->symAddress,
                myGlobals.actTime - retrievedAddress->recordCreationTime);
 #endif
 
     if (myGlobals.actTime - retrievedAddress->recordCreationTime < CONST_DNSCACHE_LIFETIME) {
         myGlobals.numFetchAddressFromCacheCallsOK++;
-        if(snprintf(buffer, MAX_LEN_SYM_HOST_NAME, "%s", retrievedAddress->symAddress) < 0)
-          BufferTooShort();
+        safe_snprintf(__FILE__, __LINE__, buffer, MAX_LEN_SYM_HOST_NAME, "%s", retrievedAddress->symAddress);
     } else {
         myGlobals.numFetchAddressFromCacheCallsSTALE++;
         buffer[0] = '\0';
@@ -842,11 +870,7 @@ void ipaddr2str(HostAddr hostIpAddress, int updateHost) {
   if(fetchAddressFromCache(hostIpAddress, buf, &type)  && (buf[0] != '\0')) {
     if(updateHost) updateHostNameInfo(hostIpAddress, buf, type);
   } else {
-#if defined(CFG_MULTITHREADED) && defined(MAKE_ASYNC_ADDRESS_RESOLUTION)
     queueAddress(hostIpAddress, !updateHost);
-#else
-    resolveAddress(&hostIpAddress, 0);
-#endif
   }
 }
 
@@ -1212,6 +1236,57 @@ static int _dn_skipname(const u_char *ptr, const u_char *eom) {
 
 /* ************************************ */
 
+static void msdns_filter_name(char *msg) {
+  int i, j, max = strlen(msg);
+
+  for(i=0, j=0; i<max; i++) {
+#ifdef MDNS_DEBUG
+    if(0) traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: [i=%d][%c][%d]", i, msg[i], msg[i]);
+#endif
+
+    if(msg[i] != '\\') {
+      if(msg[i] > 0) /* FIX: can we do better? */
+	msg[j++] = msg[i];
+    } else {
+      char tmpStr[8], tmpStr2[8];
+      int id;
+
+      if((msg[i+1] >= '0') && (msg[i+1] <= '9')) {
+#ifdef MDNS_DEBUG
+	if(0){
+	  traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: [i=%d][%c][%d]", i+1, msg[i+1], msg[i+1]);
+	  traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: [i=%d][%c][%d]", i+2, msg[i+2], msg[i+2]);
+	  traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: [i=%d][%c][%d]", i+3, msg[i+3], msg[i+3]);
+	}
+#endif
+
+	tmpStr[0] = msg[i+1];
+	tmpStr[1] = msg[i+2];
+	tmpStr[2] = msg[i+3];
+	tmpStr[3] = '\0';
+
+	id = atoi(tmpStr);
+
+	if(id == 128)
+	  msg[j++] = '\'';
+	else if(id < 128) {
+	  snprintf(tmpStr2, sizeof(tmpStr2), "%c", id);
+	  msg[j++] = tmpStr2[0];
+	}
+
+	i += 3;
+      } else {
+	i++;
+	msg[j++] = msg[i];
+      }
+    }
+  }
+
+  msg[j] = '\0';
+}
+
+/* ************************************ */
+
 static char* _res_skip(char *msg,
 		      int numFieldsToSkip,
 		      char *eom) {
@@ -1279,13 +1354,81 @@ static char* _res_skip(char *msg,
   return(cp);
 }
 
+/* ************************************ */
+
+static void setHostName(HostTraffic *srcHost, char *name) {
+  u_short tmpStrLen = min(strlen(name), MAX_LEN_SYM_HOST_NAME);
+  strncpy(srcHost->hostResolvedName, name, tmpStrLen);
+  srcHost->hostResolvedName[tmpStrLen] = '\0';
+}
+
+/* ************************************ */
+
+static void handleMdnsName(HostTraffic *srcHost, u_short sport, u_char *mdns_name) {
+  char *mdnsStrtokState, *name = NULL, *appl = NULL, *proto = NULL, *domain = NULL;
+  char *tmpStr = strdup((char*)mdns_name);
+
+  if(tmpStr != NULL) {
+    msdns_filter_name(tmpStr);
+    /* S's Music._daap._tcp.localcal */
+
+#ifdef MDNS_DEBUG
+    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: (1) [%s]", tmpStr);
+#endif
+
+    name = strtok_r(tmpStr, "._", &mdnsStrtokState);
+    if(name) {
+      appl = strtok_r(NULL, "._", &mdnsStrtokState);
+      if(appl) {
+	proto = strtok_r(NULL, "._", &mdnsStrtokState);
+	if(proto) {
+	  domain = strtok_r(NULL, "._", &mdnsStrtokState);
+	}
+      }
+    }
+
+    if((domain != NULL)
+       && ((!strcmp(domain, "local"))
+	   || (!strcmp(domain, "localafpovertcp"))
+	   )) {
+
+#ifdef MDNS_DEBUG
+    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: (2) [%s] [%s][%s][%s][%s]",
+	       srcHost->hostNumIpAddress,
+	       name, appl, proto, domain);
+#endif
+
+      if((!strcmp(appl, "ipp")) || (!strcmp(appl, "printer"))) {
+	/* Printer */
+	FD_SET(FLAG_HOST_TYPE_PRINTER, &srcHost->flags);
+	setHostName(srcHost, name);
+      } else if(!strcmp(appl, "afpovertcp")) {
+	/* Sharing name under MacOS */
+	setHostName(srcHost, name);
+      } else if(!strcmp(appl, "workstation")) {
+	/* Host name under MacOS */	
+	setHostName(srcHost, strtok(name, "["));
+      } else if(!strcmp(appl, "http")) {
+	/* HTTP server */
+	FD_SET(FLAG_HOST_TYPE_SVC_HTTP, &srcHost->flags);
+      } else if(!strcmp(appl, "daap")) {
+	/* Digital Audio Access Protocol (daap.sourceforge.net) */
+	updateHostUsers(name, BITFLAG_DAAP_USER, srcHost);
+      }
+    }
+
+     free(tmpStr);
+  }
+}
+
 /* ************************************ */
 /*
    This function needs to be rewritten from scratch
    as it does not check boundaries (see ** below)
 */
 
-u_int16_t handleDNSpacket(const u_char *ipPtr,
+u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
+			  const u_char *ipPtr,
 			  DNSHostInfo *hostPtr,
 			  short length,
 			  short *isRequest,
@@ -1364,7 +1507,7 @@ u_int16_t handleDNSpacket(const u_char *ipPtr,
   }
 
   bp	   = hostbuf;
-  buflen = sizeof(hostbuf);
+  buflen   = sizeof(hostbuf);
   cp	   = (u_char *) &answer+HFIXEDSZ;
 
   /* Process first question section. */
@@ -1430,33 +1573,51 @@ u_int16_t handleDNSpacket(const u_char *ipPtr,
        * E.g. : 89.10.67.213.in-addr.arpa
        *
        */
-      char *a, *b, *c, *d, dnsBuf[48], *strtokState;
+      char *a, *b, *c, *d, dnsBuf[128], *strtokState;
       unsigned long theDNSaddr;
 
-      len = strlen(bp); if(len >= (sizeof(dnsBuf)-1)) len = sizeof(dnsBuf)-2;
-      xstrncpy(dnsBuf, bp, len);
+      len = strlen((char*)bp);
 
-      d = strtok_r(dnsBuf, ".", &strtokState);
-      c = strtok_r(NULL, ".", &strtokState);
-      b = strtok_r(NULL, ".", &strtokState);
-      a = strtok_r(NULL, ".", &strtokState);
-
-      if(a && b && c && d) {
-	theDNSaddr = htonl(atoi(a)*(256*256*256)+atoi(b)*(256*256)+atoi(c)*256+atoi(d));
-	if(addr_list_idx >= MAX_ADDRESSES) break; /* Further check */
-	memcpy(&addr_list[addr_list_idx++], (char*)&theDNSaddr, sizeof(char*));
-	hostPtr->addrLen = INADDRSZ;
-	hostPtr->addrList[0] = theDNSaddr;
+      if(bp[0] == '_') {
+	/* Multicast DNS */
 
 	n = (short)dn_expand_(answer.qb2, eom, cp, (char *)bp, buflen);
 	if (n < 0) {
 	  cp += n;
 	  continue;
 	}
+
 	cp += n;
-	len = strlen((char *)bp) + 1;
-	memcpy(hostPtr->name, bp, len);
+
+	handleMdnsName(srcHost, sport, bp);
 	haveAnswer = TRUE;
+	continue;
+      } else {
+	if(len >= (sizeof(dnsBuf)-1)) len = sizeof(dnsBuf)-2;
+	xstrncpy(dnsBuf, (char*)bp, len);
+
+	d = strtok_r(dnsBuf, ".", &strtokState);
+	c = strtok_r(NULL, ".", &strtokState);
+	b = strtok_r(NULL, ".", &strtokState);
+	a = strtok_r(NULL, ".", &strtokState);
+
+	if(a && b && c && d) {
+	  theDNSaddr = htonl(atoi(a)*(256*256*256)+atoi(b)*(256*256)+atoi(c)*256+atoi(d));
+	  if(addr_list_idx >= MAX_ADDRESSES) break; /* Further check */
+	  memcpy(&addr_list[addr_list_idx++], (char*)&theDNSaddr, sizeof(char*));
+	  hostPtr->addrLen = INADDRSZ;
+	  hostPtr->addrList[0] = theDNSaddr;
+
+	  n = (short)dn_expand_(answer.qb2, eom, cp, (char *)bp, buflen);
+	  if (n < 0) {
+	    cp += n;
+	    continue;
+	  }
+	  cp += n;
+	  len = strlen((char *)bp) + 1;
+	  memcpy(hostPtr->name, bp, len);
+	  haveAnswer = TRUE;
+	}
       }
       break;
     } else if (type != T_A) {
@@ -1519,28 +1680,30 @@ u_int16_t handleDNSpacket(const u_char *ipPtr,
   }
 
   if ((queryType == T_A || queryType == T_PTR) && haveAnswer) {
-
-    /*
-     *  Go through the alias and address lists and return them
-     *  in the hostPtr variable.
-     */
-
-    if (numAliases > 0) {
-      for (i = 0; i < numAliases; i++) {
-	if(host_aliases[i] != NULL)
-	  memcpy(hostPtr->aliases[i], host_aliases[i], host_aliases_len[i]);
-	else break;
+    if(sport == 53 /* DNS */) {
+      /*
+       *  Go through the alias and address lists and return them
+       *  in the hostPtr variable.
+       */
+      if (numAliases > 0) {
+	for (i = 0; i < numAliases; i++) {
+	  if(host_aliases[i] != NULL)
+	    memcpy(hostPtr->aliases[i], host_aliases[i], host_aliases_len[i]);
+	  else break;
+	}
+	hostPtr->aliases[i][0] = '\0';
       }
-      hostPtr->aliases[i][0] = '\0';
-    }
-    if (numAddresses > 0) {
-      for (i = 0; i < numAddresses; i++) {
-	if(addr_list[i] != NULL)
-	  memcpy(&hostPtr->addrList[i], addr_list[i], hostPtr->addrLen);
-	else break;
+
+      if (numAddresses > 0) {
+	for (i = 0; i < numAddresses; i++) {
+	  if(addr_list[i] != NULL)
+	    memcpy(&hostPtr->addrList[i], addr_list[i], hostPtr->addrLen);
+	  else break;
+	}
+	hostPtr->addrList[i] = 0;
       }
-      hostPtr->addrList[i] = 0;
     }
+
     return(transactionId);
   }
 
@@ -1568,6 +1731,10 @@ u_int16_t handleDNSpacket(const u_char *ipPtr,
     if (n < 0) {
       return(transactionId);
     }
+
+    if(sport == 5353 /* mDNS */)
+      handleMdnsName(srcHost, sport, bp);
+
     cp += n;
     len = strlen((char *)bp) + 1;
 
@@ -1594,7 +1761,7 @@ u_int16_t handleDNSpacket(const u_char *ipPtr,
 void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
   HostTraffic *el;
 
-  for(el=getFirstHost(actualDeviceId); 
+  for(el=getFirstHost(actualDeviceId);
       el != NULL; el = getNextHost(actualDeviceId, el)) {
     if((!addrnull(&el->hostIpAddress))
        && (addrcmp(&el->hostIpAddress,&hostToCheck->hostIpAddress) == 0)) {
@@ -1604,7 +1771,7 @@ void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
 	FD_SET(FLAG_HOST_DUPLICATED_MAC, &hostToCheck->flags);
 	FD_SET(FLAG_HOST_DUPLICATED_MAC, &el->flags);
 
-	if(myGlobals.enableSuspiciousPacketDump) {
+	if(myGlobals.runningPref.enableSuspiciousPacketDump) {
 	  traceEvent(CONST_TRACE_WARNING,
 		     "Two MAC addresses found for the same IP address %s: [%s/%s] (spoofing detected?)",
 		     el->hostNumIpAddress, hostToCheck->ethAddressString, el->ethAddressString);
