@@ -113,29 +113,24 @@ static void initWriteKey(FILE *fDescr, int lang, char *indent,
 
   switch(lang) {
   case FLAG_PERL_LANGUAGE:
-    if(snprintf(buf, sizeof(buf), "%s'%s' => {\n",indent, keyName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s' => {\n",indent, keyName);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_PHP_LANGUAGE:
-    if(snprintf(buf, sizeof(buf), "%s'%s' => array(\n",indent, keyName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s' => array(\n",indent, keyName);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_PYTHON_LANGUAGE:
-    if(snprintf(buf, sizeof(buf), "%s'%s': {\n",indent, keyName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s': {\n",indent, keyName);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_XML_LANGUAGE:
-    if(snprintf(buf, sizeof(buf), "%s<%s>\n", indent, keyName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s<%s>\n", indent, keyName);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_NO_LANGUAGE:
-    if(snprintf(buf, sizeof(buf), "%s|",
-		numEntriesSent == 0 ? "key" : keyName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s|",
+		numEntriesSent == 0 ? "key" : keyName);
     sendEmitterString(fDescr, buf);
     break ;
   }
@@ -156,23 +151,19 @@ static void endWriteKey(FILE *fDescr, int lang, char *indent, char *keyName, cha
   validateString(keyName);
   switch(lang) {
   case FLAG_PERL_LANGUAGE:
-    if(snprintf(buf, sizeof(buf),"%s}%c\n",indent,last) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"%s}%c\n",indent,last);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_PHP_LANGUAGE:
-    if(snprintf(buf, sizeof(buf),"%s)%c\n",indent,last) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"%s)%c\n",indent,last);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_XML_LANGUAGE:
-    if(snprintf(buf, sizeof(buf), "%s</%s>\n",indent, keyName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s</%s>\n",indent, keyName);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_PYTHON_LANGUAGE:
-    if(snprintf(buf, sizeof(buf),"%s}%c\n",indent,last) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"%s}%c\n",indent,last);
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_NO_LANGUAGE:
@@ -183,9 +174,29 @@ static void endWriteKey(FILE *fDescr, int lang, char *indent, char *keyName, cha
 
 /* *************************** */
 
+static char* sanitize(char *value, char *buf, int buf_len) {
+  int i, j = 0;
+
+  for(i=0, j; (i<strlen(value)) && (i<buf_len); i++) {
+    switch(value[i]) {
+    case '\'':
+    case '\"':
+      break;
+    default:
+      buf[j++] = value[i];
+      break;
+    }
+  }
+
+  buf[j] = '\0';
+  return(buf);
+}
+
+/* *************************** */
+
 static void wrtStrItm(FILE *fDescr, int lang, char *indent, char *name,
 		      char *value, char last, int numEntriesSent) {
-  char buf[256];
+  char buf[256], buf1[256];
 
   validateString(name);
 
@@ -196,33 +207,37 @@ static void wrtStrItm(FILE *fDescr, int lang, char *indent, char *name,
        the pointer is not null, but the string is empty.
        In that case, don't create the key in the array.
     */
-    if((value != NULL) && (value[0] != '\0'))
-      {
-	if(snprintf(buf, sizeof(buf), "%s'%s' => '%s'%c\n", indent,name,value,last) < 0)
-	  BufferTooShort();  sendEmitterString(fDescr, buf);
+    if((value != NULL) && (value[0] != '\0')) {
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s' => '%s'%c\n",
+		      indent, name, sanitize(value, buf1, sizeof(buf1)), last);
+	sendEmitterString(fDescr, buf);
       }
     break ;
   case FLAG_XML_LANGUAGE:
     if((value != NULL) && (value[0] != '\0'))
       {
-	if(snprintf(buf, sizeof(buf), "%s<%s>%s</%s>\n", indent, name, value, name) < 0)
-	  BufferTooShort();  sendEmitterString(fDescr, buf);
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s<%s>%s</%s>\n", 
+		      indent, name, sanitize(value, buf1, sizeof(buf1)), name);
+	sendEmitterString(fDescr, buf);
       }
     break ;
   case FLAG_PYTHON_LANGUAGE:
     if((value != NULL) && (value[0] != '\0'))
       {
-	if(snprintf(buf, sizeof(buf), "%s'%s': '%s'%c\n", indent,name,value,last) < 0)
- 	  BufferTooShort();  sendEmitterString(fDescr, buf);
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s': '%s'%c\n", 
+		      indent, name, sanitize(value, buf1, sizeof(buf1)), last);
+ 	sendEmitterString(fDescr, buf);
        }
      break ;
   case FLAG_NO_LANGUAGE:
     if(value != NULL) {
-      if(snprintf(buf, sizeof(buf), "%s|", numEntriesSent == 0 ? name : value) < 0)
-	BufferTooShort();  sendEmitterString(fDescr, buf);
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s|", 
+		    numEntriesSent == 0 ? name : value);
+      sendEmitterString(fDescr, buf);
     } else {
-      if(snprintf(buf, sizeof(buf), "%s|", numEntriesSent == 0 ? name : "") < 0)
-	BufferTooShort();  sendEmitterString(fDescr, buf);
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s|", 
+		    numEntriesSent == 0 ? name : "");
+      sendEmitterString(fDescr, buf);
     }
     break ;
   }
@@ -233,8 +248,7 @@ static void wrtStrItm(FILE *fDescr, int lang, char *indent, char *name,
 static void wrtIntItm(FILE *fDescr, int lang, char *indent, char *name,
 		      int value, char last, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%d",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
   wrtStrItm(fDescr, lang, indent, name, buf, last, numEntriesSent);
 }
 
@@ -243,8 +257,7 @@ static void wrtIntItm(FILE *fDescr, int lang, char *indent, char *name,
 static void wrtIntStrItm(FILE *fDescr, int lang, char *indent,int name,
 			 char *value, char useless, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%d",name) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",name);
   wrtStrItm(fDescr, lang, indent, buf, value, ',', numEntriesSent);
 }
 
@@ -253,8 +266,7 @@ static void wrtIntStrItm(FILE *fDescr, int lang, char *indent,int name,
 static void wrtUintItm(FILE *fDescr, int lang, char *indent, char *name,
 		       unsigned int value, char useless, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%d",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
   wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
@@ -263,8 +275,7 @@ static void wrtUintItm(FILE *fDescr, int lang, char *indent, char *name,
 static void wrtUcharItm(FILE *fDescr, int lang, char *indent, char *name,
 			u_char value, char useless, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%d",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
   wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
@@ -273,8 +284,7 @@ static void wrtUcharItm(FILE *fDescr, int lang, char *indent, char *name,
 static void wrtFloatItm(FILE *fDescr, int lang, char *indent, char *name,
 			float value, char last, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%0.2f",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%0.2f",value);
   wrtStrItm(fDescr, lang, indent, name, buf, last, numEntriesSent);
 }
 
@@ -283,8 +293,7 @@ static void wrtFloatItm(FILE *fDescr, int lang, char *indent, char *name,
 static void wrtIntFloatItm(FILE *fDescr, int lang, char *indent, int name,
 			   float value, char last, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%d", name) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d", name);
   wrtFloatItm(fDescr, lang, indent, (lang == FLAG_XML_LANGUAGE) ? "number" : buf,
 	      value, last, numEntriesSent);
 }
@@ -295,8 +304,7 @@ static void wrtUlongItm(FILE *fDescr, int lang, char *indent, char *name,
 			unsigned long value, char useless, int numEntriesSent) {
   char buf[80];
 
-  if(snprintf(buf, sizeof(buf), "%lu",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%lu",value);
   wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
@@ -306,8 +314,7 @@ static void wrtLlongItm(FILE *fDescr, int lang, char* indent, char* name,
 			TrafficCounter value, char last, int numEntriesSent) {
   char buf[80];
 
-  if(snprintf(buf, sizeof(buf),  "%lu", (long unsigned int)value.value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),  "%lu", (long unsigned int)value.value);
   wrtStrItm(fDescr, lang, indent, name, buf, last, numEntriesSent);
 }
 
@@ -316,8 +323,7 @@ static void wrtLlongItm(FILE *fDescr, int lang, char* indent, char* name,
 static void wrtTime_tItm(FILE *fDescr, int lang, char *indent, char *name,
 			 time_t value, char useless, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%ld",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%ld",value);
   wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
@@ -326,8 +332,7 @@ static void wrtTime_tItm(FILE *fDescr, int lang, char *indent, char *name,
 static void wrtUshortItm(FILE *fDescr, int lang, char *indent, char *name,
 			 u_short value, char useless, int numEntriesSent) {
   char buf[80];
-  if(snprintf(buf, sizeof(buf), "%d",value) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
   wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
@@ -460,10 +465,9 @@ void dumpNtopTrafficMatrix(FILE *fDescr, char* options, int actualDeviceId) {
 	if(myGlobals.device[myGlobals.actualReportDeviceId].ipTrafficMatrix[idx]->bytesSent.value > 0) {
 	    if(numEntries == 0) initWriteArray(fDescr, lang);
 
-	    if(snprintf(buf, sizeof(buf), "%s_%s",
+	    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s_%s",
 			myGlobals.device[myGlobals.actualReportDeviceId].ipTrafficMatrixHosts[i]->hostNumIpAddress,
-			myGlobals.device[myGlobals.actualReportDeviceId].ipTrafficMatrixHosts[j]->hostNumIpAddress) < 0)
-	      BufferTooShort();
+			myGlobals.device[myGlobals.actualReportDeviceId].ipTrafficMatrixHosts[j]->hostNumIpAddress);
 
 	  REPEAT_MATRIX:
 	    initWriteKey(fDescr, lang,  "", buf, numEntries);
@@ -483,24 +487,20 @@ void dumpNtopTrafficMatrix(FILE *fDescr, char* options, int actualDeviceId) {
 	  }
 	}
       }
-    
-    
+
+
 
   if(numEntries > 0) endWriteArray(fDescr, lang);
 }
 
 /* ********************************** */
 
-static void decrementRefCount(HostTraffic *el) {
+static void updateRefCount(HostTraffic *el, int value) {
   if(el->refCount == 0) return;
 
-#ifdef CFG_MULTITHREADED
-  accessMutex(&myGlobals.hostsHashMutex, "dumpNtopHashes");
-#endif
-  el->refCount--;
-#ifdef CFG_MULTITHREADED
-  releaseMutex(&myGlobals.hostsHashMutex);
-#endif
+  lockHostsHashMutex(el, "decrementRefCount");
+  el->refCount += value;
+  unlockHostsHashMutex(el);
 }
 
 /* ********************************** */
@@ -561,27 +561,21 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 
   initWriteArray(fDescr, lang);
 
-  for(el=getFirstHost(actualDeviceId); 
+  for(el=getFirstHost(actualDeviceId);
       el != NULL; el = getNextHost(actualDeviceId, el)) {
 
-#ifdef CFG_MULTITHREADED
-    accessMutex(&myGlobals.hostsHashMutex, "dumpNtopHashes");
-#endif
-    el->refCount++;
-#ifdef CFG_MULTITHREADED
-    releaseMutex(&myGlobals.hostsHashMutex);
-#endif
+    updateRefCount(el, 1);
 
     strncpy(workSymIpAddress, el->hostResolvedName, MAX_LEN_SYM_HOST_NAME_HTML);
-    if ((angleLocation = strchr(workSymIpAddress, '<')) != NULL) {
+
+    if((angleLocation = strchr(workSymIpAddress, '<')) != NULL)
       angleLocation[0] = '\0';
-    }
 
     if(key[0] != '\0') {
       if(strcmp(el->hostNumIpAddress, key)
 	 && strcmp(el->ethAddressString, key)
 	 && strcmp(workSymIpAddress, key)) {
-	decrementRefCount(el);
+	updateRefCount(el, -1);
 	continue;
       }
     }
@@ -591,12 +585,12 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
       if(localView) {
 	if(((!subnetPseudoLocalHost(el))
 	    && (!multicastHost(el)))) {
-	  decrementRefCount(el);
+	  updateRefCount(el, -1);
 	  continue;
 	}
       }
     } else {
-      if(localView) { decrementRefCount(el); continue; }
+      if(localView) { 	updateRefCount(el, -1); continue; }
       hostKey = el->ethAddressString;
     }
 
@@ -759,6 +753,11 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
     if(checkFilter(filter, "ipBytesRcvd"))
       wrtLlongItm(fDescr, lang, "\t", "ipBytesRcvd", el->ipBytesRcvd, ',', numEntries);
 
+    if(checkFilter(filter, "ipv6Sent"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv6Sent", el->ipv6Sent, ',', numEntries);
+    if(checkFilter(filter, "ipv6Rcvd"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv6Rcvd", el->ipv6Rcvd, ',', numEntries);
+
     ctr.value = el->tcpSentLoc.value+el->tcpSentRem.value;
     if(checkFilter(filter, "tcpBytesSent"))
       wrtLlongItm(fDescr, lang, "\t", "tcpBytesSent", ctr, ',', numEntries);
@@ -813,53 +812,51 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 
       /* ***************************** */
 
-      if(checkFilter(filter, "stpSent"))
-	wrtLlongItm(fDescr, lang, "\t", "stpSent",        el->stpSent, ',', numEntries);
-      if(checkFilter(filter, "stpRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "stpRcvd",    el->stpRcvd, ',', numEntries);
-      if(checkFilter(filter, "ipxSent"))
-	wrtLlongItm(fDescr, lang, "\t", "ipxSent",        el->ipxSent, ',', numEntries);
-      if(checkFilter(filter, "ipxRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "ipxRcvd",    el->ipxRcvd, ',', numEntries);
-      if(checkFilter(filter, "osiSent"))
-	wrtLlongItm(fDescr, lang, "\t", "osiSent",        el->osiSent, ',', numEntries);
-      if(checkFilter(filter, "osiRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "osiRcvd",    el->osiRcvd, ',', numEntries);
-      if(checkFilter(filter, "dlcSent"))
-	wrtLlongItm(fDescr, lang, "\t", "dlcSent",        el->dlcSent, ',', numEntries);
-      if(checkFilter(filter, "dlcRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "dlcRcvd",    el->dlcRcvd, ',', numEntries);
+      if(el->nonIPTraffic != NULL) {
+	if(checkFilter(filter, "stpSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "stpSent", el->nonIPTraffic->stpSent, ',', numEntries);
+	if(checkFilter(filter, "stpRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "stpRcvd", el->nonIPTraffic->stpRcvd, ',', numEntries);
+	if(checkFilter(filter, "ipxSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "ipxSent", el->nonIPTraffic->ipxSent, ',', numEntries);
+	if(checkFilter(filter, "ipxRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "ipxRcvd", el->nonIPTraffic->ipxRcvd, ',', numEntries);
+	if(checkFilter(filter, "osiSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "osiSent", el->nonIPTraffic->osiSent, ',', numEntries);
+	if(checkFilter(filter, "osiRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "osiRcvd", el->nonIPTraffic->osiRcvd, ',', numEntries);
+	if(checkFilter(filter, "dlcSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "dlcSent", el->nonIPTraffic->dlcSent, ',', numEntries);
+	if(checkFilter(filter, "dlcRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "dlcRcvd", el->nonIPTraffic->dlcRcvd, ',', numEntries);
 
-      if(checkFilter(filter, "arp_rarpSent"))
-	wrtLlongItm(fDescr, lang, "\t", "arp_rarpSent",   el->arp_rarpSent, ',', numEntries);
-      if(checkFilter(filter, "arp_rarpRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "arp_rarpRcvd", el->arp_rarpRcvd, ',', numEntries);
-      if(checkFilter(filter, "arpReqPktsSent"))
-	wrtLlongItm(fDescr, lang, "\t", "arpReqPktsSent", el->arpReqPktsSent, ',', numEntries);
-      if(checkFilter(filter, "arpReplyPktsSent"))
-	wrtLlongItm(fDescr, lang, "\t", "arpReplyPktsSent", el->arpReplyPktsSent, ',', numEntries);
-      if(checkFilter(filter, "arpReplyPktsRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "arpReplyPktsRcvd", el->arpReplyPktsRcvd, ',', numEntries);
-      if(checkFilter(filter, "decnetSent"))
-	wrtLlongItm(fDescr, lang, "\t", "decnetSent",     el->decnetSent, ',', numEntries);
-      if(checkFilter(filter, "decnetRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "decnetRcvd", el->decnetRcvd, ',', numEntries);
-      if(checkFilter(filter, "appletalkSent"))
-	wrtLlongItm(fDescr, lang, "\t", "appletalkSent",  el->appletalkSent, ',', numEntries);
-      if(checkFilter(filter, "appletalkRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "appletalkRcvd",el->appletalkRcvd, ',', numEntries);
-      if(checkFilter(filter, "netbiosSent"))
-	wrtLlongItm(fDescr, lang, "\t", "netbiosSent",    el->netbiosSent, ',', numEntries);
-      if(checkFilter(filter, "netbiosRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "netbiosRcvd", el->netbiosRcvd, ',', numEntries);
-      if(checkFilter(filter, "ipv6Sent"))
-	wrtLlongItm(fDescr, lang, "\t", "ipv6Sent",        el->ipv6Sent, ',', numEntries);
-      if(checkFilter(filter, "ipv6Rcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "ipv6Rcvd",    el->ipv6Rcvd, ',', numEntries);
-      if(checkFilter(filter, "otherSent"))
-	wrtLlongItm(fDescr, lang, "\t", "otherSent",      el->otherSent, ',', numEntries);
-      if(checkFilter(filter, "otherRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "otherRcvd",  el->otherRcvd, ',', numEntries);
+	if(checkFilter(filter, "arp_rarpSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "arp_rarpSent",el->nonIPTraffic->arp_rarpSent, ',', numEntries);
+	if(checkFilter(filter, "arp_rarpRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "arp_rarpRcvd", el->nonIPTraffic->arp_rarpRcvd, ',', numEntries);
+	if(checkFilter(filter, "arpReqPktsSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "arpReqPktsSent", el->nonIPTraffic->arpReqPktsSent, ',', numEntries);
+	if(checkFilter(filter, "arpReplyPktsSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "arpReplyPktsSent", el->nonIPTraffic->arpReplyPktsSent, ',', numEntries);
+	if(checkFilter(filter, "arpReplyPktsRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "arpReplyPktsRcvd", el->nonIPTraffic->arpReplyPktsRcvd, ',', numEntries);
+	if(checkFilter(filter, "decnetSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "decnetSent", el->nonIPTraffic->decnetSent, ',', numEntries);
+	if(checkFilter(filter, "decnetRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "decnetRcvd", el->nonIPTraffic->decnetRcvd, ',', numEntries);
+	if(checkFilter(filter, "appletalkSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "appletalkSent", el->nonIPTraffic->appletalkSent, ',', numEntries);
+	if(checkFilter(filter, "appletalkRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "appletalkRcvd", el->nonIPTraffic->appletalkRcvd, ',', numEntries);
+	if(checkFilter(filter, "netbiosSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "netbiosSent", el->nonIPTraffic->netbiosSent, ',', numEntries);
+	if(checkFilter(filter, "netbiosRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "netbiosRcvd", el->nonIPTraffic->netbiosRcvd, ',', numEntries);
+	if(checkFilter(filter, "otherSent"))
+	  wrtLlongItm(fDescr, lang, "\t", "otherSent", el->nonIPTraffic->otherSent, ',', numEntries);
+	if(checkFilter(filter, "otherRcvd"))
+	  wrtLlongItm(fDescr, lang, "\t", "otherRcvd", el->nonIPTraffic->otherRcvd, ',', numEntries);
+      }
 
       /* ********************************* */
 
@@ -877,17 +874,20 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
       initWriteKey(fDescr, lang, "\t", "IP", numEntries);
 
       for(j=0; j<myGlobals.numIpProtosToMonitor; j++) {
-	if(j > 0) { endWriteKey(fDescr, lang,"\t\t", lastKey, ','); }
-
-	initWriteKey(fDescr, lang, "\t\t", (lastKey = myGlobals.protoIPTrafficInfos[j]), numEntries);
-	wrtLlongItm(fDescr, lang,"\t\t\t","sentLoc",
-		    el->protoIPTrafficInfos[j].sentLoc, ',', numEntries);
-	wrtLlongItm(fDescr, lang,"\t\t\t","sentRem",
-		    el->protoIPTrafficInfos[j].sentRem, ',', numEntries);
-	wrtLlongItm(fDescr, lang,"\t\t\t","rcvdLoc",
-		    el->protoIPTrafficInfos[j].rcvdLoc, ',', numEntries);
-	wrtLlongItm(fDescr, lang,"\t\t\t","rcvdFromRem",
-		    el->protoIPTrafficInfos[j].rcvdFromRem, ' ', numEntries);
+	if((myGlobals.ipTrafficProtosNames[j] != NULL)
+	   && (el->protoIPTrafficInfos[j] != NULL)) {
+	  if(j > 0) { endWriteKey(fDescr, lang,"\t\t", lastKey, ','); }
+ 
+	  initWriteKey(fDescr, lang, "\t\t", (lastKey = myGlobals.ipTrafficProtosNames[j]), numEntries);
+	  wrtLlongItm(fDescr, lang,"\t\t\t","sentLoc",
+		      el->protoIPTrafficInfos[j]->sentLoc, ',', numEntries);
+	  wrtLlongItm(fDescr, lang,"\t\t\t","sentRem",
+		      el->protoIPTrafficInfos[j]->sentRem, ',', numEntries);
+	  wrtLlongItm(fDescr, lang,"\t\t\t","rcvdLoc",
+		      el->protoIPTrafficInfos[j]->rcvdLoc, ',', numEntries);
+	  wrtLlongItm(fDescr, lang,"\t\t\t","rcvdFromRem",
+		      el->protoIPTrafficInfos[j]->rcvdFromRem, ' ', numEntries);
+	}
       } /* for */
 
       if(lastKey != NULL) { endWriteKey(fDescr, lang,"\t\t", lastKey, ','); }
@@ -1086,7 +1086,7 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 
     if((lang == FLAG_NO_LANGUAGE) && (numEntries == 1)) goto REPEAT_HOSTS;
 
-    decrementRefCount(el);    
+    updateRefCount(el, -1);
   } /* for */
 
   if(numEntries > 0) endWriteKey(fDescr, lang,"", (lang == FLAG_XML_LANGUAGE) ? "host-information" : hostKey, ' ');
@@ -1131,29 +1131,25 @@ void dumpNtopHashIndexes(FILE *fDescr, char* options, int actualDeviceId) {
 
   initWriteArray(fDescr, lang);
 
-  for(el=getFirstHost(actualDeviceId); 
+  for(el=getFirstHost(actualDeviceId);
       el != NULL; el = getNextHost(actualDeviceId, el)) {
-    
-#ifdef CFG_MULTITHREADED
-    accessMutex(&myGlobals.hostsHashMutex, "dumpNtopHashes");
-#endif
+
+    lockHostsHashMutex(el, "dumpNtopHashes");
 
     if(!broadcastHost(el)) {
       char *hostKey;
-      
+
       if(el->hostNumIpAddress[0] != '\0')
 	hostKey = el->hostNumIpAddress;
       else
 	hostKey = el->ethAddressString;
-      
-      wrtIntStrItm(fDescr, lang, "", 0, hostKey,'\n', numEntries);
-      
-      numEntries++;
-    }    
 
-#ifdef CFG_MULTITHREADED
-    releaseMutex(&myGlobals.hostsHashMutex);
-#endif
+      wrtIntStrItm(fDescr, lang, "", 0, hostKey,'\n', numEntries);
+
+      numEntries++;
+    }
+
+    unlockHostsHashMutex(el);
   } /* for */
 
   endWriteArray(fDescr, lang);
@@ -1232,19 +1228,16 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
       if(checkFilter(filter, "fqdn"))
 	wrtStrItm(fDescr, lang, "\t", "fqdn", myGlobals.device[i].fqdn, ',', numEntries);
 
-      if(snprintf(localbuf, sizeof(localbuf), "%s",
-	       _intoa(myGlobals.device[i].network, intoabuf, sizeof(intoabuf))) < 0)
-        BufferTooShort();
+      safe_snprintf(__FILE__, __LINE__, localbuf, sizeof(localbuf), "%s",
+	       _intoa(myGlobals.device[i].network, intoabuf, sizeof(intoabuf)));
       if(checkFilter(filter, "network"))
 	wrtStrItm(fDescr, lang, "\t", "network", localbuf, ',', numEntries);
-      if(snprintf(localbuf, sizeof(localbuf), "%s",
-	       _intoa(myGlobals.device[i].netmask, intoabuf, sizeof(intoabuf))) < 0)
-        BufferTooShort();
+      safe_snprintf(__FILE__, __LINE__, localbuf, sizeof(localbuf), "%s",
+	       _intoa(myGlobals.device[i].netmask, intoabuf, sizeof(intoabuf)));
       if(checkFilter(filter, "netmask"))
 	wrtStrItm(fDescr, lang, "\t", "netmask", localbuf, ',', numEntries);
-      if(snprintf(localbuf, sizeof(localbuf), "%s",
-	       _intoa(myGlobals.device[i].ifAddr, intoabuf, sizeof(intoabuf))) < 0)
-        BufferTooShort();
+      safe_snprintf(__FILE__, __LINE__, localbuf, sizeof(localbuf), "%s",
+	       _intoa(myGlobals.device[i].ifAddr, intoabuf, sizeof(intoabuf)));
       if(checkFilter(filter, "ifAddr"))
 	wrtStrItm(fDescr, lang, "\t", "ifAddr", localbuf, ',', numEntries);
 
@@ -1340,8 +1333,19 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 	wrtLlongItm(fDescr, lang, "\t", "upTo1024",myGlobals.device[i].rcvdPktStats.upTo1024, ',', numEntries);
       if(checkFilter(filter, "upTo1518"))
 	wrtLlongItm(fDescr, lang, "\t", "upTo1518",myGlobals.device[i].rcvdPktStats.upTo1518, ',', numEntries);
+#ifdef MAKE_WITH_JUMBO_FRAMES
+      if(checkFilter(filter, "upTo2500"))
+	wrtLlongItm(fDescr, lang, "\t", "upTo2500",myGlobals.device[i].rcvdPktStats.upTo2500, ',', numEntries);
+      if(checkFilter(filter, "upTo6500"))
+	wrtLlongItm(fDescr, lang, "\t", "upTo6500",myGlobals.device[i].rcvdPktStats.upTo6500, ',', numEntries);
+      if(checkFilter(filter, "upTo9000"))
+	wrtLlongItm(fDescr, lang, "\t", "upTo9000",myGlobals.device[i].rcvdPktStats.upTo9000, ',', numEntries);
+      if(checkFilter(filter, "above9000"))
+	wrtLlongItm(fDescr, lang, "\t", "above9000",myGlobals.device[i].rcvdPktStats.above9000, ',', numEntries);
+#else
       if(checkFilter(filter, "above1518"))
 	wrtLlongItm(fDescr, lang, "\t", "above1518",myGlobals.device[i].rcvdPktStats.above1518, ',', numEntries);
+#endif
       if(checkFilter(filter, "shortest"))
 	wrtLlongItm(fDescr, lang, "\t", "shortest",myGlobals.device[i].rcvdPktStats.shortest, ',', numEntries);
       if(checkFilter(filter, "longest"))
@@ -1416,9 +1420,9 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 	if(!shortView) { initWriteKey(fDescr, lang, "\t", "IP", numEntries); }
 
 	for(j=0; j<myGlobals.numIpProtosToMonitor; j++) {
-	  if(!shortView) {
+	  if((myGlobals.ipTrafficProtosNames[j] != NULL) && (!shortView)) {
 	    if(j > 0) endWriteKey(fDescr, lang, "\t\t", hostKey, ',');
-	    initWriteKey(fDescr, lang, "\t\t", (hostKey = myGlobals.protoIPTrafficInfos[j]), numEntries);
+	    initWriteKey(fDescr, lang, "\t\t", (hostKey = myGlobals.ipTrafficProtosNames[j]), numEntries);
 	    wrtLlongItm(fDescr, lang,"\t\t\t","local",
 			myGlobals.device[i].ipProtoStats[j].local, ',', numEntries);
 	    wrtLlongItm(fDescr, lang,"\t\t\t","local2remote",
@@ -1436,12 +1440,10 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 	      myGlobals.device[i].ipProtoStats[j].remote2local.value+
 	      myGlobals.device[i].ipProtoStats[j].remote.value;
 
-	    wrtLlongItm(fDescr, lang, "\t",  myGlobals.protoIPTrafficInfos[j],
-			ctr, ',', numEntries);
+	    wrtLlongItm(fDescr, lang, "\t", myGlobals.ipTrafficProtosNames[j], ctr, ',', numEntries);
 	  }
 	}
 
-
 	if(!shortView) {
 	  if(hostKey != NULL) endWriteKey(fDescr, lang,"\t\t", hostKey, ',');
 	  endWriteKey(fDescr, lang,"\t", "IP", ',');
