@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -40,6 +40,7 @@ char *version, *osName, *author, *buildDate, *configureDate,
             *configure_parameters,
             *host_system_type,
             *target_system_type,
+            *compiler_cppflags,
             *compiler_cflags,
             *include_path,
             *system_libs,
@@ -57,7 +58,7 @@ static u_short _headerSize[MAX_DLT_ARRAY];
 extern char _wdir[];
 #endif
 
-static char *_dataFileDirs[]   = { 
+static char *_dataFileDirs[]   = {
 #ifndef EMBEDDED
   ".",
 #endif
@@ -98,15 +99,16 @@ void initGdbm(char *prefDirectory,  /* Directory with persistent files */
 
   traceEvent(CONST_TRACE_INFO, "Initializing gdbm databases");
 
-  setSpecifiedUser();
+  if(initPrefsOnly) {
+    initSingleGdbm(&myGlobals.prefsFile,        "prefsCache.db",   prefDirectory,  FALSE, NULL);
+    initSingleGdbm(&myGlobals.pwFile,           "ntop_pw.db",      prefDirectory,  FALSE, NULL);
+    return;
+  }
 
-  initSingleGdbm(&myGlobals.prefsFile,        "prefsCache.db",   prefDirectory,  FALSE, NULL);
-  initSingleGdbm(&myGlobals.pwFile,           "ntop_pw.db",      prefDirectory,  FALSE, NULL);
-
-  if(initPrefsOnly) return;
   initSingleGdbm(&myGlobals.addressQueueFile, "addressQueue.db", spoolDirectory, TRUE,  NULL);
   initSingleGdbm(&myGlobals.dnsCacheFile,     "dnsCache.db",     spoolDirectory, -1,    NULL);
   initSingleGdbm(&myGlobals.macPrefixFile,    "macPrefix.db",    spoolDirectory, FALSE,  &statbuf);
+  initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",    spoolDirectory, FALSE,  &statbuf);
   createVendorTable(&statbuf);
 }
 
@@ -128,8 +130,16 @@ static void allocateOtherHosts() {
 	  sizeof(myGlobals.otherHostEntry->hostResolvedName));
   myGlobals.otherHostEntry->hostResolvedNameType = FLAG_HOST_SYM_ADDR_TYPE_FAKE;
   strcpy(myGlobals.otherHostEntry->ethAddressString, "00:00:00:00:00:00");
-  myGlobals.otherHostEntry->portsUsage = (PortUsage**)calloc(sizeof(PortUsage*),
-							     MAX_ASSIGNED_IP_PORTS);
+  myGlobals.otherHostEntry->portsUsage = NULL;
+}
+
+/* ************************************ */
+
+void extend8021Qmtu(void) {
+#ifndef MAKE_WITH_JUMBO_FRAMES                                  
+  /* 1500 + 14 bytes header + 4 VLAN */
+  _mtuSize[DLT_EN10MB] = 1500+sizeof(struct ether_header)+4;
+#endif
 }
 
 /* ************************************ */
@@ -137,11 +147,9 @@ static void allocateOtherHosts() {
 /*
  * Initialize all global run-time parameters to default (reasonable!!!) values
  */
-void initNtopGlobals(int argc, char * argv[]) {
+void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_started[]) {
   int i, bufLen;
-  char *startedAs;
-
-  memset(&myGlobals, 0, sizeof(myGlobals));
+  char *startedAs, *defaultPath;
 
   /*
    * Notice the program name
@@ -156,56 +164,26 @@ void initNtopGlobals(int argc, char * argv[]) {
   myGlobals.ntop_argc = argc;
   myGlobals.ntop_argv = argv;
 
-  myGlobals.accessLogFile = DEFAULT_NTOP_ACCESS_LOG_FILE;
-  myGlobals.stickyHosts = DEFAULT_NTOP_STICKY_HOSTS;
+  initUserPrefs(&myGlobals.runningPref);
 
-  myGlobals.daemonMode = DEFAULT_NTOP_DAEMON_MODE;
+  /* Overrides for above */
   if (strcmp(myGlobals.program_name, "ntopd") == 0) {
-    myGlobals.daemonMode = 1;
+    myGlobals.runningPref.daemonMode = 1;
   }
-  
-  myGlobals.rFileName = DEFAULT_NTOP_TRAFFICDUMP_FILENAME;
-  myGlobals.devices = DEFAULT_NTOP_DEVICES;
-  myGlobals.dontTrustMACaddr = DEFAULT_NTOP_DONT_TRUST_MAC_ADDR;
-  myGlobals.trackOnlyLocalHosts    = DEFAULT_NTOP_TRACK_ONLY_LOCAL;
-  myGlobals.enableSessionHandling  = DEFAULT_NTOP_ENABLE_SESSIONHANDLE;
-  myGlobals.enablePacketDecoding   = DEFAULT_NTOP_PACKET_DECODING;
-  myGlobals.filterExpressionInExtraFrame = DEFAULT_NTOP_FILTER_IN_FRAME;
-  myGlobals.pcapLog = DEFAULT_NTOP_PCAP_LOG_FILENAME;
-  myGlobals.numericFlag = DEFAULT_NTOP_NUMERIC_IP_ADDRESSES;
-  myGlobals.localAddresses = DEFAULT_NTOP_LOCAL_SUBNETS;
-  myGlobals.enableSuspiciousPacketDump = DEFAULT_NTOP_SUSPICIOUS_PKT_DUMP;
-  myGlobals.enableOtherPacketDump = DEFAULT_NTOP_OTHER_PKT_DUMP;
-  myGlobals.disablePromiscuousMode = DEFAULT_NTOP_DISABLE_PROMISCUOUS;
-  myGlobals.traceLevel = DEFAULT_TRACE_LEVEL;
-  myGlobals.currentFilterExpression = DEFAULT_NTOP_FILTER_EXPRESSION;
-  strncpy((char *) &myGlobals.domainName, DEFAULT_NTOP_DOMAIN_NAME, sizeof(myGlobals.domainName));
-  myGlobals.flowSpecs = DEFAULT_NTOP_FLOW_SPECS;
-  myGlobals.maxNumHashEntries = myGlobals.maxNumSessions = (u_int)-1;
-  myGlobals.skipVersionCheck = FALSE;
+
+  /* note that by default ntop will merge network interfaces */
+  if(myGlobals.runningPref.mergeInterfaces == 0)
+    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge disabled by default");
+  else
+    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge enabled by default");
+
   myGlobals.checkVersionStatus = FLAG_CHECKVERSION_NOTCHECKED;
   myGlobals.checkVersionStatusAgain = 0;
-  
-#ifndef WIN32
-  myGlobals.debugMode = DEFAULT_NTOP_DEBUG_MODE;
-  myGlobals.useSyslog = DEFAULT_NTOP_SYSLOG;
-#ifdef HAVE_LIBWRAP
+
+#if !defined(WIN32) && defined(HAVE_LIBWRAP)
   allow_severity = DEFAULT_TCPWRAP_ALLOW;
   deny_severity = DEFAULT_TCPWRAP_DENY;
 #endif
-#endif
-
-  myGlobals.mergeInterfaces = DEFAULT_NTOP_MERGE_INTERFACES;
-  /* note that by default ntop will merge network interfaces */
-  myGlobals.mapperURL = DEFAULT_NTOP_MAPPER_URL;
-
-#ifdef MAKE_WITH_GDCHART
-  myGlobals.throughput_chart_type = DEFAULT_NTOP_CHART_TYPE;
-#endif
-
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-   myGlobals.useSSLwatchdog = 0;
-#endif
 
   /* Other flags (to be set via command line options one day) */
   myGlobals.enableFragmentHandling = 1;
@@ -214,9 +192,13 @@ void initNtopGlobals(int argc, char * argv[]) {
   myGlobals.dataFileDirs    = _dataFileDirs;
   myGlobals.pluginDirs      = _pluginDirs;
   myGlobals.configFileDirs  = _configFileDirs;
-  myGlobals.pcapLogBasePath = strdup(CFG_DBFILE_DIR);   /* a NULL pointer will break the logic */
-  myGlobals.dbPath          = strdup(CFG_DBFILE_DIR);   /* a NULL pointer will break the logic */
-  myGlobals.spoolPath       = strdup("");              /* a NULL pointer will break the logic */
+
+#ifdef WIN32
+  defaultPath = _wdir;
+#else
+  defaultPath = CFG_DBFILE_DIR;
+#endif
+  myGlobals.dbPath          = strdup(defaultPath);     /* a NULL pointer will break the logic */
 
   /* NB: we can't init rrdPath here, because initGdbm hasn't been run */
 
@@ -238,43 +220,28 @@ void initNtopGlobals(int argc, char * argv[]) {
   /* administrative */
   myGlobals.shortDomainName = NULL;
 
-#ifdef CFG_MULTITHREADED
   myGlobals.numThreads = 0;            /* # of running threads */
 
   myGlobals.numDequeueThreads = 1;
 
-#ifdef MAKE_ASYNC_ADDRESS_RESOLUTION
   for (i = 0; i < MAX_NUM_DEQUEUE_THREADS; i ++)
     myGlobals.dequeueAddressThreadId[i] = (pthread_t)-1;
-#endif
-
-#endif /* CFG_MULTITHREADED */
 
 #ifdef HAVE_OPENSSL
   myGlobals.sslInitialized = 0;
-  myGlobals.sslPort = 0;           /* Disabled by default: it can enabled using -W <SSL port> */
+  myGlobals.runningPref.sslPort = 0; /* Disabled by default: enabled via -W */
 #endif
 
-  myGlobals.webAddr = DEFAULT_NTOP_WEB_ADDR;
-  myGlobals.webPort = DEFAULT_NTOP_WEB_PORT;
-  myGlobals.ipv4or6 = DEFAULT_NTOP_FAMILY;
-
-  /* Termination flags */
-  myGlobals.capturePackets = FLAG_NTOPSTATE_RUN;    /* By default data are collected into internal variables */
-  myGlobals.endNtop = 0;
-
   myGlobals.dnsSniffCount = 0;
   myGlobals.dnsSniffRequestCount = 0;
   myGlobals.dnsSniffFailedCount = 0;
   myGlobals.dnsSniffARPACount = 0;
   myGlobals.dnsSniffStoredInCache = 0;
 
-#if defined(MAKE_ASYNC_ADDRESS_RESOLUTION)
   myGlobals.addressQueuedCount = 0;
   myGlobals.addressQueuedDup = 0;
   myGlobals.addressQueuedCurrent = 0;
   myGlobals.addressQueuedMax = 0;
-#endif
 
   /* Address Resolution counters */
   myGlobals.numipaddr2strCalls = 0;
@@ -319,7 +286,7 @@ void initNtopGlobals(int argc, char * argv[]) {
   myGlobals.udpSvc = NULL;
   myGlobals.tcpSvc = NULL;
 
-  myGlobals.protoIPTrafficInfos = NULL;
+  myGlobals.ipTrafficProtosNames = NULL;
   myGlobals.numIpProtosToMonitor = 0;
   myGlobals.ipPortMapper.numElements = 0;
   myGlobals.ipPortMapper.theMapper = NULL;
@@ -337,9 +304,53 @@ void initNtopGlobals(int argc, char * argv[]) {
     myGlobals.numUnsuccessfulDenied[i] = 0;
     myGlobals.numUnsuccessfulForbidden[i] = 0;
   }
+  myGlobals.numSSIRequests = 0;
+  myGlobals.numBadSSIRequests = 0;
+  myGlobals.numHandledSSIRequests = 0;
+
+/* create the logView stuff Mutex first... must be before the 1st traceEvent() call */
+  createMutex(&myGlobals.logViewMutex);     /* synchronize logView buffer */
+#ifdef FORPRENPTL
+  #warning Making version for Pre NPTL Thread Library...
+  createMutex(&myGlobals.preNPTLlogMutex);     /* synchronize logView buffer */
+#endif
+  myGlobals.logViewNext = 0;
+  myGlobals.logView = (char**)calloc(sizeof(char*),
+				     CONST_LOG_VIEW_BUFFER_SIZE);
+
+  /* traceEvent(CONST_TRACE_INFO, "Initializing semaphores, mutexes and threads"); */
+
+  /* ============================================================
+   * Create semaphores and mutexes associated with packet capture
+   * ============================================================
+   */
+#ifdef HAVE_PTHREAD_ATFORK
+  i = pthread_atfork(NULL, NULL, &reinitMutexes);
+  /* traceEvent(CONST_TRACE_INFO, "NOTE: atfork() handler registered for mutexes, rc %d", i); */
+#endif
+
+  /*
+   * Create two variables (semaphores) used by functions in pbuf.c to queue packets
+   */
+  createCondvar(&myGlobals.queueCondvar);
+  createCondvar(&myGlobals.queueAddressCondvar);
+
+  createMutex(&myGlobals.gdbmMutex);        /* data to synchronize thread access to db files */
+  createMutex(&myGlobals.tcpSessionsMutex); /* data to synchronize TCP sessions access */
+  createMutex(&myGlobals.fcSessionsMutex); /* data to synchronize TCP sessions access */
+  createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
+  createMutex(&myGlobals.purgeMutex);       /* synchronize purging */
+  createMutex(&myGlobals.securityItemsMutex);
+  createMutex(&myGlobals.packetProcessMutex);
+  createMutex(&myGlobals.packetQueueMutex);
+  createMutex(&myGlobals.hostsHashLockMutex);
+
+  for(i=0; i<CONST_HASH_INITIAL_SIZE; i++) {
+    createMutex(&myGlobals.hostsHashMutex[i]);
+    myGlobals.hostsHashMutexNumLocks[i] = 0;
+  }
 
   /* Packet Capture */
-#if defined(CFG_MULTITHREADED)
   for (i = 0; i <= CONST_PACKET_QUEUE_LENGTH; i ++)
     memset(&myGlobals.packetQueue[i], 0, sizeof(PacketInformation));
 
@@ -351,7 +362,6 @@ void initNtopGlobals(int argc, char * argv[]) {
   myGlobals.receivedPacketsLostQ = 0;
   myGlobals.packetQueueHead = 0;
   myGlobals.packetQueueTail = 0;
-#endif
 
   /* NB: Log View is allocated in main.c so it's available for the very 1st traceEvent() */
 
@@ -370,9 +380,7 @@ void initNtopGlobals(int argc, char * argv[]) {
        *     of packets as "too long", 2) the suspicious packet file, if one, may have
        *     extra or missing entries, and 3) an erroneous line in the report.
        *
-       *     If headerSize is wrong, the only problem will be in nfsPlugin.c, but this may
-       *     cause problems, as it uses the value to strip off the header so it can analyze
-       *     the nfs packet.
+       *     If headerSize is wrong, there are no known problems.
        *
        *  Remember that for most protocols, mtu isn't fixed - it's set by the routers
        *  and can be tuned by the sysadmin, isp, et al for "best results".
@@ -394,8 +402,12 @@ void initNtopGlobals(int argc, char * argv[]) {
   _mtuSize[DLT_NULL] = 8232                                    /* no link-layer encapsulation */;
   _headerSize[DLT_NULL] = CONST_NULL_HDRLEN;
 
+#ifdef MAKE_WITH_JUMBO_FRAMES
+  _mtuSize[DLT_EN10MB] = 9000                                  /* Ethernet (1000Mb+ Jumbo Frames) */;
+#else
       /* 1500 + 14 bytes header Courtesy of Andreas Pfaller <a.pfaller@pop.gun.de> */
   _mtuSize[DLT_EN10MB] = 1500+sizeof(struct ether_header)      /* Ethernet (10Mb) */;
+#endif
   _headerSize[DLT_EN10MB] = sizeof(struct ether_header);
 
   _mtuSize[DLT_PRONET] = 17914                                 /* Proteon ProNET Token Ring */;
@@ -448,18 +460,7 @@ void initNtopGlobals(int argc, char * argv[]) {
 
   myGlobals.mtuSize        = _mtuSize;
   myGlobals.headerSize     = _headerSize;
-
-  /* NetFlow */
-  myGlobals.netFlowInSocket = -1;
-
-  /* sFlow */
-  myGlobals.sflowDeviceId = -1;
-  myGlobals.sflowInPort   = 0;
-  myGlobals.sflowInSocket = 0;
-
-#ifdef CFG_MULTITHREADED
   myGlobals.numDequeueThreads = MAX_NUM_DEQUEUE_THREADS;
-#endif
 
   /* ********************************** */
 
@@ -490,15 +491,15 @@ void initNtopGlobals(int argc, char * argv[]) {
   /* ********************************** */
 
   bufLen = 0;
-  for (i=0; i<argc; i++) {
-    bufLen += (2 + strlen(argv[i]));
+  for (i=0; i<argc_started; i++) {
+    bufLen += (2 + strlen(argv_started[i]));
   }
 
   startedAs = (char*)malloc(bufLen);
   memset(startedAs, 0, (size_t) bufLen);
-  for (i=0; i<argc; i++) {
-    if (argv[i] != NULL) {
-      strncat(startedAs, argv[i], (bufLen - strlen(startedAs) - 1));
+  for (i=0; i<argc_started; i++) {
+    if (argv_started[i] != NULL) {
+      strncat(startedAs, argv_started[i], (bufLen - strlen(startedAs) - 1));
       strncat(startedAs, " ", (bufLen - strlen(startedAs) - 1));
     }
   }
@@ -508,11 +509,11 @@ void initNtopGlobals(int argc, char * argv[]) {
   /* FC & SCSI-specific stuff */
   myGlobals.scsiDefaultDevType = SCSI_DEV_UNINIT;
 
-  if (!myGlobals.noFc) {
+  if (!myGlobals.runningPref.printIpOnly) {
       if (myGlobals.fcnsCacheHash != NULL) {
           free (myGlobals.fcnsCacheHash);
       }
-      myGlobals.fcnsCacheHash = malloc (CONST_HASH_INITIAL_SIZE * sizeof(FcNameServerCacheEntry *));
+      myGlobals.fcnsCacheHash = malloc(CONST_HASH_INITIAL_SIZE * sizeof(FcNameServerCacheEntry *));
       if (myGlobals.fcnsCacheHash == NULL) {
           traceEvent (CONST_TRACE_ERROR, "Unable to allocate fc Name Server Cache\n");
       }
@@ -521,8 +522,8 @@ void initNtopGlobals(int argc, char * argv[]) {
              CONST_HASH_INITIAL_SIZE * sizeof(FcNameServerCacheEntry *));
 
       myGlobals.displayOption = DISPLAY_FC_DEFAULT;
-      if (!myGlobals.defaultVsan)
-          myGlobals.defaultVsan = DEFAULT_VSAN;
+      if (!myGlobals.runningPref.defaultVsan)
+          myGlobals.runningPref.defaultVsan = DEFAULT_VSAN;
   }
   myGlobals.fcMatrixHashCollisions = 0;
   myGlobals.fcMatrixHashUnresCollisions = 0;
@@ -541,41 +542,38 @@ void initNtop(char *devices) {
     addDefaultProtocols();
 
   /*
-   * initialize memory and data
+   * initialize memory and data.
    */
   initDevices(devices);
 
-  if(myGlobals.enableSessionHandling)
+  if(myGlobals.runningPref.enableSessionHandling)
     initPassiveSessions();
 
   /* ********************************** */
 
   initGdbm(myGlobals.dbPath, myGlobals.spoolPath, 0);
 
-#ifndef WIN32
-  if(myGlobals.daemonMode) {
-    daemonize();
-    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Now running as a daemon");
+  if(myGlobals.runningPref.daemonMode) {
+    daemonizeUnderUnix();
   }
-#endif
-
-#ifdef WIN32
-  initWinsock32();
-#endif
 
   /* Handle local addresses (if any) */
-  handleLocalAddresses(myGlobals.localAddresses);
+  handleLocalAddresses(myGlobals.runningPref.localAddresses);
 
-  if((myGlobals.rFileName != NULL) && ((myGlobals.localAddresses == NULL) && !myGlobals.printFcOnly)) {
-    traceEvent(CONST_TRACE_FATALERROR, 
-	       "-m | local-subnets must be specified when the -f | --traffic-dump-file option is used");
-    exit(-1);
+  if((myGlobals.runningPref.rFileName != NULL) &&
+     ((myGlobals.runningPref.localAddresses == NULL) &&
+      !myGlobals.runningPref.printFcOnly)) {
+      setRunState(FLAG_NTOPSTATE_SHUTDOWN);
+      traceEvent(CONST_TRACE_FATALERROR,
+                 "-m | local-subnets must be specified when the -f | --traffic-dump-file option is used"
+                 "Capture not started");
+      exit(2); /* Just in case */
   }
-  
-  if(myGlobals.currentFilterExpression != NULL)
+
+  if(myGlobals.runningPref.currentFilterExpression != NULL)
     parseTrafficFilter();
   else
-    myGlobals.currentFilterExpression = strdup(""); /* so that it isn't NULL! */
+    myGlobals.runningPref.currentFilterExpression = strdup(""); /* so that it isn't NULL! */
 
   /* Handle flows (if any) */
   handleFlowsSpecs();
@@ -586,50 +584,15 @@ void initNtop(char *devices) {
   initThreads();
 
 #ifndef MAKE_MICRO_NTOP
-  if (!myGlobals.printFcOnly) {
-      /* No support for any plugins in FC mode for now */
-      traceEvent(CONST_TRACE_NOISY, "Starting Plugins");
-      startPlugins();
-      traceEvent(CONST_TRACE_NOISY, "Plugins started... continuing with initialization");
-  }
+  traceEvent(CONST_TRACE_NOISY, "Starting Plugins");
+  startPlugins();
+  traceEvent(CONST_TRACE_NOISY, "Plugins started... continuing with initialization");
 #endif
 
-#if defined(HAVE_MALLINFO_MALLOC_H) && defined(HAVE_MALLOC_H) && defined(__GNUC__)
-  {
-    struct mallinfo memStats;
-
-    memStats = mallinfo();
-    myGlobals.baseMemoryUsage = memStats.arena + memStats.hblkhd;
-
-    traceEvent(CONST_TRACE_NOISY, "MEMORY: Base memory load is %.2fMB (%d+%d)",
-	       xvertDOT00MB(myGlobals.baseMemoryUsage),
-	       memStats.arena,
-	       memStats.hblkhd);
-  }
-#endif
-  traceEvent(CONST_TRACE_NOISY, "MEMORY: Base interface structure (no hashes loaded) is %.2fMB each",
-	     xvertDOT00MB(sizeof(NtopInterface)));
-  traceEvent(CONST_TRACE_NOISY, "MEMORY:     or %.2fMB for %d interfaces",
-	     xvertDOT00MB(myGlobals.numDevices*sizeof(NtopInterface)),
-	     myGlobals.numDevices);
-  traceEvent(CONST_TRACE_NOISY, "MEMORY: ipTraffixMatrix structure (no TrafficEntry loaded) is %.2fMB",
-	     xvertDOT00MB(myGlobals.ipTrafficMatrixMemoryUsage));
-
-#ifdef NOT_YET  
-  traceEvent(CONST_TRACE_NOISY, "MEMORY: fcTrafficMatrix structure (no TrafficEntry loaded) is %.2fMB",
-	     xvertDOT00MB(myGlobals.fcTrafficMatrixMemoryUsage));
-#endif  
-
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Sniffying...");
-
-#ifdef MEMORY_DEBUG
+#if defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 3)
   resetLeaks();
 #endif
 
-#ifndef WIN32
-  saveNtopPid();
-#endif
-
   addNewIpProtocolToHandle("IGMP", 2, 0 /* no proto */);
   addNewIpProtocolToHandle("OSPF", 89, 0 /* no proto */);
   addNewIpProtocolToHandle("IPSEC", 50, 51);
@@ -639,9 +602,9 @@ void initNtop(char *devices) {
     storePrefsValue("globals.displayPolicy", "0");
   } else {
     myGlobals.hostsDisplayPolicy = atoi(value);
-    
+
     /* Out of range check */
-    if((myGlobals.hostsDisplayPolicy < showAllHosts) 
+    if((myGlobals.hostsDisplayPolicy < showAllHosts)
        || (myGlobals.hostsDisplayPolicy > showOnlyRemoteHosts))
       myGlobals.hostsDisplayPolicy = showAllHosts;
   }
@@ -651,24 +614,148 @@ void initNtop(char *devices) {
     storePrefsValue("globals.localityPolicy", "0");
   } else {
     myGlobals.localityDisplayPolicy = atoi(value);
-    
+
     /* Out of range check */
-    if((myGlobals.localityDisplayPolicy < showSentReceived) 
+    if((myGlobals.localityDisplayPolicy < showSentReceived)
        || (myGlobals.localityDisplayPolicy > showOnlyReceived))
       myGlobals.localityDisplayPolicy = showSentReceived;
   }
 
-#ifdef CFG_MULTITHREADED
-  {
-    pthread_t myThreadId;
-    createThread(&myThreadId, checkVersion, NULL);
+
+  if(myGlobals.runningPref.skipVersionCheck != TRUE) {
+      pthread_t myThreadId;
+      createThread(&myThreadId, checkVersion, NULL);
   }
-#else
-  checkVersion(NULL);
-#endif
 }
 
-/* ****************************** */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
+/* This routine enforces the state change rules
+ *
+ *
+ * Valid state transitions:
+ *   0->1           When ntop first starts up, after memset(myGlobals)
+ *   1->2           After the basic system protective environment is up...
+ *   2->3           When ntop gives up root
+ *   3->4           When ntop finishes initialization
+ *Or 2->4           When ntop finishes initialization on systems w/o root, e.g. Win32
+ *   4->5, 6, 7     Stopcap to keep webserver up after a problem or Shutdown on user request
+ *   5->6, 7        Shutdown requested
+ *   6->7           Shutdown running
+ *   7->8           Shutdown complete
+ *   8->1 (restart) FUTURE...
+ *
+ */
+
+short _setRunState(char *file, int line, short newRunState) {
+
+static short stateTransitionTable[FLAG_NTOPSTATE_TERM+1][FLAG_NTOPSTATE_TERM+1];
+static char *stateTransitionTableNames[FLAG_NTOPSTATE_TERM+1];
+static short stateTransitionTableLoaded=0;
+
+  if(stateTransitionTableLoaded == 0) {
+    /* One time load */
+    int i;
+
+    for(i=0; i<FLAG_NTOPSTATE_TERM; i++) 
+      stateTransitionTable[i][i] = 1;
+
+    stateTransitionTable[FLAG_NTOPSTATE_NOTINIT][FLAG_NTOPSTATE_PREINIT] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_PREINIT][FLAG_NTOPSTATE_INIT] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_INIT][FLAG_NTOPSTATE_INITNONROOT] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_INIT][FLAG_NTOPSTATE_SHUTDOWN] = 1; /* abort */
+    stateTransitionTable[FLAG_NTOPSTATE_INITNONROOT][FLAG_NTOPSTATE_RUN] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_INIT][FLAG_NTOPSTATE_RUN] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_RUN][FLAG_NTOPSTATE_STOPCAP] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_RUN][FLAG_NTOPSTATE_SHUTDOWNREQ] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_RUN][FLAG_NTOPSTATE_SHUTDOWN] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_STOPCAP][FLAG_NTOPSTATE_SHUTDOWNREQ] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_STOPCAP][FLAG_NTOPSTATE_SHUTDOWN] = 1;
+
+    for(i=FLAG_NTOPSTATE_PREINIT; i<FLAG_NTOPSTATE_SHUTDOWNREQ; i++) 
+      stateTransitionTable[i][FLAG_NTOPSTATE_SHUTDOWNREQ] = 1;
+
+    stateTransitionTable[FLAG_NTOPSTATE_SHUTDOWNREQ][FLAG_NTOPSTATE_SHUTDOWN] = 1;
+    stateTransitionTable[FLAG_NTOPSTATE_SHUTDOWN][FLAG_NTOPSTATE_TERM] = 1;
+
+    stateTransitionTableNames[FLAG_NTOPSTATE_NOTINIT] = "NOTINIT";
+    stateTransitionTableNames[FLAG_NTOPSTATE_PREINIT] = "PREINIT";
+    stateTransitionTableNames[FLAG_NTOPSTATE_INIT] = "INIT";
+    stateTransitionTableNames[FLAG_NTOPSTATE_INITNONROOT] = "INITNONROOT";
+    stateTransitionTableNames[FLAG_NTOPSTATE_RUN] = "RUN";
+    stateTransitionTableNames[FLAG_NTOPSTATE_STOPCAP] = "STOPCAP";
+    stateTransitionTableNames[FLAG_NTOPSTATE_SHUTDOWNREQ] = "SHUTDOWNREQ";
+    stateTransitionTableNames[FLAG_NTOPSTATE_SHUTDOWN] = "SHUTDOWN";
+    stateTransitionTableNames[FLAG_NTOPSTATE_TERM] = "TERM";
+
+    stateTransitionTableLoaded = 1;
+
+  }
+
+  if(stateTransitionTable[myGlobals.ntopRunState][newRunState] == 0) {
+    traceEvent(CONST_FATALERROR_TRACE_LEVEL, file, line,
+               "Invalid runState transition %d to %d",
+               myGlobals.ntopRunState,
+               newRunState);
+    exit(99);
+  }
+
+/* These are largely blueprints for the future */
+
+  /* Take appropriate finishing action(s) for old state... */
+  switch(newRunState) {
+    case FLAG_NTOPSTATE_NOTINIT:
+      break;
+    case FLAG_NTOPSTATE_PREINIT:
+      break;
+    case FLAG_NTOPSTATE_INIT:
+      break;
+    case FLAG_NTOPSTATE_INITNONROOT:
+      break;
+    case FLAG_NTOPSTATE_RUN:
+      break;
+    case FLAG_NTOPSTATE_STOPCAP:
+      break;
+    case FLAG_NTOPSTATE_SHUTDOWNREQ:
+      break;
+    case FLAG_NTOPSTATE_SHUTDOWN:
+      break;
+    case FLAG_NTOPSTATE_TERM:
+      break;
+  }
+
+  /* Take appropriate action(s) for new state... */
+  switch(newRunState) {
+    case FLAG_NTOPSTATE_NOTINIT:
+      break;
+    case FLAG_NTOPSTATE_PREINIT:
+      break;
+    case FLAG_NTOPSTATE_INIT:
+      break;
+    case FLAG_NTOPSTATE_INITNONROOT:
+      break;
+    case FLAG_NTOPSTATE_RUN:
+      break;
+    case FLAG_NTOPSTATE_STOPCAP:
+      break;
+    case FLAG_NTOPSTATE_SHUTDOWNREQ:
+      break;
+    case FLAG_NTOPSTATE_SHUTDOWN:
+      break;
+    case FLAG_NTOPSTATE_TERM:
+      break;
+  }
+  
+  myGlobals.ntopRunState = newRunState;
+  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "THREADMGMT[t%lu]: ntop RUNSTATE: %s(%d)",
+             pthread_self(),
+             stateTransitionTableNames[newRunState],
+             newRunState);
+
+  return(myGlobals.ntopRunState);
+}
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifdef MAKE_WITH_SYSLOG
 /*
