@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -31,9 +31,15 @@
 #define TABLE_OFF ""
 #define TH_BG     ""
 #define TD_BG     ""
-#define TR_ON     ""
+#define TR_ON     "onMouseOver=\"this.bgColor = '#EDF3FE'\" onMouseOut =\"this.bgColor = '#FFFFFF'\""
 #define TABLE_DEFAULTS " CELLSPACING=0 CELLPADDING=2"
-#define DARK_BG   "BGCOLOR=\"#E7E9F2\"" /* If you change it change it into the stylesheet too */
+#define DARK_BG   "BGCOLOR=\"#F3F3F3\"" /* If you change it change it into the
+                                         * stylesheet too */
+#define TOMATO_BG "BGCOLOR=\"#FF6347\""
+#define GOLD_BG   "BGCOLOR=\"#FFD700\""
+#define SALMON_BG "BGCOLOR=\"FA8072\""
+#define WHEAT_BG  "BGCOLOR=\"F5DEB3\""
+#define AZURE_BG  "BGCOLOR=\"F0FFFF\""
 #endif
 
 
@@ -66,21 +72,24 @@ extern void printTableDoubleEntry(char *buf, int bufLen,
 extern void printTableEntryPercentage(char *buf, int bufLen,
 				      char *label, char* label_1,
 				      char* label_2, float total,
-				      float percentage);
+				      float percentage,
+				      u_int showFlows, Counter flows);
 extern void printPageTitle(char *text);
 extern void printSectionTitle(char *text);
 extern void printHostsCharacterization(void);
 extern void printFlagedWarning(char *text);
-extern void printHeader(int reportType, int revertOrder, u_int column, 
+extern void printHeader(int reportType, int revertOrder, u_int column,
                         HostsDisplayPolicy showHostsMode,
-                        LocalityDisplayPolicy showLocalityMode);
+                        LocalityDisplayPolicy showLocalityMode,
+			char *vlanList, u_short vlanId);
 extern void printFcHeader(int reportType, int revertOrder, u_int column, u_int hour, char *url);
 extern void printFcHostHeader (HostTraffic *el, char *url, int revertOrder,
                                int column, int hostInfoPage);
 extern void printFcDisplayOptions (void);
 extern void printFooterHostLink(void);
 extern void printFooter(int reportType);
-extern char* getOSFlag(HostTraffic *el, char *_osName, int showOsName, char *tmpStr, int tmpStrLen);
+#define getOSFlag(a, b, c, d, e)   _getOSFlag(a, b, c, d, e, __FILE__, __LINE__)
+extern char* _getOSFlag(HostTraffic *el, char *_osName, int showOsName, char *tmpStr, int tmpStrLen, char *file, int line);
 extern int sortHostFctn(const void *_a, const void *_b);
 extern int sortFcHostFctn(const void *_a, const void *_b);
 extern int cmpUsersTraffic(const void *_a, const void *_b);
@@ -122,7 +131,9 @@ extern void printHostUsedServices(HostTraffic *el, int actualDeviceId);
 extern void printHostIcmpStats(HostTraffic *el);
 extern void printTableEntry(char *buf, int bufLen,
 			    char *label, char* color,
-			    float total, float percentage);
+			    float total, float percentage,
+			    u_int showFlows, Counter flows,
+			    u_int showRRD);
 extern char* buildHTMLBrowserWindowsLabel(int i, int j, u_short forIpTraffic);
 extern int cmpEventsFctn(const void *_a, const void *_b);
 extern void printHostHourlyTrafficEntry(HostTraffic *el, int i,
@@ -140,20 +151,22 @@ extern FcFabricElementHash *getFcFabricElementHash (u_short vsanId, int actualDe
 extern void dumpFcFabricElementHash (FcFabricElementHash **theHash, char* label,
                                      u_char dumpLoopbackTraffic, u_char vsanHash);
 
-#ifdef CFG_MULTITHREADED
 extern void printMutexStatus(int textPrintFlag, PthreadMutex *mutexId, char *mutexName);
-#endif
 
 extern void printPluginTrailer(char *left, char *middle);
 
 /* http.c */
+extern char* encodeString(char* in, char* out, u_int out_len);
 extern void printHTMLtrailer(void);
 extern void returnHTTPredirect(char* destination);
-extern int generateInternalPages(char* pageName);
 void returnHTTPpageNotFound(char* additionalText);
 
-extern void sendStringLen(char *theString, unsigned int len);
-extern void sendString(char *theString);
+#define sendStringLen(a, b) _sendStringLen(a, b, 1)
+#define sendStringLenWOssi(a, b) _sendStringLen(a, b, 0)
+extern void _sendStringLen(char *theString, unsigned int len, int allowSSI);
+#define sendString(a) _sendString(a, 1)
+#define sendStringWOssi(a) _sendString(a, 0)
+extern void _sendString(char *theString, int allowSSI);
 extern void printHTTPtrailer(void);
 extern void initAccessLog(void);
 extern void termAccessLog(void);
@@ -166,6 +179,7 @@ extern void printHTMLheader(char *title, char *htmlTitle, int headerFlags);
 extern char* printSSLError(int errorId);
 #endif /* HAVE_OPENSSL */
 extern void sendHTTPHeader(int mimeType, int headerFlags, int useCompressionIfAvailable);
+extern int readHTTPpostData(int len, char *buf, int buflen);
 
 /* report.c */
 extern void printBandwidthFooter(void);
@@ -174,13 +188,12 @@ extern int reportValues(time_t *lastTime);
 extern void addPageIndicator(char *url, u_int beginIdx,
 			     u_int numEntries, u_int linesPerPage,
 			     int revertOrder, int numCol);
+extern void printTrafficSummary(int revertOrder);
 extern void printTrafficStatistics(int revertOrder);
 extern int combineReportTypeLocality(int reportTypeReq, LocalityDisplayPolicy showLocalityMode);
-extern void printHostsTraffic(int reportType,
-			      int sortedColumn, int revertOrder,
-			      int pageNum, char* url, 
-                              HostsDisplayPolicy showHostsMode,
-                              LocalityDisplayPolicy showLocalityMode);
+extern void printHostsTraffic(int reportType, int sortedColumn, int revertOrder,
+			      int pageNum, char* url, HostsDisplayPolicy showHostsMode,
+                              LocalityDisplayPolicy showLocalityMode, int vlanId);
 extern void printFcHostsTraffic(int reportType,
 			      int sortedColumn, int revertOrder,
 			      int pageNum, char* url,
@@ -188,8 +201,9 @@ extern void printFcHostsTraffic(int reportType,
 extern void printMulticastStats(int sortedColumn /* ignored so far */,
                                 int revertOrder, int pageNum);
 extern void printVSANList(unsigned int deviceId);
-extern void printHostsInfo(int sortedColumn, int revertOrder, int pageNum);
-extern void printFcHostsInfo(int sortedColumn, int revertOrder, int pageNum);
+extern void printHostsInfo(int sortedColumn, int revertOrder, int pageNum, int showBytes, int vlanId);
+extern void makeDot(void);
+extern void printFcHostsInfo(int sortedColumn, int revertOrder, int pageNum, int showBytes, int vsanId);
 extern void printAllSessionsHTML (char* host, int actualDeviceId, int sortedColumn,
                                   int revertOrder, int pageNum, char *url,
                                   int hostInfoPage);
@@ -221,18 +235,20 @@ extern void printScsiLunStats (HostTraffic *el, int actualDeviceId,
                                int sortedColumn, int revertOrder,
                                int pageNum, char *lun);
 extern void printScsiLunGraphs (HostTraffic *el, int actualDeviceId);
+extern void printFcTrafficSummary (u_short vsanId);
 extern void printIpProtocolUsage(void);
 extern void printBar(char *buf, int bufLen, unsigned short percentageS, unsigned short percentageR,
                      unsigned short maxPercentage, unsigned short ratio);
-extern void printIpProtocolDistribution(int mode, int revertOrder);
-extern void printFcProtocolDistribution(int mode, int revertOrder);
-extern void printProtoTraffic(void);
+extern void printIpProtocolDistribution(int mode, int revertOrder, int printGraph);
+extern void printFcProtocolDistribution(int mode, int revertOrder, int printGraph);
+extern void printProtoTraffic(int printGraph);
 extern void printProcessInfo(int processPid, int actualReportDeviceId);
 extern void printIpTrafficMatrix(void);
 extern void printFcTrafficMatrix (u_short vsanId, u_char sent);
 extern void printThptStatsMatrix(int sortedColumn);
 extern void printThptStats(int sortedColumn);
-extern void printDomainStats(char* domainName, int sortedColumn, int revertOrder, int pageNum);
+extern void printDomainStats(char* domainName, int clusterMode, 
+			     int sortedColumn, int revertOrder, int pageNum);
 extern void printNoDataYet(void);
 extern void printNotAvailable(char* flagName);
 extern void listNetFlows(void);
@@ -240,8 +256,10 @@ extern void printHostHourlyTraffic(HostTraffic *el);
 extern void printASList(unsigned int deviceId);
 extern void printVLANList(unsigned int deviceId);
 extern void showPortTraffic(u_short portNr);
-void drawVsanStatsGraph (unsigned int deviceId);
-void printVSANList(unsigned int deviceId);
+extern void drawVsanStatsGraph (unsigned int deviceId);
+extern void printVSANList(unsigned int deviceId);
+extern void handleNtopConfig (char *url, UserPrefDisplayPage page, int postLen);
+extern void purgeHost(char *serialized_serial);
 
 /* webInterface.c */
 extern int execCGI(char* cgiName);
@@ -256,12 +274,12 @@ extern void shutdownNtop(void);
 extern void printHostHourlyTraffic(HostTraffic *el);
 extern void printNtopConfigHInfo(int textPrintFlag);
 extern void printHostColorCode(int textPrintFlag, int isInfo);
-#ifdef CFG_MULTITHREADED
 extern void printMutexStatusReport(int textPrintFlag);
-#endif
-extern void printNtopConfigInfo(int textPrintFlag);
+extern void printNtopConfigInfo(int textPrintFlag, UserPref *pref);
 extern void printNtopProblemReport(void);
 extern void initSocket(int isSSL, int ipv4or6, int *port, int *sock, char *addr);
+extern bool processNtopPref (char *key, char *value, bool savePref,
+                             UserPref *pref);
 
 #ifdef MAKE_WITH_SSLWATCHDOG
 extern int sslwatchdogWaitFor(int stateValue, int parentchildFlag, int alreadyLockedFlag);
@@ -273,6 +291,8 @@ extern void sslwatchdogSighandler(int signum);
 extern void* sslwatchdogChildThread(void* notUsed _UNUSED_);
 #endif
 extern int printNtopLogReport(int printAsText);
+extern char* makeHostAgeStyleSpec(HostTraffic *el, char *buf, int bufSize);
+extern void edit_prefs(char *db_key, char *db_val);
 
 /* **************************** */
 
@@ -344,5 +364,6 @@ extern void drawArea(short width, short height, FILE* filepointer,
 		     int   num_points, char  *labels[], float data[],
 		     char *xtitle, char *ytitle, u_short formatYlabels);
 extern void drawPie(short width, short height, FILE* filepointer,
-		    int   num_points, char  *labels[], float data[]);
+		    int   num_points, char  *labels[], float data[],
+		    int sorted);
 
