@@ -45,7 +45,7 @@ static struct in_addr *in4_cpy(struct in_addr *dst, struct in_addr *src) {
   return memcpy(dst, src, sizeof(struct in_addr));
 }
 
-static str2in6_addr(char *str, struct in6_addr *addr){
+static void str2in6_addr(char *str, struct in6_addr *addr) {
   int i;
   unsigned int x;
 
@@ -54,6 +54,7 @@ static str2in6_addr(char *str, struct in6_addr *addr){
     addr->s6_addr[i]= x & 0xff;
   }
 }
+
 static int prefixlen(void *val, int size) {
   unsigned char *name = (unsigned char *)val;
   int byte, bit, plen = 0;
@@ -304,6 +305,9 @@ static int iface_getflags(struct iface_if *ii) {
     ii->info |= IFACE_INFO_MULTICAST;
   if (lifreq.ifr_flags & IFF_PROMISC) /* receive all packets  */
     ii->info |= IFACE_INFO_PROMISC;
+
+  close(sock);
+
   return 1;
 }
 
@@ -312,7 +316,6 @@ static int iface_getflags(struct iface_if *ii) {
 struct iface_handler *iface_new(void) {
   char buf[1024];
   char straddr[33];
-  char bufntop[67];
   char ifname[20];
   int ifindex, plen, scope, status;
   int found = 0;
