@@ -2,7 +2,7 @@
  *  parsetime.c - parse time for at(1)
  *  Copyright (C) 1993, 1994  Thomas Koenig
  *
- *  modifications for english-language times
+ *  modifications for English-language times
  *  Copyright (C) 1993  David Parsons
  *
  *  A lot of modifications and extensions 
@@ -225,7 +225,7 @@ static struct SpecialToken TimeMultipliers[] = {
 
 /* File scope variables */
 
-/* context dependant list of specials for parser to recognize,
+/* context dependent list of specials for parser to recognize,
  * required for us to be able distinguish between 'mon' as 'month'
  * and 'mon' as 'monday'
  */
@@ -237,14 +237,15 @@ static char *sct;	/* scanner - next char pointer in current argument */
 static int need;	/* scanner - need to advance to next argument */
 
 static char *sc_token=NULL;	/* scanner - token buffer */
-static size_t sc_len;   /* scanner - lenght of token buffer */
+static size_t sc_len;   /* scanner - length of token buffer */
 static int sc_tokid;	/* scanner - token id */
 
 static int need_to_free = 0; /* means that we need deallocating memory */
 
 /* Local functions */
 
-void EnsureMemFree ()
+/* ntop change to quiet warning from -Wmissing-prototypes */
+static void EnsureMemFree ()
 {
   if( need_to_free )
     {
@@ -260,12 +261,12 @@ void EnsureMemFree ()
  * should return TIME_OK (aka NULL) or pointer to the error message,
  * and should be called like this: try(func(args));
  *
- * if the try is not successfull it will reset the token pointer ...
+ * if the try is not successful it will reset the token pointer ...
  *
  * [NOTE: when try(...) is used as the only statement in the "if-true"
  *  part of the if statement that also has an "else" part it should be
  *  either enclosed in the curly braces (despite the fact that it looks
- *  like a single statement) or NOT follwed by the ";"]
+ *  like a single statement) or NOT followed by the ";"]
  */
 #define try(b)		{ \
 			char *_e; \
@@ -288,7 +289,7 @@ void EnsureMemFree ()
 
 /*
  * ve() and e() are used to set the return error,
- * the most aprropriate use for these is inside panic(...) 
+ * the most appropriate use for these is inside panic(...) 
  */
 #define MAX_ERR_MSG_LEN	1024
 static char errmsg[ MAX_ERR_MSG_LEN ];
@@ -317,7 +318,7 @@ e ( char *fmt, ... )
 }
 
 /* Compare S1 and S2, ignoring case, returning less than, equal to or
-   greater than zero if S1 is lexiographically less than,
+   greater than zero if S1 is lexicographically less than,
    equal to or greater than S2.  -- copied from GNU libc*/
 static int
 mystrcasecmp (s1, s2)
@@ -450,7 +451,7 @@ token()
 
 
 /* 
- * expect2() gets a token and complins if it's not the token we want
+ * expect2() gets a token and complains if it's not the token we want
  */
 static char *
 expect2(int desired, char *complain_fmt, ...)
@@ -469,10 +470,10 @@ expect2(int desired, char *complain_fmt, ...)
 /*
  * plus_minus() is used to parse a single NUMBER TIME-UNIT pair
  *              for the OFFSET-SPEC.
- *              It allso applies those m-guessing euristics.
+ *              It also applies those m-guessing heuristics.
  */
 static char *
-plus_minus(struct time_value *ptv, int doop)
+plus_minus(struct rrd_time_value *ptv, int doop)
 {
     static int op = PLUS;
     static int prev_multiplier = -1;
@@ -552,10 +553,11 @@ plus_minus(struct time_value *ptv, int doop)
  * tod() computes the time of day (TIME-OF-DAY-SPEC)
  */
 static char *
-tod(struct time_value *ptv)
+tod(struct rrd_time_value *ptv)
 {
     int hour, minute = 0;
     int tlen;
+    
     /* save token status in  case we must abort */
     int scc_sv = scc; 
     char *sct_sv = sct; 
@@ -628,7 +630,7 @@ tod(struct time_value *ptv)
  * assign_date() assigns a date, adjusting year as appropriate
  */
 static char *
-assign_date(struct time_value *ptv, long mday, long mon, long year)
+assign_date(struct rrd_time_value *ptv, long mday, long mon, long year)
 {
     if (year > 138) {
 	if (year > 1970)
@@ -656,10 +658,11 @@ assign_date(struct time_value *ptv, long mday, long mon, long year)
  * day() picks apart DAY-SPEC-[12]
  */
 static char *
-day(struct time_value *ptv)
+day(struct rrd_time_value *ptv)
 {
     long mday=0, wday, mon, year = ptv->tm.tm_year;
     int tlen;
+    time_t montime;
 
     switch (sc_tokid) {
     case YESTERDAY:
@@ -710,10 +713,12 @@ day(struct time_value *ptv)
     case NUMBER:
 	    /* get numeric <sec since 1970>, MM/DD/[YY]YY, or DD.MM.[YY]YY
 	     */
-	    tlen = strlen(sc_token);
+
+		tlen = strlen(sc_token);
 	    mon = atol(sc_token);
-            if (mon > 10*356*24*60*60) {
-		ptv->tm=*localtime(&mon);
+            if (mon > 10*365*24*60*60) {
+                montime = mon;
+                ptv->tm=*localtime(&montime);
 		token();
 		break;
 	    }
@@ -743,7 +748,7 @@ day(struct time_value *ptv)
 		  token();
 		}
 		
-		/* flip months and days for european timing
+		/* flip months and days for European timing
 		 */
 		if (sep == DOT) {
 		  long x = mday;
@@ -773,7 +778,7 @@ day(struct time_value *ptv)
 
 /*
  * parsetime() is the external interface that takes tspec, parses
- * it and puts the result in the time_value structure *ptv.
+ * it and puts the result in the rrd_time_value structure *ptv.
  * It can return either absolute times (these are ensured to be
  * correct) or relative time references that are expected to be
  * added to some absolute time value and then normalized by
@@ -781,7 +786,7 @@ day(struct time_value *ptv)
  * the pointer to the error message in the case of problems
  */
 char *
-parsetime(char *tspec, struct time_value *ptv)
+parsetime(char *tspec, struct rrd_time_value *ptv)
 {
     time_t now = time(NULL);
     int hr = 0;
@@ -795,7 +800,8 @@ parsetime(char *tspec, struct time_value *ptv)
     ptv->type = ABSOLUTE_TIME;
     ptv->offset = 0;
     ptv->tm = *localtime(&now);
-    ptv->tm.tm_isdst = -1; /* mk time can figure this out for us ... */
+    /* local time has set this right */
+/*    ptv->tm.tm_isdst = -1; -- mk time can figure this out for us ... */
 
     token();
     switch (sc_tokid) {
@@ -844,7 +850,7 @@ parsetime(char *tspec, struct time_value *ptv)
 	    try(tod(ptv))
 	    break;
 
-	    /* evil coding for TEATIME|NOON|MIDNIGHT - we've initialised
+	    /* evil coding for TEATIME|NOON|MIDNIGHT - we've initialized
 	     * hr to zero up above, then fall into this case in such a
 	     * way so we add +12 +4 hours to it for teatime, +12 hours
 	     * to it for noon, and nothing at all for midnight, then
@@ -896,7 +902,7 @@ parsetime(char *tspec, struct time_value *ptv)
       panic(e("unparsable trailing text: '...%s%s'", sc_token, sct));
     }
 
-    ptv->tm.tm_isdst = -1; /* for mktime to guess DST status */
+    /*    ptv->tm.tm_isdst = -1; */ /* for mktime to guess DST status */
     if( ptv->type == ABSOLUTE_TIME )
       if( mktime( &ptv->tm ) == -1 ) { /* normalize & check */
         /* can happen for "nonexistent" times, e.g. around 3am */
@@ -908,8 +914,8 @@ parsetime(char *tspec, struct time_value *ptv)
 } /* parsetime */
 
 
-int proc_start_end (struct time_value *start_tv, 
-		    struct time_value *end_tv, 
+int proc_start_end (struct rrd_time_value *start_tv, 
+		    struct rrd_time_value *end_tv, 
 		    time_t *start, 
 		    time_t *end){
     if (start_tv->type == RELATIVE_TO_END_TIME  && /* same as the line above */
