@@ -1,11 +1,14 @@
 /*****************************************************************************
- * RRDtool 1.0.42  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * pngsize.c  determine the size of a PNG image
  *****************************************************************************/
 
 #include <png.h>
 
+/* ntop addition to quiet warning from -Wmissing-prototypes */
+int PngSize(FILE *fd, long *width, long *height);
+
 int
 PngSize(FILE *fd, long *width, long *height)
 {
