@@ -1,14 +1,20 @@
 
 /****************************************************************************
- * RRDtool 1.0.42  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * rrd__graph.c  make creates ne rrds
  ****************************************************************************/
 
 #include "rrd_tool.h"
+
+#include <locale.h>
+
 #include "rrd_graph.h"
 #include <gd.h>
 
+/* Shutup gcc4 warning: */
+extern void gdImageGif(gdImagePtr  gif, FILE  *fo);
+
 #ifdef LATIN2_ENABLE
 #include <gdlucidan10l2.h>
 #include <gdlucidab12l2.h>
@@ -39,15 +45,6 @@
 #define LargeFont gdLucidaBold12
 #endif
 
-/* #define DEBUG */
-
-#ifdef DEBUG
-# define DPRINT(x)    (void)(printf x, printf("\n"))
-#else
-# define DPRINT(x)
-#endif
-
-
 xlab_t xlab[] = {
     {0,        TMT_SECOND,30, TMT_MINUTE,5,  TMT_MINUTE,5,         0,"%H:%M"},
     {2,        TMT_MINUTE,1,  TMT_MINUTE,5,  TMT_MINUTE,5,         0,"%H:%M"},
@@ -56,7 +53,7 @@ xlab_t xlab[] = {
     {30,       TMT_MINUTE,10, TMT_HOUR,1,    TMT_HOUR,1,           0,"%H:%M"},
     {60,       TMT_MINUTE,30, TMT_HOUR,2,    TMT_HOUR,2,           0,"%H:%M"},
     {180,      TMT_HOUR,1,    TMT_HOUR,6,    TMT_HOUR,6,           0,"%H:%M"},
-    /*{300,      TMT_HOUR,3,    TMT_HOUR,12,   TMT_HOUR,12,    12*3600,"%a %p"},  this looks silly*/
+ /* {300,      TMT_HOUR,3,    TMT_HOUR,12,   TMT_HOUR,12,    12*3600,"%a %p"},  this looks silly */
     {600,      TMT_HOUR,6,    TMT_DAY,1,     TMT_DAY,1,      24*3600,"%a"},
     {1800,     TMT_HOUR,12,   TMT_DAY,1,     TMT_DAY,2,      24*3600,"%a"},
     {3600,     TMT_DAY,1,     TMT_WEEK,1,     TMT_WEEK,1,    7*24*3600,"Week %V"},
@@ -175,9 +172,19 @@ ytr(image_desc_t *im, double value){
     } 
 }
 
-   
-    
+/* allocate colors with re-use */
 
+int color_allocate(gdImagePtr im, int r, int g, int b) { 
+
+	int color;
+
+	if ((color = gdImageColorExact(im, r, g, b)) != -1) {
+		return(color);
+	}
+
+	return(gdImageColorAllocate(im, r, g, b));
+}
+    
 
 /* conversion function for symbolic entry names */
 
@@ -343,7 +350,7 @@ si_unit(
     im->magfact = pow((double)im->base , digits);
 
 #ifdef DEBUG
-    printf("digits %6.3f  im->magfact %6.3f\n",digits,im->magfact);
+    fprintf(stderr,"digits %6.3f  im->magfact %6.3f\n",digits,im->magfact);
 #endif
 
     if ( ((digits+symbcenter) < sizeof(symbol)) &&
@@ -374,16 +381,42 @@ expand_range(image_desc_t *im)
 
     
 #ifdef DEBUG
-    printf("Min: %6.2f Max: %6.2f MagFactor: %6.2f\n",
+    fprintf(stderr,"Min: %6.2f Max: %6.2f MagFactor: %6.2f\n",
 	   im->minval,im->maxval,im->magfact);
 #endif
 
     if (isnan(im->ygridstep)){
 	if(im->extra_flags & ALTYMRTG) { /* mrtg */
- int  decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
- double scaledstep = ceil(25 * pow((double)10,-decimals) * im->maxval) * pow((double)10,decimals - 2);
- im->minval = 0;
- im->maxval = 4 * scaledstep;
+           im->decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
+           im->quadrant = 0;
+           if (im->minval < 0) {
+               im->quadrant = 2;
+               if (im->maxval <= 0)
+                   im->quadrant = 4;
+           }
+           switch (im->quadrant) {
+           case 2:
+               im->scaledstep = ceil(50 * pow((double)10,-(im->decimals)) * max(fabs(im->maxval),fabs(im->minval))) * pow((double)10,im->decimals - 2);
+               scaled_min = -2 * im->scaledstep;
+               scaled_max = 2 * im->scaledstep;
+               break;
+           case 4:
+               im->scaledstep = ceil(25 * pow((double)10,-(im->decimals)) * fabs(im->minval)) * pow((double)10,im->decimals - 2);
+               scaled_min = -4 * im->scaledstep;
+               scaled_max = 0;
+               break;
+           default: /* quadrant 0 */
+               im->scaledstep = ceil(25 * pow((double)10,-(im->decimals)) * im->maxval) * pow((double)10,im->decimals - 2);
+               scaled_min = 0;
+               scaled_max = 4 * im->scaledstep;
+               break;
+           }
+#ifdef DEBUG
+           fprintf(stderr,"[%d] %s Min: %6.2f Max: %6.2f MagFactor: %6.2f, decimals: %d scaledstep: %6.2f -> scaled Min: %6.2f Max %6.2f\n",
+                   getpid(),im->graphfile,im->minval,im->maxval,im->magfact,im->decimals,im->scaledstep,scaled_min,scaled_max);
+#endif
+           im->minval = scaled_min;
+           im->maxval = scaled_max;
 	} else
 	if(im->extra_flags & ALTAUTOSCALE) {
 	    /* measure the amplitude of the function. Make sure that
@@ -398,7 +431,7 @@ expand_range(image_desc_t *im)
 	      if (delt < fact) {
 		adj = (fact - delt) * 0.55;
 #ifdef DEBUG
-	      printf("Min: %6.2f Max: %6.2f delt: %6.2f fact: %6.2f adj: %6.2f\n", im->minval, im->maxval, delt, fact, adj);
+	      fprintf(stderr,"Min: %6.2f Max: %6.2f delt: %6.2f fact: %6.2f adj: %6.2f\n", im->minval, im->maxval, delt, fact, adj);
 #endif
 	      }
 	      im->minval -= adj;
@@ -520,7 +553,7 @@ reduce_data(
 
 
 /* Sanity check: row_cnt should be multiple of reduce_factor */
-/* if this gets triggered, something is REALY WRONG ... we die immediately */
+/* if this gets triggered, something is REALLY WRONG ... we die immediately */
 
     if (row_cnt%reduce_factor) {
 	printf("SANITY CHECK: %lu rows cannot be reduced by %i \n",
@@ -648,7 +681,7 @@ data_fetch( image_desc_t *im )
 	    }
 	}
 	
-        /* lets see if the required data source is realy there */
+        /* lets see if the required data source is really there */
 	for(ii=0;ii<im->gdes[i].ds_cnt;ii++){
 	    if(strcmp(im->gdes[i].ds_namv[ii],im->gdes[i].ds_nam) == 0){
 		im->gdes[i].ds=ii; }
@@ -706,22 +739,33 @@ rpnp_t *
 str2rpn(image_desc_t *im,char *expr){
     int pos=0;
     long steps=-1;    
-    rpnp_t  *rpnp;
+    rpnp_t  *rpnp=NULL;
     char vname[30];
-
-    rpnp=NULL;
+    char *tmp_locale;
+    int ret=0;
 
     while(*expr){
+
 	if ((rpnp = (rpnp_t *) rrd_realloc(rpnp, (++steps + 2)* 
 				       sizeof(rpnp_t)))==NULL){
 	    return NULL;
 	}
-
+#if 0
 	else if((sscanf(expr,"%lf%n",&rpnp[steps].val,&pos) == 1) 
 	        && (expr[pos] == ',')){
  	    rpnp[steps].op = OP_NUMBER;
 	    expr+=pos;
 	} 
+#endif
+	tmp_locale = setlocale(LC_NUMERIC, "C");
+	ret=sscanf(expr,"%lf%n",&rpnp[steps].val,&pos);
+	if (tmp_locale != NULL)
+		setlocale(LC_NUMERIC, tmp_locale);		
+	
+ 	if ((ret == 1) && (expr[pos] == ',')){
+		rpnp[steps].op = OP_NUMBER;
+		expr+=pos;
+	}
 	
 #define match_op(VV,VVV) \
         else if (strncmp(expr, #VVV, strlen(#VVV))==0 && \
@@ -1346,7 +1390,7 @@ data_proc( image_desc_t *im ){
 	}
     }
 
-    /* if min or max have not been asigned a value this is because
+    /* if min or max have not been assigned a value this is because
        there was no data in the graph ... this is not good ...
        lets set these to dummy values then ... */
 
@@ -1366,6 +1410,10 @@ data_proc( image_desc_t *im ){
 	else
 	    im->maxval = maxval;
     }
+    /* make sure min is smaller than max */
+    if (im->minval > im->maxval) {
+       im->minval = 0.99 * im->maxval;
+    }
     /* make sure min and max are not equal */
     if (im->minval == im->maxval) {
       im->maxval *= 1.01; 
@@ -1466,7 +1514,7 @@ find_next_time(
 	    tm.tm_year += basestep;	
 	}
 	madetime = mktime(&tm);
-    } while (madetime == -1); /* this is necessary to skip impssible times
+    } while (madetime == -1); /* this is necessary to skip impossible times
 				 like the daylight saving time skips */
     return madetime;
 	  
@@ -1484,7 +1532,7 @@ void gator( gdImagePtr gif, int x, int y){
 
 /* the fact that the text of what gets put into the graph is not
    visible in the function, has lead some to think this is for
-   obfuscation reasons. While this is a nice side effect (I addmit),
+   obfuscation reasons. While this is a nice side effect (I admit),
    it is not the prime reason. The prime reason is, that the font
    used, is so small, that I had to hand edit the characters to ensure
    readability. I could thus not use the normal gd functions to write,
@@ -1553,9 +1601,12 @@ print_calc(image_desc_t *im, char ***prdata)
 	    for(ii=im->gdes[vidx].ds+im->gdes[vidx].ds_cnt;
 		ii < max_ii+im->gdes[vidx].ds_cnt;
 		ii+=im->gdes[vidx].ds_cnt){
-		 if (! finite(im->gdes[vidx].data[ii]))
+#ifdef DEBUG
+                fprintf(stderr,"[%d] %s value: %6.2f\n",getpid(),im->graphfile,im->gdes[vidx].data[ii]);
+#endif
+  	        if (! finite(im->gdes[vidx].data[ii]))
 		     continue;
-		 if (isnan(printval)){
+		if (isnan(printval)){
 		     printval = im->gdes[vidx].data[ii];
 		     validsteps++;
 		    continue;
@@ -1663,7 +1714,7 @@ leg_place(image_desc_t *im)
     char  prt_fctn; /*special printfunctions */
     int  *legspace;
 
-    if( !(im->extra_flags & NOLEGEND) ) {
+    if( !(im->extra_flags & NOLEGEND) & !(im->extra_flags & ONLY_GRAPH) ) {
       if ((legspace = malloc(im->gdes_c*sizeof(int)))==NULL){
          rrd_set_error("malloc for legspace");
          return -1;
@@ -1672,17 +1723,19 @@ leg_place(image_desc_t *im)
     for(i=0;i<im->gdes_c;i++){
 	fill_last = fill;
 
-	if (im->gdes[i].gf == GF_HRULE &&
-	    (im->gdes[i].yrule < im->minval || im->gdes[i].yrule > im->maxval))
-	    im->gdes[i].legend[0] = '\0';
+	if( !(im->extra_flags & FORCE_RULES_LEGEND) ) {
+	    if (im->gdes[i].gf == GF_HRULE &&
+		(im->gdes[i].yrule < im->minval || im->gdes[i].yrule > im->maxval))
+		im->gdes[i].legend[0] = '\0';
 
-	if (im->gdes[i].gf == GF_VRULE &&
-	    (im->gdes[i].xrule < im->start || im->gdes[i].xrule > im->end))
-	    im->gdes[i].legend[0] = '\0';
+	    if (im->gdes[i].gf == GF_VRULE &&
+		(im->gdes[i].xrule < im->start || im->gdes[i].xrule > im->end))
+		im->gdes[i].legend[0] = '\0';
+	}
 
 	leg_cc = strlen(im->gdes[i].legend);
 	
-	/* is there a controle code ant the end of the legend string ? */ 
+	/* is there a control code ant the end of the legend string ? */ 
 	if (leg_cc >= 2 && im->gdes[i].legend[leg_cc-2] == '\\') {
 	    prt_fctn = im->gdes[i].legend[leg_cc-1];
 	    leg_cc -= 2;
@@ -1885,7 +1938,7 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
 	    if(i % labfact == 0){
 		if (i==0 || im->symbol == ' ') {
 		    if(scaledstep < 1){
-			if(im->extra_flags & ALTYGRID) {
+			if(i!=0 && im->extra_flags & ALTYGRID) {
 			    sprintf(graph_label,labfmt,scaledstep*i);
 			}
 			else {
@@ -1941,10 +1994,8 @@ int
 horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
 {
  int      i, xLeft, xRight, y;
- double   scaledstep;
  char     graph_label[100];
  int      styleMajor[2];
- int      decimals;
  char     labfmt[64];
 
  /* does the scale of this graph make it impossible to put lines
@@ -1958,21 +2009,25 @@ horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
  xLeft = im->xorigin;
  xRight = im->xorigin + im->xsize;
 
- decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
- scaledstep = ceil(25 * pow((double)10,-decimals) * im->maxval) * pow((double)10,decimals - 2) / im->magfact;
-
- sprintf(labfmt,"%%4.%df",1 - (scaledstep > (double)10 || ceil(scaledstep) == scaledstep));
+ if (im->scaledstep / im->magfact * max(abs(im->quadrant),abs(4 - im->quadrant)) <= (double)1)
+    strcpy(labfmt,"%5.2f");
+ else
+    sprintf(labfmt,"%%4.%df",1 - (im->scaledstep / im->magfact > (double)10 || ceil(im->scaledstep / im->magfact) == im->scaledstep /im->magfact));
  if (im->symbol != ' ' || im->unit)
-  strcat(labfmt," ");
+    strcat(labfmt," ");
  if (im->symbol != ' ')
-  sprintf(labfmt + strlen(labfmt),"%c",im->symbol);
+    sprintf(labfmt + strlen(labfmt),"%c",im->symbol);
  if (im->unit)
-  strcat(labfmt,im->unit);
+    strcat(labfmt,im->unit);
 
  for (i = 0; i <= 4; i++)
   {y = im->yorigin - im->ysize * i / 4;
    if (y >= im->yorigin-im->ysize && y <= im->yorigin)
-    {sprintf(graph_label,labfmt,scaledstep * i);
+    {sprintf(graph_label,labfmt,im->scaledstep / im->magfact * (i - im->quadrant));
+#ifdef DEBUG
+     fprintf(stderr,"[%d] scaledstep * (i - quadrant): %6.2f format '%s' label: '%s'\n",
+             getpid(),im->scaledstep * (i - im->quadrant),labfmt,graph_label);
+#endif
      gdImageString(gif,SmallFont,(xLeft - (strlen(graph_label) * SmallFont->w) - 7),
                                  y - SmallFont->h / 2 + 1,
                                  (unsigned char *)graph_label,graph_col[GRC_FONT].i);
@@ -1985,7 +2040,7 @@ horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
  return 1;
 }
 
-/* logaritmic horizontal grid */
+/* logarithmic horizontal grid */
 int
 horizontal_log_grid(gdImagePtr gif, image_desc_t   *im)
 {
@@ -2291,7 +2346,7 @@ grid_paint(
 		    (unsigned char *)im->title, graph_col[GRC_FONT].i);
     
     /* graph labels */
-    if( !(im->extra_flags & NOLEGEND) ) {
+    if( !(im->extra_flags & NOLEGEND) & !(im->extra_flags & ONLY_GRAPH) ) {
       for(i=0;i<im->gdes_c;i++){
 	if(im->gdes[i].legend[0] =='\0')
 	    continue;
@@ -2351,13 +2406,13 @@ MkLineBrush(image_desc_t *im,long cosel, enum gf_en typsel){
     return NULL;
   }
 
-  gdImageColorTransparent(brush, 
-			  gdImageColorAllocate(brush, 0, 0, 0));
 
   pen = gdImageColorAllocate(brush, 
 			     im->gdes[cosel].col.red,
 			     im->gdes[cosel].col.green,
 			     im->gdes[cosel].col.blue);
+
+  gdImageColorTransparent(brush, gdImageColorAllocate(brush, 0, 0, 0));
     
   switch (typsel){
   case GF_LINE1:
@@ -2395,7 +2450,7 @@ copyImage(gdImagePtr gif, char *image, int copy_white)
      {c2 = gdImageGetPixel(img,i1,i2);	/* get pixel */
       if (copy_white || img->red[c2] != 255 || img->green[c2] != 255 || img->blue[c2] != 255)
        {if ((c1 = gdImageColorExact(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
-         if ((c1 = gdImageColorAllocate(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
+         if ((c1 = color_allocate(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
           c1 = gdImageColorClosest(gif,img->red[c2],img->green[c2],img->blue[c2]);
         gdImageSetPixel(gif,i1,i2,c1);
        }
@@ -2490,18 +2545,39 @@ graph_paint(image_desc_t *im, char ***calcpr)
        on the cmdline is the graph area. But we need more as we have
        draw labels and other things outside the graph area */
 
+    if( (im->extra_flags & ONLY_GRAPH) ) {
+        if ( im->ysize > 64 ) {
+            rrd_set_error("height over 32 pixels with --only-graph option is not available");
+	    return -1;
+        }
+        im->xorigin = 0;
+    } else {
+        im->xorigin = 10 + im->unitslength * SmallFont->w;
+    }
 
-    im->xorigin = 10 + 9 * SmallFont->w+SmallFont->h;
-    xtr(im,0); 
+    xtr(im,0);
+
+    if(im->ylegend[0] != '\0')
+        im->xorigin += SmallFont->h;
+
+    if( (im->extra_flags & ONLY_GRAPH) ) {
+        im->yorigin = im->ysize;
+    } else {
+        im->yorigin = 14 + im->ysize;
+    }
 
-    im->yorigin = 14 + im->ysize;
     ytr(im,DNAN);
 
     if(im->title[0] != '\0')
 	im->yorigin += (LargeFont->h+4);
 
-    im->xgif=20+im->xsize + im->xorigin;
-    im->ygif= im->yorigin+2*SmallFont->h;
+    if( (im->extra_flags & ONLY_GRAPH) ) {
+        im->xgif= im->xsize;
+        im->ygif= im->yorigin;
+    } else {
+        im->xgif=16+im->xsize + im->xorigin;
+        im->ygif= im->yorigin+2*SmallFont->h;
+    }
     
     /* determine where to place the legends onto the graphics.
        and set im->ygif to match space requirements for text */
@@ -2517,13 +2593,13 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	/* check for user override values */
 	if(im->graph_col[i].red != -1)
 	    graph_col[i].i = 
-		gdImageColorAllocate( gif,
+		color_allocate( gif,
 				      im->graph_col[i].red, 
 				      im->graph_col[i].green, 
 				      im->graph_col[i].blue);
 	else
 	    graph_col[i].i = 
-		gdImageColorAllocate( gif,
+		color_allocate( gif,
 				      graph_col[i].red, 
 				      graph_col[i].green, 
 				      graph_col[i].blue);
@@ -2534,7 +2610,7 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	/* only for elements which have a color defined */
 	if (im->gdes[i].col.red != -1)
 	    im->gdes[i].col.i = 
-		gdImageColorAllocate(gif,
+		color_allocate(gif,
 				     im->gdes[i].col.red,
 				     im->gdes[i].col.green,
 				     im->gdes[i].col.blue);
@@ -2543,7 +2619,7 @@ graph_paint(image_desc_t *im, char ***calcpr)
        graph elements and then drawing them */
 
   if (im->bkg_image)	/* background image added first */
-   copyImage(gif,im->bkg_image,1);
+    copyImage(gif,im->bkg_image,1);
   else
    {
     back[0].x = 0;
@@ -2574,7 +2650,8 @@ graph_paint(image_desc_t *im, char ***calcpr)
     if (im->maxval < 0.0)
 	areazero = im->maxval;
 
-    axis_paint(im,gif);
+    if( !(im->extra_flags & ONLY_GRAPH) ) 
+        axis_paint(im,gif);
 
     for(i=0;i<im->gdes_c;i++){	
         
@@ -2658,7 +2735,8 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	}
     }
     
-    grid_paint(im,gif);
+    if( !(im->extra_flags & ONLY_GRAPH) ) 
+        grid_paint(im,gif); 
 
     /* the RULES are the last thing to paint ... */
     for(i=0;i<im->gdes_c;i++){	
@@ -2728,7 +2806,7 @@ gdes_alloc(image_desc_t *im){
 
     long def_step = (im->end-im->start)/im->xsize;
     
-    if (im->step > def_step) /* step can be increassed ... no decreassed */
+    if (im->step > def_step) /* step can be increased ... no decreased */
       def_step = im->step;
 
     im->gdes_c++;
@@ -2796,7 +2874,7 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
     unsigned int            col_red,col_green,col_blue;
     long           scancount;
     int linepass = 0; /* stack can only follow directly after LINE* AREA or STACK */    
-    struct time_value start_tv, end_tv;
+    struct rrd_time_value start_tv, end_tv;
     char *parsetime_error = NULL;
     int stroff;    
 
@@ -2825,6 +2903,7 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
     im.maxval = DNAN;    
     im.interlaced = 0;
     im.unitsexponent= 9999;
+    im.unitslength= 9;
     im.extra_flags= 0;
     im.rigid = 0;
     im.imginfo = NULL;
@@ -2869,11 +2948,14 @@ im.unit = 0; /* do not display unit if not set */
 	    {"imgformat",  required_argument, 0,  'a'},
 	    {"lazy",       no_argument,       0,  'z'},
 	    {"no-legend",  no_argument,       0,  'g'},
+            {"only-graph",  no_argument,       0,  'j'},
+      	    {"force-rules-legend",no_argument,0,  'F'},
 	    {"alt-y-grid", no_argument,       0,   'Y'},
             {"no-minor",   no_argument,       0,   'N'},
  	    {"alt-autoscale", no_argument,    0,   'A'},
 	    {"alt-autoscale-max", no_argument,    0,   'M'},
 	    {"units-exponent",required_argument, 0,  'X'},
+	    {"units-length",required_argument, 0,  'L'},
 	    {"step",       required_argument, 0,  'S'},
 	    {"alt-y-mrtg", no_argument,       0,  'R'},
 	    {"background", required_argument, 0,  'B'},
@@ -2885,7 +2967,7 @@ im.unit = 0; /* do not display unit if not set */
 
 	
 	opt = getopt_long(argc, argv, 
-			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgYAMX:S:RB:O:U:",
+			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgjFYAMX:L:S:RB:O:U:",
 			  long_options, &option_index);
 
 	if (opt == EOF)
@@ -2904,12 +2986,21 @@ im.unit = 0; /* do not display unit if not set */
 	case 'M':
 	    im.extra_flags |= ALTAUTOSCALE_MAX;
 	    break;
+	case 'j':
+	    im.extra_flags |= ONLY_GRAPH;
+	    break;
 	case 'g':
 	    im.extra_flags |= NOLEGEND;
 	    break;
+	case 'F':
+	    im.extra_flags |= FORCE_RULES_LEGEND;
+	    break;
 	case 'X':
 	    im.unitsexponent = atoi(optarg);
 	    break;
+	case 'L':
+	    im.unitslength = atoi(optarg);
+	    break;
 	case 'S':
 	    im.step =  atoi(optarg);
 	    break;
