@@ -1,40 +1,12 @@
 /*****************************************************************************
- * RRDtool 1.0.42  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_update.c  RRD Update Function
  *****************************************************************************
- * $Id: rrd_update.c,v 1.3 2003/06/19 10:06:04 deri Exp $
+ * $Id: rrd_update.c,v 1.4 2004/09/06 10:20:05 deri Exp $
  * $Log: rrd_update.c,v $
- * Revision 1.3  2003/06/19 10:06:04  deri
- * Remvoed DOS ^M
- *
- * Revision 1.2  2003/06/18 16:55:24  deri
- * Added fix until an official patch is released. This fix prevents ntop from
- * crashing while calling rrd_update(). For more info see my post to the RRD
- * mailing list:
- *
- * Dear all,
- * I have found a problem on rrd_diff(). Basically the function is
- * accessing invalid memory. This is because the two function parameters
- * (*a and *b) can be of different length (e.g. a = "1234", b = "13") so
- * the statement
- *
- * for (x=0; x<m; x++) { ... }
- *
- * moves the pointers (a1, b1, r1) for m locations (a1--;b1--;r1--;) where
- * m = max(strlen(a),strlen(b));
- *
- * In the above example m=4, but b1 cannot be shifted for more than 2
- * positions (strlen("13")=2), therefore the b1-- statement causes a crash
- * when b1 is moved for the third time.
- *
- * I have enclosed a quick hack (see below): fell free to accept it or to
- * fix it the way you want. Anything is fine as long as the problem is fixed.
- *
- * Regards, Luca
- *
- * Revision 1.1  2003/04/22 17:02:29  deri
- * Local RRD tree
+ * Revision 1.4  2004/09/06 10:20:05  deri
+ * Updated to RRDtool 1.0.49
  *
  * Revision 1.1.1.1  2002/02/26 10:21:38  oetiker
  * Intial Import
@@ -63,7 +35,7 @@ int
 main(int argc, char **argv){
         rrd_update(argc,argv);
         if (rrd_test_error()) {
-                printf("RRDtool 1.0.42  Copyright 1997-2000 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
+                printf("RRDtool 1.0.49  Copyright 1997-2000 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
                         "Usage: rrdupdate filename\n"
                         "\t\t\t[--template|-t ds-name:ds-name:...]\n"
                         "\t\t\ttime|N:value[:value...]\n\n"
@@ -126,6 +98,7 @@ rrd_update(int argc, char **argv)
     char             **updvals;
     int              wrote_to_file = 0;
     char             *template = NULL;          
+    char             *endptr; /* used in the conversion */
 
 
     while (1) {
@@ -167,10 +140,10 @@ rrd_update(int argc, char **argv)
 
         When a file is opened with udpate mode ('+' as the second
         or third character in the ... list of mode argument
-        variables), both input and ouptut may be performed on the
+        variables), both input and output may be performed on the
         associated stream.  However, ...  input may not be directly
         followed by output without an intervening call to a file
-        positioning function, unless the input oepration encounters
+        positioning function, unless the input operation encounters
         end-of-file. */
     fseek(rrd_file, 0, SEEK_CUR);
 
@@ -269,7 +242,7 @@ rrd_update(int argc, char **argv)
 	char *stepper = malloc((strlen(argv[arg_i])+1)*sizeof(char));
         char *step_start = stepper;
         if (stepper == NULL){
-                rrd_set_error("faild duplication argv entry");
+                rrd_set_error("failed duplication argv entry");
                 free(updvals);
                 free(pdp_temp);  
                 free(tmpl_idx);
@@ -305,7 +278,18 @@ rrd_update(int argc, char **argv)
 	if (strcmp(updvals[0],"N")==0){
 	    current_time = time(NULL);
 	} else {
-	    current_time = atol(updvals[0]);
+            errno = 0;
+	    current_time = strtol(updvals[0],&endptr,10);
+            if (errno > 0){
+                rrd_set_error("converting  '%s' to long: %s",updvals[0],strerror(errno));
+                free(step_start);
+                break;
+            };
+            if (endptr[0] != '\0'){
+                rrd_set_error("conversion of '%s' to long not complete: tail '%s'",updvals[0],endptr);
+                free(step_start);
+                break;
+            }
 	}
 	
 	if(current_time <= rrd.live_head->last_up){
@@ -370,7 +354,7 @@ rrd_update(int argc, char **argv)
 	       rrd.ds_def[i].par[DS_mrhb_cnt].u_cnt >= interval) {
 	       double rate = DNAN;
 	       /* the data source type defines how to process the data */
-		/* pdp_temp contains rate * time ... eg the bytes
+		/* pdp_temp contains rate * time ... e.g. the bytes
 		 * transferred during the interval. Doing it this way saves
 		 * a lot of math operations */
 		
@@ -379,9 +363,18 @@ rrd_update(int argc, char **argv)
 		case DST_COUNTER:
 		case DST_DERIVE:
 		    if(rrd.pdp_prep[i].last_ds[0] != 'U'){
+       		       for(ii=0;updvals[i+1][ii] != '\0';ii++){
+       		            if(updvals[i+1][ii] < '0' || updvals[i+1][ii] > '9' || (ii==0 && updvals[i+1][ii] == '-')){
+                                 rrd_set_error("not a simple integer: '%s'",updvals[i+1]);
+                                 break;
+                            }
+      		       }
+             	       if (rrd_test_error()){
+	        	    break;
+        	       }
 		       pdp_new[i]= rrd_diff(updvals[i+1],rrd.pdp_prep[i].last_ds);
 		       if(dst_idx == DST_COUNTER) {
-			  /* simple overflow catcher sugestet by andres kroonmaa */
+			  /* simple overflow catcher suggested by andres Kroonmaa */
 			  /* this will fail terribly for non 32 or 64 bit counters ... */
 			  /* are there any others in SNMP land ? */
 			  if (pdp_new[i] < (double)0.0 ) 
@@ -396,11 +389,29 @@ rrd_update(int argc, char **argv)
 		   }
 		   break;
 		case DST_ABSOLUTE:
-		    pdp_new[i]= atof(updvals[i+1]);
+                    errno = 0;
+	            pdp_new[i] = strtod(updvals[i+1],&endptr);
+                    if (errno > 0){
+                        rrd_set_error("converting  '%s' to float: %s",updvals[i+1],strerror(errno));
+                        break;
+                    };
+                    if (endptr[0] != '\0'){
+                        rrd_set_error("conversion of '%s' to float not complete: tail '%s'",updvals[i+1],endptr);
+                        break;
+                    }
 		    rate = pdp_new[i] / interval;		  
 		    break;
 		case DST_GAUGE:
-		    pdp_new[i] = atof(updvals[i+1]) * interval;
+                    errno = 0;
+		    pdp_new[i] = strtod(updvals[i+1],&endptr) * interval;
+                    if (errno > 0){
+                        rrd_set_error("converting  '%s' to float: %s",updvals[i+1],strerror(errno));
+                        break;
+                    };
+                    if (endptr[0] != '\0'){
+                        rrd_set_error("conversion of '%s' to float not complete: tail '%s'",updvals[i+1],endptr);
+                        break;
+                    }
 		    rate = pdp_new[i] / interval;		   
 		    break;
 		default:
