@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -108,7 +108,7 @@ void initWinsock32() {
 
     
 #ifdef WIN32
-  if(myGlobals.pcapLogBasePath) free(myGlobals.pcapLogBasePath); myGlobals.pcapLogBasePath = strdup(_wdir);
+  if(myGlobals.runningPref.pcapLogBasePath) free(myGlobals.runningPref.pcapLogBasePath); myGlobals.runningPref.pcapLogBasePath = strdup(_wdir);
   if(myGlobals.dbPath)          free(myGlobals.dbPath); myGlobals.dbPath = strdup(_wdir);
   if(myGlobals.spoolPath)       free(myGlobals.spoolPath); myGlobals.spoolPath = strdup(_wdir);
 #endif
@@ -166,7 +166,7 @@ ULONG GetHostIPAddr () {
 	for(i=0; (dotp[i] != '\0') && (dotp[i] != '.'); i++)
 	  ;
 
-	if(dotp[i] == '.') strncpy(myGlobals.domainName, &dotp[i+1], sizeof(myGlobals.domainName));
+	if(dotp[i] == '.') strncpy(myGlobals.runningPref.domainName, &dotp[i+1], sizeof(myGlobals.runningPref.domainName));
       }
     }
   }
@@ -203,6 +203,8 @@ ULONG GetHostIPAddr () {
 
 WIN32 MULTITHREAD STUFF
 
+http://www-128.ibm.com/developerworks/eserver/articles/es-MigratingWin32toLinux.html
+
 ************************************** */
 
 int createThread(pthread_t *threadId,
@@ -224,13 +226,20 @@ int createThread(pthread_t *threadId,
 
 /* ************************************ */
 
-int killThread(pthread_t *threadId) {
+int _killThread(pthread_t *threadId) {
   CloseHandle((HANDLE)*threadId);
   return(0);
 }
 
 /* ************************************ */
 
+int _joinThread(pthread_t *threadId) {
+  WaitForSingleObject((HANDLE)*threadId, INFINITE);
+  return(0);
+}
+
+/* ************************************ */
+
 int _createMutex(PthreadMutex *mutexId, char* fileName, int fileLine) {
 
   memset(mutexId, 0, sizeof(PthreadMutex));
@@ -285,13 +294,11 @@ int _accessMutex(PthreadMutex *mutexId, char* where,
   WaitForSingleObject(mutexId->mutex, INFINITE);
 
   mutexId->numLocks++;
-  mutexId->isLocked = 1;
-  mutexId->lockTime = time(NULL);
-
-  if(fileName != NULL) {
-    strcpy(mutexId->lockFile, fileName);
-    mutexId->lockLine = fileLine;
-  }
+    mutexId->isLocked = 1;
+    if(!myGlobals.runningPref.disableMutexExtraInfo) {
+      memcpy(&(mutexId->lock), &(mutexId->attempt), sizeof(Holder));
+      memset(&(mutexId->attempt), 0, sizeof(Holder));
+    }
 
   return(0);
 }
@@ -300,23 +307,26 @@ int _accessMutex(PthreadMutex *mutexId, char* where,
 
 int _tryLockMutex(PthreadMutex *mutexId, char* where,
 		  char* fileName, int fileLine) {
+	int rc;
 #ifdef DEBUG
   traceEvent(CONST_TRACE_INFO, "Try to Lock 0x%X @ %s [%s:%d]",
 	     mutexId->mutex, where, fileName, fileLine);
   fflush(stdout);
 #endif
 
-  if(WaitForSingleObject(mutexId->mutex, 0) == WAIT_FAILED)
+  rc = WaitForSingleObject(mutexId->mutex, 0);
+
+  /* traceEvent(CONST_TRACE_INFO, "_tryLockMutex=%d", rc); */
+
+  if(rc != WAIT_OBJECT_0 /* OK */)
     return(1);
   else {
     mutexId->numLocks++;
     mutexId->isLocked = 1;
-    mutexId->lockTime = time(NULL);
-
-    if(fileName != NULL) {
-      strcpy(mutexId->lockFile, fileName);
-      mutexId->lockLine = fileLine;
-    }
+    if(!myGlobals.runningPref.disableMutexExtraInfo) {
+      memcpy(&(mutexId->lock), &(mutexId->attempt), sizeof(Holder));
+      memset(&(mutexId->attempt), 0, sizeof(Holder));
+	}
 
     return(0);
   }
@@ -324,8 +334,7 @@ int _tryLockMutex(PthreadMutex *mutexId, char* where,
 
 /* ************************************ */
 
-int _releaseMutex(PthreadMutex *mutexId,
-		  char* fileName, int fileLine) {
+int _releaseMutex(PthreadMutex *mutexId, char* fileName, int fileLine) {
 
   time_t lockDuration;
   BOOL rc;
@@ -340,16 +349,19 @@ int _releaseMutex(PthreadMutex *mutexId,
     traceEvent(CONST_TRACE_WARNING, "Unlock failed for 0x%X [%s:%d] (LastError=%d)",
 	       mutexId->mutex, fileName, fileLine, GetLastError());
   }
+   mutexId->isLocked = 0;
+    mutexId->numReleases++;
 
-  lockDuration = time(NULL) - mutexId->lockTime;
+    if(!myGlobals.runningPref.disableMutexExtraInfo) {
 
-  if((mutexId->maxLockedDuration < lockDuration)
-     || (mutexId->maxLockedDurationUnlockLine == 0 /* Never set */)) {
-    mutexId->maxLockedDuration = lockDuration;
+      setHolder(mutexId->unlock);
+      lockDuration = timeval_subtract(mutexId->unlock.time, mutexId->lock.time);
 
-    if(fileName != NULL) {
-      strcpy(mutexId->maxLockedDurationUnlockFile, fileName);
-      mutexId->maxLockedDurationUnlockLine = fileLine;
+      if((mutexId->maxLockedDuration < lockDuration)
+         || (mutexId->max.line == 0 /* Never set */)) {
+        memcpy(&(mutexId->max), &(mutexId->lock), sizeof(Holder));
+        mutexId->maxLockedDuration = lockDuration;
+      }
     }
 
 #ifdef DEBUG
@@ -357,16 +369,8 @@ int _releaseMutex(PthreadMutex *mutexId,
 	       &(mutexId->mutex), fileName, fileLine,
 	       mutexId->maxLockedDuration);
 #endif
-  }
 
-  mutexId->isLocked = 0;
-  mutexId->numReleases++;
-  if(fileName != NULL) {
-    strcpy(mutexId->unlockFile, fileName);
-    mutexId->unlockLine = fileLine;
-  }
-
-  return(0);
+   return(0);
 }
 
 /* ************************************ */
@@ -421,14 +425,18 @@ void printAvailableInterfaces() {
   int i, numInterfaces = 0;
   pcap_if_t *devpointer;
 
+  ebuf[0] = '\0';
+
   printf("\nAvailable interfaces (-i <interface index>):\n");
 
   if(pcap_findalldevs(&devpointer, ebuf) < 0) {
-	;
+    ;
   } else {
-	  for (i = 0; devpointer != 0; i++) {
-	  printf("   [index=%d] %s\n             (%s)\n",
+    for (i = 0; devpointer != 0; i++) {
+		if(validInterface(devpointer->description)) {
+		printf("   [index=%d] %s\n             (%s)\n",
 			numInterfaces++, devpointer->description, devpointer->name);
+		}
 
       devpointer = devpointer->next;
     } /* for */
@@ -665,7 +673,7 @@ int gettimeofday(struct timeval *tv,
 unsigned long waitForNextEvent(unsigned long ulDelay /* ms */) {
   unsigned long ulSlice = 1000L; /* 1 Second */
 
-  while ((myGlobals.capturePackets != FLAG_NTOPSTATE_TERM) && (ulDelay > 0L)) {
+  while ((myGlobals.ntopRunState < FLAG_NTOPSTATE_SHUTDOWN) && (ulDelay > 0L)) {
     if (ulDelay < ulSlice)
       ulSlice = ulDelay;
     Sleep(ulSlice);
@@ -1189,8 +1197,8 @@ void installService(int argc, char **argv)
 	/* ****************************************** */
 	  // Set the service name. Courtesy of Yuri Francalacci <yuri@ntop.org>
       sprintf(szParamKey2, "SYSTEM\\CurrentControlSet\\Services\\%s",SZSERVICENAME);
-	snprintf(szDescr, sizeof(szDescr), "Ntop v.%s %s - Web-based network traffic monitor. http://www.ntop.org/",
-	   version, THREAD_MODE);
+      safe_snprintf(__FILE__, __LINE__, szDescr, sizeof(szDescr), "Ntop v.%s %s - Web-based network traffic monitor. http://www.ntop.org/",
+	   version, "MT");
 
       // Set the file value (where the message resources are located.... in this case, our runfile.)
       if(0 != setStringValue((const unsigned char *)szDescr,
@@ -1715,7 +1723,7 @@ void main(int argc, char **argv)
       printf("/c                - Run ntop on a console\n");
       printf("/r                - Deinstall the service\n");
       printf("/h                - Prints this help\n\n");
-      initNtopGlobals(argc, argv); 
+      initNtopGlobals(argc, argv, argc, argv); 
       usage(stdout);
     }
 
