@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *                      
  *  			    http://www.ntop.org/
  *  					
@@ -36,12 +36,6 @@
 #define EOPNOTSUPP              WSAEOPNOTSUPP
 #endif
 
-/*
-  Under Win32 we still use qsort.c as the builting sort
-  function seems to have some bugs
-*/
-#define qsort(a, b, c, d) quicksort(a, b, c, d)
-
 /* Courtesy of Wies-Software <wies@wiessoft.de> */
 extern char* getadminpass(const char *prompt);
 extern unsigned long waitForNextEvent(unsigned long ulDelay /* ms */);
@@ -81,9 +75,16 @@ extern char* getpass(const char *prompt);
 extern ULONG GetHostIPAddr();
 
 #define MAKE_WITH_ZLIB
+#define HAVE_DIRENT_H
+#define HAVE_PCAP_FREEALLDEVS
 
+#if !defined(__GNUC__)
 #define INET6
 #define in6_addr in_addr6
+#endif
+
+#define pthread_self GetCurrentThreadId
+
 
 /* *************************************************************** */
 
@@ -103,7 +104,7 @@ extern ULONG GetHostIPAddr();
     
 */
 
-#if defined(WIN32) && defined(__GNUC__)
+#if defined(WIN32) && defined(__GNUC__) && !defined(__MINGW32__)
 #define DIRENT_INCLUDED
 
 struct dirent
@@ -169,7 +170,7 @@ struct	ether_header {
 /************************************************************************/
 
 /* on mingw, tcp_seq is defined - Scott Renfro <scott@renfro.org> */
-#if defined (WIN32) && !defined (__GNUC__)
+#if defined (WIN32) && !defined (tcp_seq)
 typedef	u_int	tcp_seq;
 #endif
 
