@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -22,8 +22,10 @@
 
 #ifdef MAKE_STATIC_PLUGIN
 extern PluginInfo* icmpPluginEntryFctn(void);
-extern PluginInfo* nfsPluginEntryFctn(void);
 extern PluginInfo* sflowPluginEntryFctn(void);
+#ifdef HAVE_SNMP
+extern PluginInfo *snmpPluginEntryFctn(void);
+#endif
 #ifndef EMBEDDED
 extern PluginInfo* rrdPluginEntryFctn(void);
 #endif
@@ -32,99 +34,13 @@ extern PluginInfo* netflowPluginEntryFctn(void);
 
 /* ******************* */
 
-#ifdef AIX
-
-static char* dlerror() {
-  char *errMsg[768];
-  static char tmpStr[256];
-
-  if(loadquery(L_GETMESSAGES, &errMsg, 768) != -1) {
-    int i, j, errCode;
-    char* errName;
-
-    for(i=0; errMsg[i] != NULL; i++){
-      errCode=atoi(errMsg[i]);
-      errName = "";
-	  
-      for(j=1; errMsg[i][j] != '\0'; j++)
-	if(errMsg[i][j] != ' ') {
-	  errName = &errMsg[i][j];
-	  break;
-	}
-	  
-      switch(errCode) {
-	/* sys/ldr.h */
-      case 1:
-	return("Too many errors, rest skipped");
-	break;
-      case 2:
-	if(snprintf(tmpStr, sizeof(tmpStr), "Can't load library [%s]", errName) < 0) 
-	  BufferTooShort();
-	break;
-      case 3:
-	if(snprintf(tmpStr, sizeof(tmpStr), "Can't find symbol in library [%s]", errName) < 0) 
-	  BufferTooShort();
-	break;
-      case 4:
-	return("Rld data offset or symbol index out of range or bad relocation type");
-	break;
-      case 5:
-	if(snprintf(tmpStr, sizeof(tmpStr), "File not valid, executable xcoff [%s]", errName) < 0)
-	  BufferTooShort();
-	return(tmpStr);
-	break;
-      case 6:
-	if(snprintf(tmpStr, sizeof(tmpStr), "The errno associated with the failure if not ENOEXEC,"
-		" it indicates the underlying error, such as no memory [%s][errno=%d]", 
-		errName, errno) < 0) BufferTooShort();
-	return(tmpStr);
-	break;
-      case 7:
-	if(snprintf(tmpStr, sizeof(tmpStr), 
-		    "Member requested from a file which is not an archive or does not"
-		    "contain the member [%s]", errName) < 0) BufferTooShort();
-	return(tmpStr);
-	break;
-      case 8:
-	if(snprintf(tmpStr, sizeof(tmpStr), "Symbol type mismatch [%s]", errName) < 0)
-	  BufferTooShort();
-	return(tmpStr);
-	break;
-      case 9:
-	return("Text alignment in file is wrong");
-	break;
-      case 10:
-	return("Insufficient permission to create a loader domain");
-	break;
-      case 11:
-	return("Insufficient permission to add entries to a loader domain");
-	break;
-      default:
-	if(snprintf(tmpStr, sizeof(tmpStr), "Unknown error [%d]", errCode) < 0) 
-	  BufferTooShort();
-	return(tmpStr);
-      }
-    }
-  }
-}
-
-#endif /* AIX */
-
-/* ******************* */
-
 #if (defined(HAVE_DIRENT_H) && defined(HAVE_DLFCN_H)) || defined(WIN32) || defined(DARWIN)
 static void loadPlugin(char* dirName, char* pluginName) {
   char pluginPath[256];
   char tmpBuf[LEN_GENERAL_WORK_BUFFER];
   int i;
-#ifdef HPUX /* Courtesy Rusetsky Dmitry <dimania@mail.ru> */
-  shl_t pluginPtr;
-#else
 #ifndef WIN32
   void *pluginPtr = NULL;
-#endif
-#endif
-#ifndef WIN32
   void *pluginEntryFctnPtr;
 #endif
   PluginInfo* pluginInfo;
@@ -133,54 +49,26 @@ static void loadPlugin(char* dirName, char* pluginName) {
 #ifndef WIN32
   PluginInfo* (*pluginJumpFunc)();
 #endif
-  FlowFilterList *newFlow;
+  FlowFilterList *newFlow,
+                 *work,
+                 *prev;
 
-  if(snprintf(pluginPath, sizeof(pluginPath), "%s/%s", dirName != NULL ? dirName : ".", pluginName) < 0)
-    BufferTooShort();
+  safe_snprintf(__FILE__, __LINE__, pluginPath, sizeof(pluginPath), "%s/%s", dirName != NULL ? dirName : ".", pluginName);
 
   traceEvent(CONST_TRACE_NOISY, "Loading plugin '%s'", pluginPath);
 
 #ifndef MAKE_STATIC_PLUGIN
-#ifdef HPUX /* Courtesy Rusetsky Dmitry <dimania@mail.ru> */
-  /* Load the library */
-  pluginPtr = shl_load(pluginPath, BIND_IMMEDIATE|BIND_VERBOSE|BIND_NOSTART ,0L);
-#else
-#ifdef AIX
-  pluginPtr = load(pluginName, 1, dirName); /* Load the library */
-#else
   pluginPtr = (void*)dlopen(pluginPath, RTLD_NOW /* RTLD_LAZY */); /* Load the library */
-#endif /* AIX */
-#endif /* HPUX  */
 
   if(pluginPtr == NULL) {
     traceEvent(CONST_TRACE_WARNING, "Unable to load plugin '%s'", pluginPath);
-    traceEvent(CONST_TRACE_WARNING, "Message is '%s'", 
-#if HPUX /* Courtesy Rusetsky Dmitry <dimania@mail.ru> */
-	                            strerror(errno)
-#else
-	                            dlerror()
-#endif
-              );
+    traceEvent(CONST_TRACE_WARNING, "Message is '%s'", dlerror());
     return;
   }
 
-#ifdef HPUX /* Courtesy Rusetsky Dmitry <dimania@mail.ru> */
-  if(shl_findsym(&pluginPtr ,CONST_PLUGIN_ENTRY_FCTN_NAME,
-		 TYPE_PROCEDURE, &pluginEntryFctnPtr) == -1)
-    pluginEntryFctnPtr = NULL;
-#else
-#ifdef AIX
-  pluginEntryFctnPtr = pluginPtr;
-#else
-  pluginEntryFctnPtr = (void*)dlsym(pluginPtr, CONST_PLUGIN_ENTRY_FCTN_NAME);
-#endif /* AIX */
-#endif /* HPUX */
+  pluginEntryFctnPtr = (void*)dlsym(pluginPtr, "PluginEntryFctn" /* CONST_PLUGIN_ENTRY_FCTN_NAME */);
 
   if(pluginEntryFctnPtr == NULL) {
-#ifdef HPUX /* Courtesy Rusetsky Dmitry <dimania@mail.ru> */
-    traceEvent(CONST_TRACE_WARNING, "Unable to locate plugin '%s' entry function [%s]",
-	       pluginPath, strerror(errno));
-#else
 #ifdef WIN32
     traceEvent(CONST_TRACE_WARNING, "Unable to locate plugin '%s' entry function [%li]", 
 	       pluginPath, GetLastError());
@@ -188,7 +76,6 @@ static void loadPlugin(char* dirName, char* pluginName) {
     traceEvent(CONST_TRACE_WARNING, "Unable to locate plugin '%s' entry function [%s]",
 	       pluginPath, dlerror());
 #endif /* WIN32 */
-#endif /* HPUX */
     return;
   }
 
@@ -198,12 +85,14 @@ static void loadPlugin(char* dirName, char* pluginName) {
 
   if(strcmp(pluginName, "icmpPlugin") == 0)
     pluginInfo = icmpPluginEntryFctn();
-  else if(strcmp(pluginName, "nfsPlugin") == 0)
-    pluginInfo = nfsPluginEntryFctn();
   else if(strcmp(pluginName, "sflowPlugin") == 0)
     pluginInfo = sflowPluginEntryFctn();
   else if(strcmp(pluginName, "netflowPlugin") == 0)
     pluginInfo = netflowPluginEntryFctn();
+#ifdef HAVE_SNMP
+  else if(strcmp(pluginName, "snmpPlugin") == 0)
+    pluginInfo = snmpPluginEntryFctn();
+#endif
 #ifndef EMBEDDED
   else if(strcmp(pluginName, "rrdPlugin") == 0)
     pluginInfo = rrdPluginEntryFctn();
@@ -232,7 +121,7 @@ static void loadPlugin(char* dirName, char* pluginName) {
   
   if(newFlow == NULL) {
     traceEvent(CONST_TRACE_FATALERROR, "Not enough memory for plugin flow filter - aborting");
-    exit(-1);
+    exit(42); /* Just in case */
   } else {
     newFlow->fcode = (struct bpf_program*)calloc(MAX_NUM_DEVICES, sizeof(struct bpf_program));
     newFlow->flowName = strdup(pluginInfo->pluginName);
@@ -248,8 +137,10 @@ static void loadPlugin(char* dirName, char* pluginName) {
       tmpBuf[sizeof(tmpBuf)-1] = '\0'; /* just in case bpfFilter is too long... */
       
       for(i=0; i<myGlobals.numDevices; i++) 
-	if(!myGlobals.device[i].virtualDevice && 
-           !myGlobals.device[i].dummyDevice) {
+	if((!myGlobals.device[i].virtualDevice)
+	   && (!myGlobals.device[i].dummyDevice)
+	   && (myGlobals.device[i].pcapPtr)
+	   ) {
 	  traceEvent(CONST_TRACE_NOISY, "Compiling filter '%s' on interface %s", 
 		     tmpBuf, myGlobals.device[i].name);
 	  rc = pcap_compile(myGlobals.device[i].pcapPtr, 
@@ -275,8 +166,7 @@ static void loadPlugin(char* dirName, char* pluginName) {
 #endif
     newFlow->pluginStatus.pluginPtr       = pluginInfo;
 
-    if(snprintf(key, sizeof(key), "pluginStatus.%s", pluginInfo->pluginName) < 0)
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, key, sizeof(key), "pluginStatus.%s", pluginInfo->pluginName);
 
     if(fetchPrefsValue(key, value, sizeof(value)) == -1) {
       storePrefsValue(key, pluginInfo->activeByDefault ? "1" : "0");
@@ -288,8 +178,25 @@ static void loadPlugin(char* dirName, char* pluginName) {
 	newFlow->pluginStatus.activePlugin = 0;
     }
 
-    newFlow->next = myGlobals.flowsList;
-    myGlobals.flowsList = newFlow;
+    /* Find where to insert */
+    if((work = myGlobals.flowsList) == NULL) {
+      myGlobals.flowsList = newFlow;
+    } else {
+      while((work != NULL) && (strcasecmp(newFlow->flowName, work->flowName) > 0)) {
+        prev = work;
+        work = work->next;
+      }
+      if (work == myGlobals.flowsList) {
+        /* 1st in chain */
+        newFlow->next = myGlobals.flowsList;
+        myGlobals.flowsList = newFlow;
+      } else {
+        /* Insert in chain */
+        newFlow->next = prev->next;
+        prev->next = newFlow;
+      }
+    }
+
     /* traceEvent(CONST_TRACE_INFO, "Adding: %s", pluginInfo->pluginName); */
   }
 
@@ -310,8 +217,7 @@ void loadPlugins(void) {
   
 #ifndef MAKE_STATIC_PLUGIN
   for(idx=0; myGlobals.pluginDirs[idx] != NULL; idx++) {
-    if(snprintf(dirPath, sizeof(dirPath), "%s", myGlobals.pluginDirs[idx]) < 0) 
-      BufferTooShort();
+    safe_snprintf(__FILE__, __LINE__, dirPath, sizeof(dirPath), "%s", myGlobals.pluginDirs[idx]);
 
     directoryPointer = opendir(dirPath);
 
@@ -341,7 +247,6 @@ void loadPlugins(void) {
   closedir(directoryPointer);
 #else /* MAKE_STATIC_PLUGIN */
   loadPlugin(NULL, "icmpPlugin");
-  loadPlugin(NULL, "nfsPlugin");
   loadPlugin(NULL, "sflowPlugin");
   loadPlugin(NULL, "netflowPlugin");
 #ifndef EMBEDDED
@@ -365,22 +270,14 @@ void unloadPlugins(void) {
 #endif
       if((flows->pluginStatus.pluginPtr->termFunct != NULL)
 	 && (flows->pluginStatus.activePlugin))
-	flows->pluginStatus.pluginPtr->termFunct();
+	flows->pluginStatus.pluginPtr->termFunct(1 /* term ntop */);
 
 #ifndef MAKE_STATIC_PLUGIN
-#ifdef HPUX /* Courtesy Rusetsky Dmitry <dimania@mail.ru> */
-      shl_unload((shl_t)flows->pluginStatus.pluginMemoryPtr);
-#else
 #ifdef WIN32
       FreeLibrary((HANDLE)flows->pluginStatus.pluginMemoryPtr);
-#else
-#ifdef AIX
-      unload(flows->pluginStatus.pluginMemoryPtr);
 #else
       dlclose(flows->pluginStatus.pluginMemoryPtr);
-#endif /* AIX */
 #endif /* WIN32 */
-#endif /* HPUX */
 #endif
       flows->pluginStatus.pluginPtr       = NULL;
       flows->pluginStatus.pluginMemoryPtr = NULL;
@@ -416,6 +313,19 @@ void startPlugins(void) {
   }
 }
 
-/* ************************************* */
+/* ******************* */
 
+void handlePluginHostCreationDeletion(HostTraffic *el, u_short deviceId, u_char hostCreation) {
+  FlowFilterList *flows = myGlobals.flowsList;
+
+  while(flows != NULL) {
+    if(flows->pluginStatus.pluginMemoryPtr != NULL) {
+      if(flows->pluginStatus.activePlugin 
+	 && (flows->pluginStatus.pluginPtr->crtDltFunct != NULL))
+	flows->pluginStatus.pluginPtr->crtDltFunct(el, deviceId, hostCreation);
+    }
+    
+    flows = flows->next;
+  }
+}
 
