@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -48,7 +48,7 @@ struct id_rdiscovery {
 
 /* F o r w a r d */
 
-static void termIcmpFunct(void);
+static void termIcmpFunct(u_char);
 static void handleIcmpWatchHTTPrequest(char* url);
 static void printICMPdata(int icmpColumnSort, u_int revertOrder, u_int num, HostTraffic **hosts);
 
@@ -59,20 +59,23 @@ static void printICMPdata(int icmpColumnSort, u_int revertOrder, u_int num, Host
 
 static PluginInfo icmpPluginInfo[] = {
   { VERSION, /* current ntop version */
-    "icmpWatchPlugin",
-    "This plugin produces a report about the ICMP packets that ntop has seen. "
+    "ICMP Watch",
+    "This plugin produces a report about the ICMP packets that ntop has seen.<br>"
     "The report includes each host, byte and per-type counts (sent/received).",
-    "2.4", /* version */
+    "2.4a", /* version */
     "<A HREF=\"http://luca.ntop.org/\" alt=\"Luca's home page\">L.Deri</A>",
     "icmpWatch", /* http://<host>:<port>/plugins/icmpWatch */
     0, /* Active by default */
+    ViewOnly,
     0, /* Inactive setup */
     NULL, /* no special startup after init */
     termIcmpFunct, /* TermFunc   */
     NULL, /* PluginFunc */
     handleIcmpWatchHTTPrequest,
+    NULL, /* no host creation/deletion handle */
     NULL /* no capture */,
-    NULL /* no status */
+    NULL, /* no status */
+    NULL /* no extra pages */
   }
 };
 
@@ -118,7 +121,6 @@ static int sortICMPhostsRcvd(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsRcvd() (1)");
@@ -142,7 +144,6 @@ static int sortICMPhostsEcho(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsEcho() (1)");
@@ -169,7 +170,6 @@ static int sortICMPhostsReply(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsReply() (1)");
@@ -196,7 +196,6 @@ static int sortICMPhostsUnreach(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsUnreach() (1)");
@@ -223,7 +222,6 @@ static int sortICMPhostsRedirect(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsRedirect() (1)");
@@ -250,7 +248,6 @@ static int sortICMPhostsAdvert(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsAdvert() (1)");
@@ -277,7 +274,6 @@ static int sortICMPhostsTimeout(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsTimeout() (1)");
@@ -304,7 +300,6 @@ static int sortICMPhostsBadParam(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsBadParam() (1)");
@@ -331,7 +326,6 @@ static int sortICMPhostsQuench(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsQuench() (1)");
@@ -358,7 +352,6 @@ static int sortICMPhostsTimestamp(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsTimestamp() (1)");
@@ -389,7 +382,6 @@ static int sortICMPhostsInfo(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsInfo() (1)");
@@ -420,7 +412,6 @@ static int sortICMPhostsNetmask(const void *_a, const void *_b) {
   HostTraffic **a = (HostTraffic **)_a;
   HostTraffic **b = (HostTraffic **)_b;
   Counter n1, n2;
-  int rc;
 
   if(((*a) == NULL) && ((*b) != NULL)) {
     traceEvent(CONST_TRACE_WARNING, "sortICMPhostsNetmask() (1)");
@@ -489,10 +480,9 @@ static void formatSentRcvd(Counter sent, Counter rcvd) {
 
   if (sent + rcvd == 0) {
     strcpy(buf, "<TD "TD_BG" ALIGN=center>&nbsp;</TD>");
-  } else if(snprintf(buf, sizeof(buf), "<TD "TD_BG" ALIGN=center>%s/%s</TD>",
+  } else safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<TD "TD_BG" ALIGN=center>%s/%s</TD>",
 		     formatPkts(sent, formatBuf, sizeof(formatBuf)),
-		     formatPkts(rcvd, formatBuf1, sizeof(formatBuf1))) < 0)
-    BufferTooShort();
+		     formatPkts(rcvd, formatBuf1, sizeof(formatBuf1)));
   sendString(buf);
 }
 
@@ -522,16 +512,15 @@ static void printICMPdata(int icmpColumnSort, u_int revertOrder,
     if(abs(icmpColumnSort) == i) arrow[i] = arrowGif; else arrow[i] = "";
 
   sendString("<CENTER>\n<TABLE BORDER=1 "TABLE_DEFAULTS">\n");
-  if(snprintf(buf, sizeof(buf), "<TR "TR_ON" "DARK_BG">"
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<TR "TR_ON" "DARK_BG">"
               "<TH "TH_BG" rowspan=\"2\" valign=\"bottom\">%s?%s%d>Host %s</A></TH>\n"
 	      "<TH "TH_BG" colspan=\"2\">Bytes</TH>\n"
               "<TH "TH_BG" colspan=\"11\">Sent/Recived by ICMP Type</TH>\n"
 	      "</TR>\n",
-	      pluginName, sign, CONST_ICMP_SORT_HOST, arrow[CONST_ICMP_SORT_HOST]) < 0)
-    BufferTooShort();
+	      pluginName, sign, CONST_ICMP_SORT_HOST, arrow[CONST_ICMP_SORT_HOST]);
   sendString(buf);
 
-  if(snprintf(buf, sizeof(buf), "<TR "TR_ON" "DARK_BG">"
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<TR "TR_ON" "DARK_BG">"
 	      "<TH "TH_BG">%s?%s%d>Sent %s</A></TH>\n"
 	      "<TH "TH_BG">%s?%s%d>Rcvd %s</A></TH>\n"
 	      "<TH "TH_BG">%s?%s%d>Echo<br>Request %s</A></TH>\n"
@@ -558,8 +547,7 @@ static void printICMPdata(int icmpColumnSort, u_int revertOrder,
 	      pluginName, sign, CONST_ICMP_SORT_NETMASK, arrow[CONST_ICMP_SORT_NETMASK],
 	      pluginName, sign, CONST_ICMP_SORT_QUENCH, arrow[CONST_ICMP_SORT_QUENCH],
 	      pluginName, sign, CONST_ICMP_SORT_TIMESTAMP, arrow[CONST_ICMP_SORT_TIMESTAMP],
-	      pluginName, sign, CONST_ICMP_SORT_INFO, arrow[CONST_ICMP_SORT_INFO]) < 0)
-    BufferTooShort();
+	      pluginName, sign, CONST_ICMP_SORT_INFO, arrow[CONST_ICMP_SORT_INFO]);
   sendString(buf);
 
   qsort(hosts, num, sizeof(HostTraffic **), cmpFctnICMP[icmpColumnSort]);
@@ -574,21 +562,18 @@ static void printICMPdata(int icmpColumnSort, u_int revertOrder,
       else
 	idx = i;
 
-      if(snprintf(buf, sizeof(buf), "<TR "TR_ON" %s> %s",
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<TR "TR_ON" %s> %s",
 		  getRowColor(),
 		  makeHostLink(hosts[idx], FLAG_HOSTLINK_HTML_FORMAT, 0, 0,
-			       hostLinkBuf, sizeof(hostLinkBuf))) < 0)
-	BufferTooShort();
+			       hostLinkBuf, sizeof(hostLinkBuf)));
       sendString(buf);
 
-      if(snprintf(buf, sizeof(buf), "<TD "TD_BG" ALIGN=center>%s</TD>",
-		  formatBytes(hosts[idx]->icmpSent.value, 1, formatBuf, sizeof(formatBuf))) < 0)
-	BufferTooShort();
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<TD "TD_BG" ALIGN=center>%s</TD>",
+		  formatBytes(hosts[idx]->icmpSent.value, 1, formatBuf, sizeof(formatBuf)));
       sendString(buf);
 
-      if(snprintf(buf, sizeof(buf), "<TD "TD_BG" ALIGN=center>%s</TD>",
-		  formatBytes(hosts[idx]->icmpRcvd.value, 1, formatBuf, sizeof(formatBuf))) < 0)
-	BufferTooShort();
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<TD "TD_BG" ALIGN=center>%s</TD>",
+		  formatBytes(hosts[idx]->icmpRcvd.value, 1, formatBuf, sizeof(formatBuf)));
       sendString(buf);
 
       formatSentRcvd((Counter)(hosts[idx]->icmpInfo->icmpMsgSent[ICMP_ECHO].value),
@@ -629,7 +614,7 @@ static void printICMPdata(int icmpColumnSort, u_int revertOrder,
       sendString("</TR>\n");
 
       /* Avoid huge tables */
-      if(printedEntries++ > myGlobals.maxNumLines)
+      if(printedEntries++ > myGlobals.runningPref.maxNumLines)
 	break;
     }
 
@@ -640,7 +625,6 @@ static void printICMPdata(int icmpColumnSort, u_int revertOrder,
 /* ******************************* */
 
 static void handleIcmpWatchHTTPrequest(char* url) {
-  char fileName[NAME_MAX] = "/tmp/ntop-icmpPlugin-XXXXXX";
   u_int i, revertOrder=0, num, tot = 0;
   int icmpId=-1;
   HostTraffic **hosts;
@@ -649,6 +633,9 @@ static void handleIcmpWatchHTTPrequest(char* url) {
   float *s, *r;
   FILE *fd;
   int icmpColumnSort = 0;
+#ifdef WIN32
+  char fileName[256];
+#endif
 
   i = sizeof(float)*myGlobals.device[myGlobals.actualReportDeviceId].actualHashSize;
   s = (float*)malloc(i); r = (float*)malloc(i);
@@ -696,10 +683,9 @@ static void handleIcmpWatchHTTPrequest(char* url) {
 #ifndef EMBEDDED
 #ifdef CFG_USE_GRAPHICS
     if(strncmp(url, "chart", strlen("chart")) == 0) {
-      unsigned long  sc[2] = { 0xf08080L, 0x4682b4L }; /* see clr[] in graph.c */
 
       /* Avoid to draw too many entries */
-      if(num > myGlobals.maxNumLines) num = myGlobals.maxNumLines;
+      if(num > myGlobals.runningPref.maxNumLines) num = myGlobals.runningPref.maxNumLines;
 
       if(icmpColumnSort<0) icmpColumnSort=0;
       if(icmpColumnSort>cmpFctnICMPmax) icmpColumnSort=0;
@@ -791,7 +777,7 @@ static void handleIcmpWatchHTTPrequest(char* url) {
 
 /* ****************************** */
 
-static void termIcmpFunct(void) {
+static void termIcmpFunct(u_char termNtop /* 0=term plugin, 1=term ntop */) {
   traceEvent(CONST_TRACE_INFO, "ICMP: Thanks for using icmpWatch"); fflush(stdout);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ICMP: Done"); fflush(stdout);
 }
@@ -804,7 +790,7 @@ PluginInfo* icmpPluginEntryFctn(void) {
 #else
   PluginInfo* PluginEntryFctn(void) {
 #endif
-    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ICMP: Welcome to %s. (C) 1999-2004 by Luca Deri",
+    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ICMP: Welcome to %s. (C) 1999-2005 by Luca Deri",
 	       icmpPluginInfo->pluginName);
 
     return(icmpPluginInfo);
