@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -52,13 +52,14 @@ void handleBootp(HostTraffic *srcHost,
 		 int actualDeviceId) {
   BootProtocol bootProto;
   u_int len;
+  int rc;
   char savechar; /* Courtesy of
 		    Axel Thimm <Axel.Thimm+ntop@physik.fu-berlin.de>
 		 */
 
-  if((!myGlobals.enablePacketDecoding)
+  if((!myGlobals.runningPref.enablePacketDecoding)
      || (packetData == NULL) /* packet too short ? */
-     || (myGlobals.dontTrustMACaddr))
+     || (myGlobals.runningPref.dontTrustMACaddr))
     return;
 
   memset(&bootProto, 0, sizeof(BootProtocol));
@@ -124,9 +125,10 @@ void handleBootp(HostTraffic *srcHost,
 	      dstHost is a broadcast address
 	    */
 
-	    realDstHost = findHostByMAC(bootProto.bp_chaddr, actualDeviceId);
+	    realDstHost = findHostByMAC((char*)bootProto.bp_chaddr, dstHost->vlanId, actualDeviceId);
 	    if(realDstHost == NULL) {
-	      realDstHost = lookupHost(/* &bootProto.bp_yiaddr */ NULL, bootProto.bp_chaddr, 0, 0, actualDeviceId);
+	      realDstHost = lookupHost(/* &bootProto.bp_yiaddr */ NULL, bootProto.bp_chaddr, 
+				       srcHost->vlanId, 0, 0, actualDeviceId);
 	    } else {
 #ifdef DHCP_DEBUG
 	      traceEvent(CONST_TRACE_INFO, "<<=>> %s (%d)",
@@ -166,7 +168,7 @@ void handleBootp(HostTraffic *srcHost,
 		strncpy(realDstHost->hostNumIpAddress,
 			_intoa(realDstHost->hostIp4Address, buf, sizeof(buf)),
 			sizeof(realDstHost->hostNumIpAddress));
-		if(myGlobals.numericFlag == 0) ipaddr2str(realDstHost->hostIpAddress, 1);
+		if(myGlobals.runningPref.numericFlag == 0) ipaddr2str(realDstHost->hostIpAddress, 1);
                 if (realDstHost->dnsDomainValue != NULL) free(realDstHost->dnsDomainValue);
 		realDstHost->dnsDomainValue = NULL;
                 if (realDstHost->dnsTLDValue != NULL) free(realDstHost->dnsTLDValue);
@@ -205,7 +207,7 @@ void handleBootp(HostTraffic *srcHost,
 #endif
 		  /* *************** */
 		  addr.Ip4Address.s_addr = hostIpAddress.s_addr;
-		  trafficHost = findHostByNumIP(addr, actualDeviceId);
+		  trafficHost = findHostByNumIP(addr, srcHost->vlanId, actualDeviceId);
 		  if(trafficHost != NULL) {
 		    incrementUsageCounter(&realDstHost->contactedRouters, trafficHost, actualDeviceId);
 		    FD_SET(FLAG_GATEWAY_HOST, &trafficHost->flags);
@@ -246,14 +248,13 @@ void handleBootp(HostTraffic *srcHost,
 		    tmpHostName[i] = '\0';
 
 		    /* Fix courtesy of Christoph Zens <chris@topfen.homeip.net> */
-		    strncpy(tmpDomainName, &bootProto.bp_vend[idx], len);
+		    strncpy(tmpDomainName, (char*)&bootProto.bp_vend[idx], len);
 		    tmpDomainName[len] = '\0';
 
 		    if(strcmp(tmpHostName, tmpDomainName) != 0) {
-		      if(snprintf(tmpName, sizeof(tmpName), "%s.%s",
-				  tmpHostName, tmpDomainName) < 0)
-			BufferTooShort();
-		      else {
+		      rc = safe_snprintf(__FILE__, __LINE__, tmpName, sizeof(tmpName), "%s.%s",
+				  tmpHostName, tmpDomainName);
+		      if (rc >= 0) {
 			hostLen = len;
 			len = strlen(tmpName);
 
@@ -305,7 +306,7 @@ void handleBootp(HostTraffic *srcHost,
 		  /* *************** */
 		  addr.Ip4Address.s_addr = hostIpAddress.s_addr;
 
-		  trafficHost = findHostByNumIP(addr, actualDeviceId);
+		  trafficHost = findHostByNumIP(addr, srcHost->vlanId, actualDeviceId);
 		  if(trafficHost != NULL) {
 		    FD_SET(FLAG_HOST_TYPE_SVC_WINS, &trafficHost->flags);
 		  }
@@ -468,9 +469,10 @@ void handleBootp(HostTraffic *srcHost,
 	      dstHost is a broadcast address
 	    */
 
-	    realClientHost = findHostByMAC(bootProto.bp_chaddr, actualDeviceId);
+	    realClientHost = findHostByMAC((char*)bootProto.bp_chaddr, srcHost->vlanId, actualDeviceId);
 	    if(realClientHost == NULL) {
-	      realClientHost = lookupHost(/*&bootProto.bp_yiaddr*/ NULL, bootProto.bp_chaddr, 0, 0, actualDeviceId);
+	      realClientHost = lookupHost(/*&bootProto.bp_yiaddr*/ NULL, bootProto.bp_chaddr,
+					  srcHost->vlanId, 0, 0, actualDeviceId);
 	    } else {
 #ifdef FLAG_DHCP_DEBUG
 	      traceEvent(CONST_TRACE_INFO, "<<=>> %s (%d)",
@@ -505,7 +507,7 @@ void handleBootp(HostTraffic *srcHost,
 		    len = MAX_LEN_SYM_HOST_NAME-1;
 		  }
 
-                  setResolvedName(realClientHost, &bootProto.bp_vend[idx], FLAG_HOST_SYM_ADDR_TYPE_NAME);
+                  setResolvedName(realClientHost, (char*)&bootProto.bp_vend[idx], FLAG_HOST_SYM_ADDR_TYPE_NAME);
 		  realClientHost->hostResolvedName[len] = '\0';
 		  bootProto.bp_vend[idx+len] = savechar;
 		  idx += len;
@@ -567,7 +569,8 @@ void handleBootp(HostTraffic *srcHost,
 
 /* ************************************ */
 
-u_int16_t processDNSPacket(const u_char *packetData,
+u_int16_t processDNSPacket(HostTraffic *srcHost, u_short sport,
+			   const u_char *packetData,
 			   u_int length,
 			   short *isRequest,
 			   short *positiveReply) {
@@ -581,14 +584,15 @@ u_int16_t processDNSPacket(const u_char *packetData,
 
   if(myGlobals.dnsCacheFile == NULL) return(-1); /* ntop is quitting... */
 
-  if((!myGlobals.enablePacketDecoding)
+  if((!myGlobals.runningPref.enablePacketDecoding)
      ||(packetData == NULL) /* packet too short ? */)
     return(transactionId);
 
   myGlobals.dnsSniffCount++;
   memset(&hostPtr, 0, sizeof(DNSHostInfo));
 
-  transactionId = handleDNSpacket(packetData, &hostPtr, length,
+  transactionId = handleDNSpacket(srcHost, sport,
+				  packetData, &hostPtr, length,
 				  isRequest, positiveReply);
 
 #ifdef DNS_SNIFF_DEBUG
@@ -624,24 +628,25 @@ u_int16_t processDNSPacket(const u_char *packetData,
   }
 
   for(i=0; i<MAX_ADDRESSES; i++) {
-  	/* Symbolic => Numeric */
+    /* Symbolic => Numeric */
 
     if(hostPtr.addrList[i] != 0) {
-      StoredAddress storedAddress;
+      StoredAddress addrStore;
+      int len;
 
-      memset(&storedAddress, 0, sizeof(storedAddress));
-      storedAddress.recordCreationTime = myGlobals.actTime;
-      memcpy(&storedAddress.symAddress,
-             hostPtr.queryName,
-             min(MAX_LEN_SYM_HOST_NAME-1, strlen(hostPtr.queryName)));
-      storedAddress.symAddressType=FLAG_HOST_SYM_ADDR_TYPE_NAME;
+      memset(&addrStore, 0, sizeof(addrStore));
+      addrStore.recordCreationTime = myGlobals.actTime;
+      len = min(sizeof(addrStore.symAddress)-1, strlen(hostPtr.queryName));
+      memcpy(&addrStore.symAddress,
+             hostPtr.queryName, len);
+      addrStore.symAddress[len] = '\0';
+      addrStore.symAddressType = FLAG_HOST_SYM_ADDR_TYPE_NAME;
 
-      if(snprintf(tmpBuf, sizeof(tmpBuf), "%u", ntohl(hostPtr.addrList[i])) < 0)
-         BufferTooShort();
+      safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%u", ntohl(hostPtr.addrList[i]));
       key_data.dptr = (void*)&tmpBuf;
       key_data.dsize = strlen(key_data.dptr)+1;
-      data_data.dptr = (void*)&storedAddress;
-      data_data.dsize = sizeof(storedAddress)+1;
+      data_data.dptr = (void*)&addrStore;
+      data_data.dsize = sizeof(addrStore);
 
 #ifdef DNS_SNIFF_DEBUG
       traceEvent(CONST_TRACE_INFO, "DNS_SNIFF_DEBUG: Sniffed DNS response: %s(%d) = %s(t=%d)",
@@ -669,13 +674,14 @@ void handleNetbios(HostTraffic *srcHost,
 		   const u_char* packetData,
 		   u_int length,
 		   u_int hlen) {
-  u_char *name, nbName[64], domain[64], *data;
+  u_char *data, *name, *p;
   int nodeType, i, udpDataLen;
   char *tmpdata = (char*)packetData + (hlen + sizeof(struct udphdr));
-  u_char *p;
+  char nbName[64], domain[64];
   int offset=0, displ, notEnoughData = 0;
 
-  if((!myGlobals.enablePacketDecoding) || (srcHost->nonIPTraffic != NULL) /* Already set */
+  if((!myGlobals.runningPref.enablePacketDecoding) ||
+     (srcHost->nonIPTraffic != NULL) /* Already set */
      || (packetData == NULL)) /* packet too short ? */
     return;
   
@@ -688,11 +694,11 @@ void handleNetbios(HostTraffic *srcHost,
 
       opcode = (tmpdata[2] >> 3) & 0x0F;
 
-      data = (char*)malloc(udpDataLen);
+      data = (u_char*)malloc(udpDataLen);
       memcpy(data, tmpdata, udpDataLen);
 
       name = data + 12;
-      p = (u_char*)name;
+      p = name;
       if ((*p & 0xC0) == 0xC0) {
 	displ = p[1] + 255 * (p[0] & ~0xC0);
 	if((displ + 14) >= udpDataLen)
@@ -718,7 +724,7 @@ void handleNetbios(HostTraffic *srcHost,
 
       if(!notEnoughData) {      
 	u_int8_t doDecode = 0;
-	nodeType = name_interpret(name, nbName, udpDataLen-displ);
+	nodeType = name_interpret((char*)name, nbName, udpDataLen-displ);
 
 	switch(opcode) {
 	case 0: /* Query */
@@ -750,11 +756,11 @@ void handleNetbios(HostTraffic *srcHost,
   } else if(dport == 138 /*  NetBIOS */) {
     if(udpDataLen > 32) {
       /* 32 bytes or less is not enough */
-      data = (char*)malloc(udpDataLen);
+      data = (u_char*)malloc(udpDataLen);
       memcpy(data, tmpdata, udpDataLen);
 
       name = data + 14;
-      p = (u_char*)name;
+      p = name;
       if ((*p & 0xC0) == 0xC0) {
 	displ = p[1] + 255 * (p[0] & ~0xC0);
 	if((displ + 14) >= udpDataLen)
@@ -779,7 +785,7 @@ void handleNetbios(HostTraffic *srcHost,
       }
 
       if(!notEnoughData) {
-	nodeType = name_interpret(name, nbName, udpDataLen-displ);
+	nodeType = name_interpret((char*)name, nbName, udpDataLen-displ);
 
 	if(nodeType != -1) {
 	  setNBnodeNameType(srcHost, (char)nodeType, 0, nbName);
@@ -788,18 +794,18 @@ void handleNetbios(HostTraffic *srcHost,
 
 	  if(displ < udpDataLen) {
 	    name = data + offset; /* ** */
-	    p = (u_char*)name;
+	    p = name;
 	    if ((*p & 0xC0) == 0xC0) {
 	      displ = hlen + 8 + (p[1] + 255 * (p[0] & ~0xC0));
 
 	      if(displ < length)
-		name = ((char*)packetData+displ);
+		name = (u_char*)((char*)packetData+displ);
 	      else
 		notEnoughData = 1;
 	    }
 
 	    if(!notEnoughData) {
-	      nodeType = name_interpret(name, domain, length-displ);
+	      nodeType = name_interpret((char*)name, domain, length-displ);
 
 	      if(nodeType != -1) {
 		for(i=0; domain[i] != '\0'; i++)
@@ -808,7 +814,7 @@ void handleNetbios(HostTraffic *srcHost,
 		setNBnodeNameType(dstHost, (char)nodeType, 0, domain);
 
 		if(udpDataLen > 200) {
-		  char *tmpBuffer = &data[151];
+		  char *tmpBuffer = (char*)&data[151];
 
 		  /*
 		    We'll check if this this is
@@ -848,7 +854,7 @@ void handleNetbios(HostTraffic *srcHost,
 
     if(udpDataLen > 32) {
       /* 32 bytes or less is not enough */
-      data = (char*)malloc(udpDataLen);
+      data = (u_char*)malloc(udpDataLen);
       memcpy(data, tmpdata, udpDataLen);
 
       if(data[0] == 0x81) /* Session request */ {
@@ -856,7 +862,7 @@ void handleNetbios(HostTraffic *srcHost,
 	int pos;
 
 	pos = 5;
-	decodeNBstring(&data[5], decodedStr);
+	decodeNBstring((char*)&data[5], decodedStr);
 
 	if(srcHost->nonIPTraffic == NULL) srcHost->nonIPTraffic = (NonIPTraffic*)calloc(1, sizeof(NonIPTraffic));
 	if(dstHost->nonIPTraffic == NULL) dstHost->nonIPTraffic = (NonIPTraffic*)calloc(1, sizeof(NonIPTraffic));
@@ -865,7 +871,7 @@ void handleNetbios(HostTraffic *srcHost,
 	  dstHost->nonIPTraffic->nbHostName = strdup(decodedStr); /* dst before src */
 
 	pos = 5+(2*strlen(decodedStr))+2;
-	decodeNBstring(&data[pos], decodedStr);
+	decodeNBstring((char*)&data[pos], decodedStr);
 
 	if((decodedStr[0] != '\0') && (srcHost->nonIPTraffic->nbHostName == NULL))
 	  srcHost->nonIPTraffic->nbHostName = strdup(decodedStr);
@@ -885,8 +891,7 @@ void handleNetbios(HostTraffic *srcHost,
 	  if(srcHost->fingerprint == NULL) {
 	    char buffer[64];
 
-	    if(snprintf(buffer, sizeof(buffer), ":%s", &data[45]) < 0)
-              BufferTooShort();
+	    safe_snprintf(__FILE__, __LINE__, buffer, sizeof(buffer), ":%s", &data[45]);
 	    accessAddrResMutex("makeHostLink");
 	    srcHost->fingerprint = strdup(buffer);
 	    releaseAddrResMutex();
@@ -900,7 +905,7 @@ void handleNetbios(HostTraffic *srcHost,
 	  i = 65+len;
 
 	  if(srcHost->nonIPTraffic == NULL) srcHost->nonIPTraffic = (NonIPTraffic*)calloc(1, sizeof(NonIPTraffic));
-	  if(srcHost->nonIPTraffic->nbAccountName == NULL) srcHost->nonIPTraffic->nbAccountName = strdup(&data[i]);
+	  if(srcHost->nonIPTraffic->nbAccountName == NULL) srcHost->nonIPTraffic->nbAccountName = strdup((char*)&data[i]);
 #ifdef DEBUG
 	  printf("Account Name: %s\n", &data[i]);
 #endif
@@ -909,19 +914,16 @@ void handleNetbios(HostTraffic *srcHost,
 #ifdef DEBUG
 	  printf("Domain: %s\n", &data[i]);
 #endif
-	  if(srcHost->nonIPTraffic->nbDomainName == NULL) srcHost->nonIPTraffic->nbDomainName = strdup(&data[i]);
+	  if(srcHost->nonIPTraffic->nbDomainName == NULL) srcHost->nonIPTraffic->nbDomainName = strdup((char*)&data[i]);
 	  while((data[i] != 0) && (i < sizeof(data))) i++;
 	  i++;
 #ifdef DEBUG
 	  printf("OS: %s\n", &data[i]);
 #endif
-
-
 	  if(srcHost->fingerprint == NULL) {
 	    char buffer[64];
 
-	    if(snprintf(buffer, sizeof(buffer), ":%s", &data[i]) < 0)
-              BufferTooShort();
+	    safe_snprintf(__FILE__, __LINE__, buffer, sizeof(buffer), ":%s", &data[i]);
 	    accessAddrResMutex("makeHostLink");
 	    srcHost->fingerprint = strdup(buffer);
 	    releaseAddrResMutex();
