@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -42,8 +42,8 @@ void termIPServices(void) {
     }
   }
 
-  free(myGlobals.udpSvc);
-  free(myGlobals.tcpSvc);
+  if(myGlobals.udpSvc) free(myGlobals.udpSvc);
+  if(myGlobals.tcpSvc) free(myGlobals.tcpSvc);
 
   /* ****************************** */
 
@@ -90,9 +90,10 @@ void termIPSessions(void) {
 /* ************************************************ */
 
 void termGdbm() {
-    if(myGlobals.dnsCacheFile)     { gdbm_close(myGlobals.dnsCacheFile); myGlobals.dnsCacheFile = NULL;   }
-    if(myGlobals.addressQueueFile) { gdbm_close(myGlobals.addressQueueFile); myGlobals.addressQueueFile = NULL; }
-    if(myGlobals.pwFile)           { gdbm_close(myGlobals.pwFile); myGlobals.pwFile = NULL;               }
-    if(myGlobals.prefsFile)        { gdbm_close(myGlobals.prefsFile); myGlobals.prefsFile = NULL;         }
-    if(myGlobals.macPrefixFile)    { gdbm_close(myGlobals.macPrefixFile); myGlobals.macPrefixFile = NULL;         }
+  if(myGlobals.dnsCacheFile)     { gdbm_close(myGlobals.dnsCacheFile); myGlobals.dnsCacheFile = NULL; }
+  if(myGlobals.addressQueueFile) { gdbm_close(myGlobals.addressQueueFile); myGlobals.addressQueueFile = NULL; }
+  if(myGlobals.pwFile)           { gdbm_close(myGlobals.pwFile); myGlobals.pwFile = NULL; }
+  if(myGlobals.prefsFile)        { gdbm_close(myGlobals.prefsFile); myGlobals.prefsFile = NULL; }
+  if(myGlobals.macPrefixFile)    { gdbm_close(myGlobals.macPrefixFile); myGlobals.macPrefixFile = NULL; }
+  if(myGlobals.fingerprintFile)  { gdbm_close(myGlobals.fingerprintFile); myGlobals.fingerprintFile = NULL; }
 }
