@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2004 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2005 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -41,9 +41,14 @@ static void updateThptStats(int deviceToUpdate,
   int i;
   HostTraffic *topHost;
   float topThpt;
+#ifdef DEBUG  
+  char formatBuf[32];
+#endif  
 
+  /*
   if(myGlobals.device[deviceToUpdate].dummyDevice)
     return;
+  */
 
 #ifdef DEBUG
   traceEvent(CONST_TRACE_INFO, "updateThptStats(%d, %d, %d, %d, %d, %d)",
@@ -77,9 +82,10 @@ static void updateThptStats(int deviceToUpdate,
 
   myGlobals.device[deviceToUpdate].last60MinutesThpt[0].trafficValue = myGlobals.device[deviceToUpdate].lastMinThpt;
 
-#ifdef DEBUG
-  traceEvent(CONST_TRACE_INFO, "LastMinThpt: %s", formatThroughput(myGlobals.device[deviceToUpdate].lastMinThpt));
-#endif
+#ifdef DEBUG  
+  traceEvent (CONST_TRACE_ALWAYSDISPLAY, "LastMinThpt: %s",
+              formatThroughput(myGlobals.device[deviceToUpdate].lastMinThpt, 0, formatBuf, sizeof (formatBuf)));
+#endif  
 
   topHost = findHostBySerial(topSentSerial, deviceToUpdate); 
   if(topHost != NULL) topThpt = topHost->actualSentThpt; else topThpt = 0;
@@ -230,18 +236,18 @@ void updateDeviceThpt(int deviceToUpdate, int quickUpdate) {
   /* timeDiff++; */
   myGlobals.device[deviceToUpdate].actualThpt = (float)myGlobals.device[deviceToUpdate].throughput/(float)timeDiff;
   myGlobals.device[deviceToUpdate].actualPktsThpt = 
-    (float)myGlobals.device[deviceToUpdate].packetThroughput/(float)timeDiff;
-
+      (float)myGlobals.device[deviceToUpdate].packetThroughput/(float)timeDiff;
+  
   if(myGlobals.device[deviceToUpdate].actualThpt > myGlobals.device[deviceToUpdate].peakThroughput)
-    myGlobals.device[deviceToUpdate].peakThroughput = myGlobals.device[deviceToUpdate].actualThpt;
-
+      myGlobals.device[deviceToUpdate].peakThroughput = myGlobals.device[deviceToUpdate].actualThpt;
+  
   if(myGlobals.device[deviceToUpdate].actualPktsThpt > myGlobals.device[deviceToUpdate].peakPacketThroughput)
-    myGlobals.device[deviceToUpdate].peakPacketThroughput = myGlobals.device[deviceToUpdate].actualPktsThpt;
+      myGlobals.device[deviceToUpdate].peakPacketThroughput = myGlobals.device[deviceToUpdate].actualPktsThpt;
 
   myGlobals.device[deviceToUpdate].throughput = myGlobals.device[deviceToUpdate].ethernetBytes.value;
   myGlobals.device[deviceToUpdate].packetThroughput = myGlobals.device[deviceToUpdate].ethernetPkts.value;
 
-  if((timeMinDiff = myGlobals.actTime-myGlobals.device[deviceToUpdate].lastMinThptUpdate) > 60 /* 1 minute */) {
+  if((timeMinDiff = myGlobals.actTime-myGlobals.device[deviceToUpdate].lastMinThptUpdate) >= 60 /* 1 minute */) {
     updateMinThpt = 1;
     myGlobals.device[deviceToUpdate].lastMinEthernetBytes.value = myGlobals.device[deviceToUpdate].ethernetBytes.value -
       myGlobals.device[deviceToUpdate].lastMinEthernetBytes.value;
@@ -257,7 +263,7 @@ void updateDeviceThpt(int deviceToUpdate, int quickUpdate) {
     myGlobals.device[deviceToUpdate].lastMinThptUpdate = myGlobals.actTime;
   }
 
-  if((timeMinDiff = myGlobals.actTime-myGlobals.device[deviceToUpdate].lastFiveMinsThptUpdate) > 300 /* 5 minutes */) {
+  if((timeMinDiff = myGlobals.actTime-myGlobals.device[deviceToUpdate].lastFiveMinsThptUpdate) >= 300 /* 5 minutes */) {
     myGlobals.device[deviceToUpdate].lastFiveMinsEthernetBytes.value = 
       myGlobals.device[deviceToUpdate].ethernetBytes.value - myGlobals.device[deviceToUpdate].lastFiveMinsEthernetBytes.value;
     myGlobals.device[deviceToUpdate].lastFiveMinsThptUpdate = timeMinDiff;
@@ -278,12 +284,15 @@ void updateDeviceThpt(int deviceToUpdate, int quickUpdate) {
     myGlobals.device[deviceToUpdate].lastFiveMinsThptUpdate = myGlobals.actTime;
   }
 
-  if(quickUpdate) return;
+  if(quickUpdate) {
+      myGlobals.device[deviceToUpdate].lastThptUpdate = myGlobals.actTime;
+      return;
+  }
 
 #ifdef DEBUG
   traceEvent(CONST_TRACE_INFO, "updateDeviceStats() called.");
 #endif
-    
+  
   totalTime = myGlobals.actTime-myGlobals.initialSniffTime;
 
   if((timeHourDiff = myGlobals.actTime-myGlobals.
@@ -293,9 +302,8 @@ void updateDeviceThpt(int deviceToUpdate, int quickUpdate) {
   }
 
   for(el=getFirstHost(deviceToUpdate); el != NULL; el = getNextHost(deviceToUpdate, el)) {
-    if(broadcastHost(el)) {
+    if(!isFcHost (el) && broadcastHost(el))
       continue;
-    }
 
     el->actualRcvdThpt       = (float)(el->bytesRcvd.value-el->lastBytesRcvd.value)/timeDiff;
     if(el->peakRcvdThpt      < el->actualRcvdThpt) el->peakRcvdThpt = el->actualRcvdThpt;
@@ -344,8 +352,8 @@ void updateDeviceThpt(int deviceToUpdate, int quickUpdate) {
 	  }
 	} else {
 	    if(emptySerial(&secondSentSerial)) {
-	    if((el != myGlobals.broadcastEntry) && (el != myGlobals.otherHostEntry))
-	      secondSentSerial = el->hostSerial;
+                if((el != myGlobals.broadcastEntry) && (el != myGlobals.otherHostEntry))
+                    secondSentSerial = el->hostSerial;
 	  } else {
 	    topHost = findHostBySerial(secondSentSerial, deviceToUpdate); 
 	    if(topHost != NULL) topThpt = topHost->actualSentThpt; else topThpt = 0;
@@ -517,7 +525,7 @@ void updateThpt(int fullUpdate) {
   traceEvent(CONST_TRACE_INFO, "updateThpt() called");
 #endif
 
-  if(myGlobals.mergeInterfaces)
+  if(myGlobals.runningPref.mergeInterfaces)
     updateDeviceThpt(0, !fullUpdate);
   else {
     for(i=0; i<myGlobals.numDevices; i++)
@@ -529,7 +537,8 @@ void updateThpt(int fullUpdate) {
 
 /* Check if a host can be potentially added the host matrix */
 int isMatrixHost(HostTraffic *host, int actualDeviceId) {
-  if((deviceLocalAddress(&host->hostIpAddress, actualDeviceId) || multicastHost(host))
+  if((host->hostIpAddress.hostFamily == AF_INET) /* Only IPv4 addresses are used in the matrix */
+     && (deviceLocalAddress(&host->hostIpAddress, actualDeviceId) || multicastHost(host))
      && (!broadcastHost(host)))
     return(1);
   else
@@ -543,48 +552,47 @@ unsigned int matrixHostHash(HostTraffic *host, int actualDeviceId, int rehash) {
   char tmpBuf[80], *str;
   int c;
 
-    if (host->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) {
-        if (host->hostIpAddress.hostFamily == AF_INET)
-            hash = host->hostIp4Address.s_addr;
+  if(myGlobals.device[actualDeviceId].numHosts  == 0) return(0);
+
+  if (host->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) {
+    if (host->hostIpAddress.hostFamily == AF_INET)
+      hash = host->hostIp4Address.s_addr;
 #ifdef INET6
-        else if (host->hostIpAddress.hostFamily == AF_INET6)
-            hash = *(u_int32_t *)&host->hostIp6Address.s6_addr[0];
+    else if (host->hostIpAddress.hostFamily == AF_INET6)
+      hash = *(u_int32_t *)&host->hostIp6Address.s6_addr[0];
 #endif
+  }
+  else {
+    if (host->fcCounters->vsanId) {
+      hash ^= host->fcCounters->vsanId;
+      hash ^= host->fcCounters->hostFcAddress.domain;
+      hash ^= host->fcCounters->hostFcAddress.area;
+      hash ^= host->fcCounters->hostFcAddress.port;
+      safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof (tmpBuf), "%x.%x.%x.%x.%x", host->fcCounters->vsanId, 
+		  host->fcCounters->hostFcAddress.domain, host->fcCounters->hostFcAddress.area,
+		  host->fcCounters->hostFcAddress.port, hash);
+    } else {
+      safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof (tmpBuf), "%x.%x.%x.%x",
+		   host->fcCounters->hostFcAddress.domain, host->fcCounters->hostFcAddress.area,
+		   host->fcCounters->hostFcAddress.port, host);
     }
-    else {
-        if (host->vsanId) {
-            hash ^= host->vsanId;
-            hash ^= host->hostFcAddress.domain;
-            hash ^= host->hostFcAddress.area;
-            hash ^= host->hostFcAddress.port;
-            if(snprintf (tmpBuf, sizeof (tmpBuf), "%x.%x.%x.%x.%x", host->vsanId, 
-                      host->hostFcAddress.domain, host->hostFcAddress.area,
-                      host->hostFcAddress.port, hash) < 0)
-              BufferTooShort();
-        }
-        else {
-            if(snprintf (tmpBuf, sizeof (tmpBuf), "%x.%x.%x.%x",
-                      host->hostFcAddress.domain, host->hostFcAddress.area,
-                      host->hostFcAddress.port, host) < 0)
-              BufferTooShort();
-        }
-        str = tmpBuf;
-
-        /* sdbm hash algorithm */
-        hash = 0;
-        while (c = *str++) {
-            hash = c + (hash << 6) + (hash << 16) - hash;
-        }
-
-        /* Assuming that the numHosts for FC is always 1024, 1021 is nearest
-         * prime */
-        if (rehash) {
-            c = (5 - hash%5);
-            hash += c;
-        }
+    str = tmpBuf;
+	
+    /* sdbm hash algorithm */
+    hash = 0;
+    while((c = *str++)) {
+      hash = c + (hash << 6) + (hash << 16) - hash;
     }
+
+    /* Assuming that the numHosts for FC is always 1024, 1021 is nearest
+     * prime */
+    if (rehash) {
+      c = (5 - hash%5);
+      hash += c;
+    }
+  }
     
-    return((unsigned int)(hash) % myGlobals.device[actualDeviceId].numHosts);
+  return((unsigned int)(hash) % myGlobals.device[actualDeviceId].numHosts);
 }
 
 /* ******************************* */
@@ -593,6 +601,9 @@ void updateTrafficMatrix(HostTraffic *srcHost,
 			 HostTraffic *dstHost,
 			 TrafficCounter length, 
 			 int actualDeviceId) {
+
+  if(myGlobals.device[actualDeviceId].numHosts == 0) return;
+  
   if(isMatrixHost(srcHost, actualDeviceId) 
      && isMatrixHost(dstHost, actualDeviceId)) {
     unsigned int a, b, id;    
@@ -621,61 +632,60 @@ void updateTrafficMatrix(HostTraffic *srcHost,
 void updateFcTrafficMatrix(HostTraffic *srcHost,
 			 HostTraffic *dstHost,
 			 TrafficCounter length, 
-			 int actualDeviceId)
-{
-    unsigned int a, b, id, i, j, found = 0;    
+			   int actualDeviceId) {
+  unsigned int a, b, id;
 
-    a = matrixHostHash (srcHost, actualDeviceId, 0);
-    b = matrixHostHash (dstHost, actualDeviceId, 0);
+  a = matrixHostHash (srcHost, actualDeviceId, 0);
+  b = matrixHostHash (dstHost, actualDeviceId, 0);
 
+  if ((myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != NULL) &&
+      (myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != srcHost)) {
+    myGlobals.fcMatrixHashCollisions++;
+    a = matrixHostHash (srcHost, actualDeviceId, 1);
     if ((myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != NULL) &&
-        (myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != srcHost)) {
-        myGlobals.fcMatrixHashCollisions++;
-        a = matrixHostHash (srcHost, actualDeviceId, 1);
-        if ((myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != NULL) &&
-            (myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != srcHost)) {
-            traceEvent (CONST_TRACE_WARNING, "Unable to resolve conflict in matrix host hash for %s with %s\n",
-                        myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a]->hostNumFcAddress,
-                        srcHost->hostNumFcAddress);
-            myGlobals.fcMatrixHashUnresCollisions++;
-            return;
-        }
+	(myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] != srcHost)) {
+      traceEvent (CONST_TRACE_WARNING, "Unable to resolve conflict in matrix host hash for %s with %s\n",
+		  myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a]->fcCounters->hostNumFcAddress,
+		  srcHost->fcCounters->hostNumFcAddress);
+      myGlobals.fcMatrixHashUnresCollisions++;
+      return;
     }
+  }
 
+  if ((myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != NULL) &&
+      (myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != dstHost)) {
+    myGlobals.fcMatrixHashCollisions++;
+    b = matrixHostHash (dstHost, actualDeviceId, 1);
     if ((myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != NULL) &&
-        (myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != dstHost)) {
-        myGlobals.fcMatrixHashCollisions++;
-        b = matrixHostHash (dstHost, actualDeviceId, 1);
-        if ((myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != NULL) &&
-            (myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != dstHost)) {
-            traceEvent (CONST_TRACE_WARNING, "Unable to resolve conflict in matrix host hash for %s with %s\n",
-                        myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b]->hostNumFcAddress,
-                        dstHost->hostNumFcAddress);
-            myGlobals.fcMatrixHashUnresCollisions++;
-            return;
-        }
+	(myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] != dstHost)) {
+      traceEvent (CONST_TRACE_WARNING, "Unable to resolve conflict in matrix host hash for %s with %s\n",
+		  myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b]->fcCounters->hostNumFcAddress,
+		  dstHost->fcCounters->hostNumFcAddress);
+      myGlobals.fcMatrixHashUnresCollisions++;
+      return;
     }
+  }
     
-    myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] = srcHost, 
+  myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[a] = srcHost, 
     myGlobals.device[actualDeviceId].fcTrafficMatrixHosts[b] = dstHost;
 
-    id = a*myGlobals.device[actualDeviceId].numHosts+b;
-    if (myGlobals.device[actualDeviceId].fcTrafficMatrix[id] == NULL) {
-        myGlobals.device[actualDeviceId].fcTrafficMatrix[id] = (TrafficEntry*)calloc(1, sizeof(TrafficEntry));
-        myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->vsanId = srcHost->vsanId;
-    }
+  id = a*myGlobals.device[actualDeviceId].numHosts+b;
+  if (myGlobals.device[actualDeviceId].fcTrafficMatrix[id] == NULL) {
+    myGlobals.device[actualDeviceId].fcTrafficMatrix[id] = (TrafficEntry*)calloc(1, sizeof(TrafficEntry));
+    myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->vsanId = srcHost->fcCounters->vsanId;
+  }
     
-    incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->bytesSent, length.value);
-    incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->pktsSent, 1);
+  incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->bytesSent, length.value);
+  incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->pktsSent, 1);
 
-    id = b*myGlobals.device[actualDeviceId].numHosts+a;
-    if(myGlobals.device[actualDeviceId].fcTrafficMatrix[id] == NULL) {
-      myGlobals.device[actualDeviceId].fcTrafficMatrix[id] = (TrafficEntry*)calloc(1, sizeof(TrafficEntry));
-      myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->vsanId = dstHost->vsanId;
-    }
+  id = b*myGlobals.device[actualDeviceId].numHosts+a;
+  if(myGlobals.device[actualDeviceId].fcTrafficMatrix[id] == NULL) {
+    myGlobals.device[actualDeviceId].fcTrafficMatrix[id] = (TrafficEntry*)calloc(1, sizeof(TrafficEntry));
+    myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->vsanId = dstHost->fcCounters->vsanId;
+  }
     
-    incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->bytesRcvd, length.value);
-    incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->pktsRcvd, 1);
+  incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->bytesRcvd, length.value);
+  incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->pktsRcvd, 1);
 }
 
 /* ************************ */
